/*******************************************************************************
 * Copyright (c) 2014 TwelveTone LLC and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Steven Spungin <steven@spungin.tv> - initial API and implementation
 *******************************************************************************/
package com.perdemia.permissionanalyzer.application;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.Date;

import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.internal.workbench.E4Workbench;
import org.eclipse.e4.ui.internal.workbench.swt.E4Application;
import org.eclipse.e4.ui.internal.workbench.swt.PartRenderingEngine;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.advanced.MPerspective;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.e4.ui.workbench.lifecycle.PostContextCreate;
import org.eclipse.e4.ui.workbench.lifecycle.PreSave;
import org.eclipse.e4.ui.workbench.lifecycle.ProcessAdditions;
import org.eclipse.e4.ui.workbench.lifecycle.ProcessRemovals;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.export.ExportHelper;
import com.perdemia.permissionanalyzer.application.jface.dialogs.PasswordDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.parts.FilePart;
import com.perdemia.permissionanalyzer.application.parts.ScanResultsPart;
import com.perdemia.permissionanalyzer.application.preferences.AESEncrypter;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.ProgramParameters;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.util.DBUtil;

/**
 * This is a stub implementation containing e4 LifeCycle annotated methods.<br />
 * There is a corresponding entry in <em>plugin.xml</em> (under the
 * <em>org.eclipse.core.runtime.products' extension point</em>) that references
 * this class.
 **/
@SuppressWarnings("restriction")
public class E4LifeCycle {
	private static Logger logger = LoggerFactory.getLogger(E4LifeCycle.class.getName());
	public static boolean isSilentMode;
		
	/**
	 * @see {@link #startScheduledScan(MApplication, EModelService, EPartService)}
	 * @see {@link #startScheduledReports(MApplication, EModelService, EPartService)}
	 */
	private int scheduledTaskCount;
	
	static {
		isSilentMode = ProgramParameters.isScan() || ProgramParameters.isRunReports();
	}
		
	@PostContextCreate
	public void postContextCreate(IEclipseContext workbenchContext, final Display display) {
		PreferenceHelper prefHelper = PreferenceHelper.getInstance();
		
		logger.info("Application version: " + Activator.getContext().getBundle().getVersion().toString());
		logger.info("Application license: " + LicenseHelper.getEditionString());
		logger.info("Application license status: " + LicenseHelper.getLicenseStatusString(LicenseHelper.getLicense()));
		logger.info("Application license activation: " + LicenseHelper.getActivateStatusString(LicenseHelper.getLicense()));
		logger.info("Application parameters: " + ProgramParameters.getParametersString());
		
		long maxBytes = Runtime.getRuntime().maxMemory();
		logger.info("Max memory: " + maxBytes / 1024 / 1024 + "M");
		
		if (prefHelper.getMasterPasswordHash() != null) {
			if (ProgramParameters.getPassword() != null) {
				try {
					if (AESEncrypter.hashPasswordToString(ProgramParameters.getPassword()).equals(PreferenceHelper.getInstance().getMasterPasswordHash())){
						//password hashes match
						PreferenceHelper.getInstance().setMasterPassword(ProgramParameters.getPassword());
					} else {
						logger.error("WRONG PASSWORD given in the command line options. The application will now exit.");
						System.exit(0);
					}
				} catch (NoSuchAlgorithmException | InvalidKeySpecException e) {
					logger.error("Error while checking the password: " + e.getMessage(), e);
					return;
				}
			} else {
				// ask for the password
				if (!isSilentMode) {
					final Shell shell = new Shell(SWT.INHERIT_NONE);
					PasswordDialog dialog = new PasswordDialog(shell);
					dialog.create();
					
					// apply the css styling on the login dialog (this styling has not been loaded yet by the application)
					String cssTheme = "com.perdemia.permissionanalyzer.styletheme.login";
					workbenchContext.set(E4Application.THEME_ID, cssTheme);
					String cssURI = "css/default.css";
					workbenchContext.set(E4Workbench.CSS_URI_ARG, cssURI);
			 
					PartRenderingEngine.initializeStyling(shell.getDisplay(), workbenchContext);
			      
					if (dialog.open() != IDialogConstants.OK_ID) {
						System.exit(0);
					}
				} else {
					logger.error("Application was used in scheduled mode using -scan or -report parameter, but no password was provided while one is required.");
					System.exit(0);
				}
			}
			
		}
		
		// load the preferences now the password has been set (if configured)
		prefHelper.loadPreferences();
		
		// initialize the database connection
		try {
			DBUtil.getDBConnection();
		} catch (DBException ex) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while initializing the database connection: " + ex.getMessage());
		}
				
		// check for database layout changes
		DBUtil.checkDBChanges();
		
		// check if the number of users/groups in the database does not exceed the maximum defined by the license
		Member.isUserGroupCountAllowed();
	}
	 
	@PreSave
	public void preSave(IEclipseContext workbenchContext) {
	}

	@ProcessAdditions
	public void processAdditions(final MApplication application, final EModelService modelService, final EPartService partService) {
		// print edition in the window title?				
		final MWindow window = (MWindow) modelService.find("com.perdemia.permissionanalyzer.application.window", application);
		window.setLabel("Permission Analyzer - " + LicenseHelper.getEditionString());
		
		if (LicenseHelper.getLicense() != null) {
			int daysRemaining = LicenseHelper.getLicense().getLicenseText().getLicenseExpireDaysRemaining(new Date());
			String message = "";
			
			if (daysRemaining > 1 && daysRemaining < 15) {
				message = "Your license will expire in " + daysRemaining + " days. Please see the menu [Help] > [License information] for more information.";
				logger.warn(message);
			} else if (daysRemaining == 1) {
				message = "Your license will expire in 1 day. Please see the menu [Help] > [License information] for more information.";
				logger.warn(message);
			} else if (daysRemaining == 0) {
				message = "Your license has expired. Please see the menu [Help] > [License information] for more information.";
				logger.error(message);
			}
		
			if (message.length() > 0) {
				SyncMessageDialog.showMessage(MessageDialog.WARNING, "License expiration", message);
			}		
		}
		
		LicenseHelper.validateLicenseOnline();
		
		if (ProgramParameters.isScan()) {
			startScheduledScan(application, modelService, partService);
			
		} else if (ProgramParameters.isRunReports()) {
			// if a scan is scheduled, then the report will run after the scan is finished!
			startScheduledReports(application, modelService, partService);
		}
	}

	@ProcessRemovals
	public void processRemovals(IEclipseContext workbenchContext) {
	}
	
	/**
	 * This method is triggered in case the application has been started with the -scan parameter. We have to wait until the application has an active Window otherwise
	 * the <code>partService.findPart(ScanResultsPart.ID)</code> will fail with an exception. Therefore we wait a second before executing the code (<code>timerExec</code>).
	 * If the code still fails it means there is not active Window yet and we trigger the method again. Again, we wait one second and try again.
	 * 
	 * {@link #scheduledTaskCount} is increased every try and if it is more than 60 we stop trying.
	 *  
	 * @param application
	 * @param modelService
	 * @param partService
	 */
	private void startScheduledScan(final MApplication application, final EModelService modelService, final EPartService partService) {	
		Display.getDefault().timerExec(1000, new Runnable() {
			public void run() {
				try {
					// open the scan perspective
					ApplicationUtil.setBusyCursor();
					MPerspective scanPerspective = (MPerspective) modelService.find("com.perdemia.permissionanalyzer.application.perspective.scan", application);
					partService.switchPerspective(scanPerspective);
					ApplicationUtil.setDefaultCursor();
					
					MPart part = partService.findPart(ScanResultsPart.ID);
					ScanResultsPart scanResultsPart = (ScanResultsPart) part.getObject();
					if (scanResultsPart != null) {
						scanResultsPart.startScan();
					}
				} catch (Exception ex) {
					if (scheduledTaskCount > 60) {
						logger.error("Could not get a reference to the ScanResultsPart", ex);
						return;
					} else {
						scheduledTaskCount++;
						startScheduledScan(application, modelService, partService);
					}
				}
			}
		});
	}
	
	/**
	 * This method is triggered in case the application has been started with a report parameter (and no -scan parameter). We have to wait until the application has an active Window otherwise
	 * the <code>partService.findPart(ScanResultsPart.ID)</code> will fail with an exception. Therefore we wait a second before executing the code (<code>timerExec</code>).
	 * If the code still fails it means there is not active Window yet and we trigger the method again. Again, we wait one second and try again.
	 * 
	 * {@link #scheduledTaskCount} is increased every try and if it is more than 60 we stop trying.
	 *  
	 * @param application
	 * @param modelService
	 * @param partService
	 */
	private void startScheduledReports(final MApplication application, final EModelService modelService, final EPartService partService) {
		Display.getDefault().timerExec(1000, new Runnable() {
			public void run() {
				try {
					// get a reference to the IWorkbench (how the hell do you normally get a reference to the workbench at this point??)
					IWorkbench workbench = null;
					MPart part = partService.findPart(FilePart.ID); //<<<ERROR
					FilePart filePart = (FilePart) part.getObject();
					if (filePart != null) {
						workbench = filePart.getWorkbench();
					}
					
					ExportHelper.exportScheduledReports(workbench);
				} catch (Exception ex) {
					if (scheduledTaskCount > 60) {
						logger.error("Could not get a reference to the FilePart", ex);
						return;
					} else {
						scheduledTaskCount++;
						startScheduledReports(application, modelService, partService);
					}
				}
			}
		});
	}
}