package com.perdemia.permissionanalyzer.application.export;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.NoSuchFileException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.mail.EmailException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Display;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.CompareReportsDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.EmailHelper;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.DBObject;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.Report.ReportType;
import com.perdemia.permissionanalyzer.data.ReportComparator1;
import com.perdemia.permissionanalyzer.data.ReportComparator2;
import com.perdemia.permissionanalyzer.data.ReportHistory;
import com.perdemia.permissionanalyzer.data.util.DBUtil;

public class ReportComparator {
	private static Logger logger = LoggerFactory.getLogger(ReportComparator.class.getName());
	
	private final static String HTML_DATA_BLOCK_PREFIX = "$(document).ready(function() {";
	
	private final static String HTML_META_REPORT_TYPE_PREFIX = "<meta name=\"reporttype\" content=\"";
	
	private final static String HTML_TITLE_PREFIX = "<title>";
	
	/** Inidicator to find out if the HTML file contains a normal table or the simplified presentation of a report */
	private final static String HTML_SIMPLE_DATATABLE_PRESENTATION_INDICATOR = "\t\t\t#fileTable th {";
	
	/** Add the new "Difference" <td> to the dynamic beforeRow(....) function */
	private final static String HTML_ACL_REPORT_NESTED_MEMBER_PERMISSION_COLUMN = "'<td class=\"col-inherited-from\">' + this.cell(row[0]._DT_RowIndex, 66).data() + '</td><td></td></tr>'";
	
	/** Additional TD column <td>' + this.cell(row[0]._DT_RowIndex, 68).data() + '</td> */
	private final static String HTML_ACL_REPORT_NESTED_MEMBER_PERMISSION_COLUMN_NEW = "'<td class=\"col-inherited-from\">' + this.cell(row[0]._DT_RowIndex, 66).data() + '</td><td></td><td>' + this.cell(row[0]._DT_RowIndex, 68).data() + '</td></tr>'";
	
	/** Add the new "Difference" <td> to the dynamic beforeRow(....) function */
	private final static String HTML_ACL_REPORT_MEMBER_PERMISSION_COLUMN_SIMPLE_PRESENTATION = "'<td class=\"col-ptext\">' + this.cell(row[0]._DT_RowIndex, 9).data() + '</td></tr>'";
	
	/** Additional TD column <td>' + this.cell(row[0]._DT_RowIndex, 13).data() + '</td> */
	private final static String HTML_ACL_REPORT_MEMBER_PERMISSION_COLUMN_NEW_SIMPLE_PRESENTATION = "'<td class=\"col-ptext\">' + this.cell(row[0]._DT_RowIndex, 9).data() + '</td><td>' + this.cell(row[0]._DT_RowIndex, 13).data() + '</td></tr>'";
	
	/** The div element that contains the report title, which will be replaced */
	private final static String HTML_REPORT_TITLE_DIV = "<div id=\"title\">";
	
	/** The div element that contains the report date, which will be replaced */
	private final static String HTML_REPORT_DATE_DIV = "<div id=\"reportDate\">Report date: ";
	
	/** The div element that contains the count, which we clear in the comparison report */
	private final static String HTML_REPORT_COUNT_DIV = "<div id=\"fileCount\">";
	
	/** The div element that contains the report info, which we clear in the comparison report */
	private final static String HTML_REPORT_INFO_DIV = "<div id=\"reportInfo\">";
	
	/** A character that is present as first character in a CSV file, see ExportHelper */
	private static final char UTF_8_WITH_BOM_MARKER = '\ufeff';
	
	/**
	 * Compare last two report runs.
	 *
	 * @param report the report to compare
	 * @param includeChangedItems can be given by command-line parameter
	 * @param includeAddedItems can be given by command-line parameter
	 * @param includeDeletedItems can be given by command-line parameter
	 * @param sendEmail can be given by command-line parameter
	 * @param useCompareDialog if true the method will display the CompareDialog instead of directly executing the compare method
	 */
	public static void compareLastTwoReportRuns(Report report, boolean includeChangedItemsOldValue, boolean includeChangedItemsNewValue, boolean includeAddedItems, boolean includeDeletedItems, boolean sendEmail, boolean useCompareDialog, IProgressMonitor monitor) {
		try {
			// find the last two runs of this report
			List<ReportHistory> reports = ReportHistory.where("REPORT_ID = ? AND WAS_COMPARISON = FALSE", new Object[]{report.getId()}, "RUN_DATE DESC", 2);
			
			if (reports == null || reports.size() != 2) {
				MessageDialog.openError(null, "Error", "Could not find the last two runs of this report in the report history. View the report history for more details.");
				return;
			}
			
			compareTwoReportRuns(reports.get(1), reports.get(0), report, includeChangedItemsOldValue, includeChangedItemsNewValue, includeAddedItems, includeDeletedItems, sendEmail, useCompareDialog, monitor);
			
		} catch (DBException ex) {
			logger.error("Error while searching the report history in the database", ex);
			MessageDialog.openError(null, "Error", "Error while searching the report history in the database: " + ex.getMessage());
		}
	}
	
	/**
	 * Compare two report runs.
	 *
	 * @param report1 the report 1
	 * @param report2 the report 2
	 * @param report the report to compare
	 * @param includeChangedItems can be given by command-line parameter
	 * @param includeAddedItems can be given by command-line parameter
	 * @param includeDeletedItems can be given by command-line parameter
	 * @param sendEmail can be given by command-line parameter
	 * @param useCompareDialog if true the method will display the CompareDialog instead of directly executing the compare method
	 */
	public static void compareTwoReportRuns(ReportHistory report1, ReportHistory report2, Report report, boolean includeChangedItemsOldValue, boolean includeChangedItemsNewValue, boolean includeAddedItems, boolean includeDeletedItems, boolean sendEmail, boolean useCompareDialog, IProgressMonitor monitor) {
		// determine a default output path
		String report2Path = report2.getFilePath();
		String outputPath = "";
		DateFormat dateformat = new SimpleDateFormat("yyyy-MM-dd HHmmss");
		if (report2Path.toLowerCase().endsWith(".html")) {
			outputPath = report2Path.substring(0, report2Path.length() - 5) + "_comparison_" + dateformat.format(new Date()) + ".html";
		} else if (report2Path.toLowerCase().endsWith(".csv")) {
			outputPath = report2Path.substring(0, report2Path.length() - 4) + "_comparison_" + dateformat.format(new Date()) + ".csv";
		} else {
			// should not happen, unknown or empty report path??
			outputPath = report2Path + "_comparison_" + dateformat.format(new Date()) + ".html";
		}
		
		if (useCompareDialog) {
			CompareReportsDialog compareDialog = new CompareReportsDialog(null, report1.getFilePath(), report2Path, outputPath, report2.getEmailAddress(), report);
			compareDialog.open();
		} else {
			startReportComparison(report1.getFilePath(), report2Path, outputPath, report, includeChangedItemsOldValue, includeChangedItemsNewValue, includeAddedItems, includeDeletedItems, sendEmail ? report.getEmailAddress() : null, monitor);
		}		
	}
	
	public static void startReportComparison(final String path1, final String path2, final String outputFile, final Report report, final boolean includeChangedItemsOldValue, final boolean includeChangedItemsNewValue, final boolean includeAddedItems, final boolean includeDeletedItems, final String emailAddress, IProgressMonitor monitor) {
		// already in a export task with a progress monitor?
		if (monitor != null) {
			executeReportComparison(path1, path2, outputFile, report, includeChangedItemsOldValue, includeChangedItemsNewValue, includeAddedItems, includeDeletedItems, emailAddress, monitor);
		} else {
			// start with a progress monitor
			final ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getDefault().getActiveShell());
			try {
				progressDialog.run(true, true, new IRunnableWithProgress() {
					@Override
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						executeReportComparison(path1, path2, outputFile, report, includeChangedItemsOldValue, includeChangedItemsNewValue, includeAddedItems, includeDeletedItems, emailAddress, monitor);
					}
				});
			} catch (InvocationTargetException | InterruptedException e) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while comparing report: " + e.getMessage());
				logger.error("Error while comparing report", e);
			}
		}
	}
	
	/**
	 * Compares the report files by inserting the data blocks into two database tables. Via joins (inner, left outer and right outer) it determines the report items that have
	 * been changed, added and removed. The method writes the results to a new output file.
	 * 
	 * @param path1
	 * @param path2
	 * @param outputFile
	 * @param includeChangedItems
	 * @param includeAddedItems
	 * @param includeDeletedItems
	 * @param emailAddress
	 */
	public static void executeReportComparison(final String path1, final String path2, final String outputFile, final Report report, final boolean includeChangedItemsOldValue, final boolean includeChangedItemsNewValue, final boolean includeAddedItems, final boolean includeDeletedItems, final String emailAddress, IProgressMonitor monitor) {
		final PreferenceHelper prefs = PreferenceHelper.getInstance();

		BufferedReader isReader1 = null;
		BufferedReader isReader2 = null;
		BufferedWriter writer = null;
		ResultSet rs = null;
		int resultCount = 0;
		
		try {
			Path file1 = Paths.get(path1);
			Path file2 = Paths.get(path2);
			isReader1 = Files.newBufferedReader(file1, Charset.forName("UTF-8"));
			isReader2 = Files.newBufferedReader(file2, Charset.forName("UTF-8"));
			writer = Files.newBufferedWriter(Paths.get(outputFile), StandardCharsets.UTF_8, StandardOpenOption.WRITE, StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING);
			
			ReportType report1Type = null;
			ReportType report2Type = null;
			boolean report1IsSimpleTable = false;
			boolean report2IsSimpleTable = false;
			boolean isHtmlReportType = path1.toLowerCase().endsWith(".html");
			
			// in case of CSV report type, the separator that is found in the CSV file
			String csvSeparator = null;
			
			// ---------------------------------------------
			// HTML REPORT
			// ---------------------------------------------
			if (isHtmlReportType) {
				// find the JSON data block for both readers. The first reader will copy its content to the output file
				// we first loop through both the readers by reading an entire line, when we find "$(document).ready(function() {" we know we have found the start of the JSON block
				// after that, we loop again, but now per character because the JSON block is a large data block on one line
				
				// FIRST LOOP (check per line)
				monitor.setTaskName("Analyzing first file");
				boolean dataBlockFound = false;
				String nextLine;
				while (!dataBlockFound && (nextLine = isReader1.readLine()) != null) {
					if (nextLine.trim().equals(HTML_DATA_BLOCK_PREFIX)) {
						dataBlockFound = true;
					}
					
					// determine the report type, if not already done. First on the meta data, if that not exists check the title of the HTML document
					if (nextLine.trim().toLowerCase().startsWith(HTML_META_REPORT_TYPE_PREFIX)) {
						if (report1Type == null) {
							report1Type = determineReportTypeBasedOnMetaData(nextLine.trim().substring(HTML_META_REPORT_TYPE_PREFIX.length(), nextLine.trim().length() - 2));
						}
						
						nextLine = nextLine.replace(HTML_META_REPORT_TYPE_PREFIX, HTML_META_REPORT_TYPE_PREFIX + "Comparison ");
					}
					
					if (nextLine.trim().toLowerCase().startsWith(HTML_TITLE_PREFIX)) {
						if (report1Type == null) {
							report1Type = determineReportTypeBasedOnMetaData(nextLine.trim().substring(HTML_TITLE_PREFIX.length(), nextLine.trim().length() - HTML_TITLE_PREFIX.length() - 1));
						}
						
						// replace the title with a compare prefix
						nextLine = nextLine.replace(HTML_TITLE_PREFIX, HTML_TITLE_PREFIX + "Comparison ");
					}
					
					// check if we're having a simple datatable in this HTML report
					// if so, the JSON array will have a different (smaller) format
					if (nextLine.equals(HTML_SIMPLE_DATATABLE_PRESENTATION_INDICATOR)) {
						report1IsSimpleTable = true;
					}
										
					// copy the report header content to the output file
					writer.write(nextLine);
					writer.newLine();
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				
				if (report1Type == null) {
					report1Type = report.getReportType();
					logger.warn("The first report file was not recognized as a Permission Analyzer HTML report: " + path1);
				}
				
				if (!dataBlockFound) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "No data block found in the first report file " + path1);
					logger.error("No data block found in report file " + path1);
					return;
				}
				
				// Find data block in second file
				monitor.setTaskName("Analyzing second file");
				dataBlockFound = false;
				while (!dataBlockFound && (nextLine = isReader2.readLine()) != null) {
					if (nextLine.trim().equals(HTML_DATA_BLOCK_PREFIX)) {
						dataBlockFound = true;
					}
					
					// determine the report type, if not already done. First on the meta data, if that not exists check the title of the HTML document
					if (report2Type == null && nextLine.trim().toLowerCase().startsWith(HTML_META_REPORT_TYPE_PREFIX)) {
						report2Type = determineReportTypeBasedOnMetaData(nextLine.trim().substring(HTML_META_REPORT_TYPE_PREFIX.length(), nextLine.trim().length() - 2));
					}
					
					if (report2Type == null && nextLine.trim().toLowerCase().startsWith(HTML_TITLE_PREFIX)) {
						report2Type = determineReportTypeBasedOnMetaData(nextLine.trim().substring(HTML_TITLE_PREFIX.length(), nextLine.trim().length() - HTML_TITLE_PREFIX.length() - 1));
					}
					
					// check if we're having a simple datatable in this HTML report
					// if so, the JSON array will have a different (smaller) format
					if (nextLine.equals(HTML_SIMPLE_DATATABLE_PRESENTATION_INDICATOR)) {
						report2IsSimpleTable = true;
					}
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				
				if (!dataBlockFound) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "No data block found in report file " + path2);
					logger.error("No data block found in report file " + path2);
					return;
				}
				
				if (report2Type == null) {
					report2Type = report.getReportType();
					logger.warn("The second report file was not recognized as a Permission Analyzer HTML report: " + path2);
				}
				
				if (!report1Type.equals(report2Type)) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Both HTML files contain a different report type and cannot be compared.\r\n\r\nFirst report file: " + Report.reportTypeToString(report1Type)  + "\r\nSecond report file: " + Report.reportTypeToString(report2Type));
					logger.error("Both HTML files contain a different report type and cannot be compared.\r\n\r\nFirst report file: " + Report.reportTypeToString(report1Type)  + "\r\nSecond report file: " + Report.reportTypeToString(report2Type));
					return;
				}
				
				if (report1IsSimpleTable != report2IsSimpleTable) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Both HTML files contain a different report presentation and cannot be compared. One report has a simple table layout presentation and the other has the default extended table.");
					logger.error("Both HTML files contain a different report presentation and cannot be compared. One report has a simple table layout presentation and the other has the default extended table.");
					return;
				}
				
				// SECOND LOOP (check per character)
				// We have found the JSON data block in both files
				// Now look for the first JSON object (the text "[[")
				boolean jsonArrayFound = false;
				int nextCharacter;
				while (!jsonArrayFound && (nextCharacter = isReader1.read()) != -1) {
					if ((char) nextCharacter == '[') {
						jsonArrayFound = true;
					}
					
					// write each character to the output file
					writer.write(nextCharacter);
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				
				if (!jsonArrayFound) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "No JSON array found in report file " + path1);
					logger.error("No JSON array found in report file " + path1);
					return;
				}
				
				// Find the JSON array in second file
				jsonArrayFound = false;
				while (!jsonArrayFound && (nextCharacter = isReader2.read()) != -1) {
					if ((char) nextCharacter == '[') {
						jsonArrayFound = true;
					}
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				
				if (!jsonArrayFound) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "No JSON array found in report file " + path2);
					logger.error("No JSON array found in report file " + path2);
					return;
				}
				
				//----------------------------------------------------------------------
				// add all the JSON arrays from report file 1 to a temp database table
				//----------------------------------------------------------------------
				monitor.setTaskName("Preparing database to compare the files");
				DBUtil.truncateTable(ReportComparator1.class);
				DBUtil.truncateTable(ReportComparator2.class);
				
				DBObject.setAutoCommit(false);
				
				// determine the keys that make each of the rows in the HTML report unique
				List<Integer> rowKeyIndexes = new ArrayList<Integer>();
				switch (report1Type) {
					case EFFECTIVE: rowKeyIndexes.add(1); break; // file path
					case FOLDERS_AND_USERS_EFFECTIVE: rowKeyIndexes.add(1); rowKeyIndexes.add(6); break; // file path, username
					case ACL: rowKeyIndexes.add(1); rowKeyIndexes.add(5); rowKeyIndexes.add(8); break; //filePath, explicit/inherited, memberName
					case ACL_DIRECT_GROUP_MEMBERS:
					case ACL_NESTED_GROUP_MEMBERS: 
						if (report1IsSimpleTable) {
							rowKeyIndexes.add(1); rowKeyIndexes.add(8); rowKeyIndexes.add(11); //filePath, memberName, groupMemberName
						} else {
							rowKeyIndexes.add(1); rowKeyIndexes.add(8); rowKeyIndexes.add(37); //filePath, memberName, groupMemberName
						}
						break;
					case ALL_MEMBERS: 
						if (report1IsSimpleTable) {
							rowKeyIndexes.add(1); rowKeyIndexes.add(4); rowKeyIndexes.add(5); rowKeyIndexes.add(8); //memberName, filePath, explicit, ACE flags (This folder for example)
						} else {
							rowKeyIndexes.add(1); rowKeyIndexes.add(4); rowKeyIndexes.add(6); rowKeyIndexes.add(35); //memberName, filePath, explicit, ACE flags (This folder for example)
						}
						break;
					case MATCHING_GROUPS_AND_DIRECT_MEMBERS:
					case MATCHING_GROUPS_AND_NESTED_MEMBERS:
					case GROUPS_WITH_PERMISSIONS_AND_DIRECT_MEMBERS:
					case GROUPS_WITH_PERMISSIONS_AND_NESTED_MEMBERS:
						rowKeyIndexes.add(0); rowKeyIndexes.add(4); break; //groupName, memberName
					case MATCHING_USERS_AND_DIRECT_GROUP_MEMBERSHIPS:
						rowKeyIndexes.add(0); rowKeyIndexes.add(3); break; //groupName, memberName
					case MATCHING_USERS_AND_NESTED_GROUP_MEMBERSHIPS:
						rowKeyIndexes.add(0); rowKeyIndexes.add(3); rowKeyIndexes.add(5); break; //groupName, memberName, viaGroup
				}
				
				monitor.beginTask("Inserting file 1 into temporary database table (step 1 of the 5)", (int) Files.size(file1));
				String nextJsonArray = findNextJsonArray(isReader1);
				while (nextJsonArray != null) {
					ReportComparator1 reportComparator1 = new ReportComparator1();
					reportComparator1.setReportRowKey(findJsonArrayItem(nextJsonArray, rowKeyIndexes));
					reportComparator1.setReportRowValue(nextJsonArray);
					reportComparator1.save();
					
					monitor.worked(nextJsonArray.getBytes().length);
					
					nextJsonArray = findNextJsonArray(isReader1);
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				
				monitor.beginTask("Inserting file 2 into temporary database table (step 2 of the 5)", (int) Files.size(file2));
				nextJsonArray = findNextJsonArray(isReader2);
				while (nextJsonArray != null) {
					ReportComparator2 reportComparator2 = new ReportComparator2();
					reportComparator2.setReportRowKey(findJsonArrayItem(nextJsonArray, rowKeyIndexes));
					reportComparator2.setReportRowValue(nextJsonArray);
					reportComparator2.save();
					
					monitor.worked(nextJsonArray.getBytes().length);
					
					nextJsonArray = findNextJsonArray(isReader2);
					
					if (monitor.isCanceled()) {
						return;
					}
				}
			} else {
				// --------------------------------------
				// CSV REPORT
				// --------------------------------------

				monitor.setTaskName("Analyzing first file");
				String firstLine = isReader1.readLine();
				
				csvSeparator = firstLine.contains(",") ? "," : firstLine.contains(";") ? ";" : prefs.getCsvSeparator();
				
				report1Type = determineReportTypeBasedOnCsvHeader(firstLine, csvSeparator);
				
				// copy the report CSV header content to the output file
				writer.write(firstLine + csvSeparator + "Difference");
				writer.newLine();
				
				if (report1Type == null) {
					report1Type = report.getReportType();
					logger.warn("The first report file was not recognized as a Permission Analyzer CSV report: " + path1);
				}
				
				monitor.setTaskName("Analyzing second file");
				firstLine = isReader2.readLine();
				report2Type = determineReportTypeBasedOnCsvHeader(firstLine, csvSeparator);
				
				if (report2Type == null) {
					report2Type = report.getReportType();
					logger.warn("The second report file was not recognized as a Permission Analyzer CSV report: " + path2);
				}
				
				if (!report1Type.equals(report2Type)) {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Both HTML files contain a different report type and cannot be compared.\r\n\r\nFirst report file: " + Report.reportTypeToString(report1Type)  + "\r\nSecond report file: " + Report.reportTypeToString(report2Type));
					logger.error("Both HTML files contain a different report type and cannot be compared.\r\n\r\nFirst report file: " + Report.reportTypeToString(report1Type)  + "\r\nSecond report file: " + Report.reportTypeToString(report2Type));
					return;
				}
				
				//----------------------------------------------------------------------
				// add all the CSV lines from report file 1 and file 2 to a temp database table
				//----------------------------------------------------------------------
				monitor.setTaskName("Preparing database to compare the files");
				DBUtil.truncateTable(ReportComparator1.class);
				DBUtil.truncateTable(ReportComparator2.class);
				
				DBObject.setAutoCommit(false);
				
				// determine the keys that make each of the rows in the HTML report unique
				List<Integer> rowKeyIndexes = new ArrayList<Integer>();
				switch (report1Type) {
					case EFFECTIVE: rowKeyIndexes.add(1); break; // file path
					case FOLDERS_AND_USERS_EFFECTIVE: rowKeyIndexes.add(1); rowKeyIndexes.add(6); break; // file path, username
					case ACL: rowKeyIndexes.add(1); rowKeyIndexes.add(5); rowKeyIndexes.add(8); break; //filePath, explicit/inherited, memberName
					case ACL_DIRECT_GROUP_MEMBERS:
					case ACL_NESTED_GROUP_MEMBERS: 
						rowKeyIndexes.add(1); rowKeyIndexes.add(5); rowKeyIndexes.add(8); rowKeyIndexes.add(25); //filePath, explicit/inherited, memberName, groupMemberName
						break;
					case ALL_MEMBERS: 
						rowKeyIndexes.add(0); rowKeyIndexes.add(2); rowKeyIndexes.add(6); rowKeyIndexes.add(22); //memberName, filePath, explicit, ACE flags (This folder for example)
						break;
					case MATCHING_GROUPS_AND_DIRECT_MEMBERS:
					case MATCHING_GROUPS_AND_NESTED_MEMBERS:
					case GROUPS_WITH_PERMISSIONS_AND_DIRECT_MEMBERS:
					case GROUPS_WITH_PERMISSIONS_AND_NESTED_MEMBERS:
						rowKeyIndexes.add(0); rowKeyIndexes.add(1); break; //groupName, memberName
					case MATCHING_USERS_AND_DIRECT_GROUP_MEMBERSHIPS:
						rowKeyIndexes.add(0); rowKeyIndexes.add(1); break; //groupName, memberName
					case MATCHING_USERS_AND_NESTED_GROUP_MEMBERSHIPS:
						rowKeyIndexes.add(0); rowKeyIndexes.add(1); rowKeyIndexes.add(2); break; //groupName, memberName, viaGroup
				}
				
				monitor.beginTask("Inserting file 1 into temporary database table (step 1 of the 5)", (int) Files.size(file1));
				String nextCsvLine;
				while ((nextCsvLine = isReader1.readLine()) != null) {							
					ReportComparator1 reportComparator1 = new ReportComparator1();
					reportComparator1.setReportRowKey(findCsvItem(nextCsvLine, rowKeyIndexes, csvSeparator));
					reportComparator1.setReportRowValue(nextCsvLine);
					reportComparator1.save();
					
					monitor.worked(nextCsvLine.getBytes().length);
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				
				monitor.beginTask("Inserting file 2 into temporary database table (step 2 of the 5)", (int) Files.size(file2));
				while ((nextCsvLine = isReader2.readLine()) != null) {
					ReportComparator2 reportComparator2 = new ReportComparator2();
					reportComparator2.setReportRowKey(findCsvItem(nextCsvLine, rowKeyIndexes, csvSeparator));
					reportComparator2.setReportRowValue(nextCsvLine);
					reportComparator2.save();
					
					monitor.worked(nextCsvLine.getBytes().length);
													
					if (monitor.isCanceled()) {
						return;
					}
				}
			}
			
			//-- Changed folders/files
			//SELECT * FROM "PERMISSIONANALYZER"."REPORTCOMPARATOR1" R1 INNER JOIN "PERMISSIONANALYZER"."REPORTCOMPARATOR2" R2 ON R1.REPORT1_ROW_KEY = R2.REPORT2_ROW_KEY WHERE R1.REPORT1_ROW_VALUE != R2.REPORT2_ROW_VALUE
			
			//-- New folders/files
			//SELECT * FROM "PERMISSIONANALYZER"."REPORTCOMPARATOR1" R1 RIGHT OUTER JOIN "PERMISSIONANALYZER"."REPORTCOMPARATOR2" R2 ON R1.REPORT1_ROW_KEY = R2.REPORT2_ROW_KEY WHERE R1.ID IS NULL

			//-- Removed folders
			//SELECT * FROM "PERMISSIONANALYZER"."REPORTCOMPARATOR1" R1 LEFT OUTER JOIN "PERMISSIONANALYZER"."REPORTCOMPARATOR2" R2 ON R1.REPORT1_ROW_KEY = R2.REPORT2_ROW_KEY WHERE R2.ID IS NULL
			
			// Folder/files that have been changed
			if (includeChangedItemsOldValue || includeChangedItemsNewValue) {
				monitor.beginTask("Determining changed report data (step 3 of 5)", IProgressMonitor.UNKNOWN);
				rs = DBObject.executeQuery("SELECT * FROM PERMISSIONANALYZER.REPORTCOMPARATOR1 R1 INNER JOIN PERMISSIONANALYZER.REPORTCOMPARATOR2 R2 ON R1.REPORT1_ROW_KEY = R2.REPORT2_ROW_KEY WHERE R1.REPORT1_ROW_VALUE != R2.REPORT2_ROW_VALUE");
				
				while(rs.next()) {
					String report1RowValue = rs.getString("REPORT1_ROW_VALUE");
					String report2RowValue = rs.getString("REPORT2_ROW_VALUE");
					
					if (includeChangedItemsOldValue) {
						if (isHtmlReportType) {
							// add to the JSON array
							report1RowValue = report1RowValue.substring(0, report1RowValue.length() - 1) + ", \"Changed (old)\"],";
						} else {
							// add to the CSV line
							report1RowValue += csvSeparator + "Changed (old)\r\n";
						}
						writer.write(report1RowValue);
					}
					
					if (includeChangedItemsNewValue) {
						if (isHtmlReportType) {
							// add to the JSON array
							report2RowValue = report2RowValue.substring(0, report2RowValue.length() - 1) + ", \"Changed (new)\"],";
						} else {
							// add to the CSV line
							report2RowValue += csvSeparator + "Changed (new)\r\n";
						}
						
						writer.write(report2RowValue);
					}							
					
					resultCount++;
					
					if (monitor.isCanceled()) {
						DBObject.closeResultSet(rs);
						return;
					}
				}
				DBObject.closeResultSet(rs);
			}
			
			// Folder/files that have been added
			if (includeAddedItems) {
				monitor.beginTask("Determining added report data (step 4 of 5)", IProgressMonitor.UNKNOWN);
				rs = DBObject.executeQuery("SELECT * FROM PERMISSIONANALYZER.REPORTCOMPARATOR1 R1 RIGHT OUTER JOIN PERMISSIONANALYZER.REPORTCOMPARATOR2 R2 ON R1.REPORT1_ROW_KEY = R2.REPORT2_ROW_KEY WHERE R1.ID IS NULL");
				
				while(rs.next()) {
					String reportRowValue = rs.getString("REPORT2_ROW_VALUE");
					
					if (isHtmlReportType) {
						// add to the JSON array
						reportRowValue = reportRowValue.substring(0, reportRowValue.length() - 1) + ", \"Added\"],";
					} else {
						// add to the CSV line
						reportRowValue += csvSeparator + "Added\r\n";
					}
					
					writer.write(reportRowValue);
					
					resultCount++;
					
					if (monitor.isCanceled()) {
						DBObject.closeResultSet(rs);
						return;
					}
				}
				DBObject.closeResultSet(rs);
			}
						
			// Folder/files that have been removed
			if (includeDeletedItems) {
				monitor.beginTask("Determining removed report data (step 5 of 5)", IProgressMonitor.UNKNOWN);
				rs = DBObject.executeQuery("SELECT * FROM PERMISSIONANALYZER.REPORTCOMPARATOR1 R1 LEFT OUTER JOIN PERMISSIONANALYZER.REPORTCOMPARATOR2 R2 ON R1.REPORT1_ROW_KEY = R2.REPORT2_ROW_KEY WHERE R2.ID IS NULL");
				
				while(rs.next()) {
					String reportRowValue = rs.getString("REPORT1_ROW_VALUE");
					
					if (isHtmlReportType) {
						// add to the JSON array
						reportRowValue = reportRowValue.substring(0, reportRowValue.length() - 1) + ", \"Deleted\"],";
					} else {
						// add to the CSV line
						reportRowValue += csvSeparator + "Deleted\r\n";
					}							
					
					writer.write(reportRowValue);
					
					resultCount++;
					
					if (monitor.isCanceled()) {
						return;
					}
				}
				DBObject.closeResultSet(rs);
			}
			
			// finish the HTML report
			if (isHtmlReportType) {
				// write the closing bracket for the data block variable (json array of arrays)
				writer.write("]");
				
				// write the rest of the original report (like the data table HTML) to the output file
				
				String nextLine;
				boolean reportInfoDivFound = false;
				
				while ((nextLine = isReader1.readLine()) != null) {
					if (nextLine.trim().equals("</tr>")) {
						nextLine = "<th>Difference</th></tr>";
					}
					
					// add the new "Difference" <td> to the dynamic beforeRow(....) function (for the NORMAL PRESENTATION report)
					if (!report1IsSimpleTable && (report1Type.equals(ReportType.ACL_DIRECT_GROUP_MEMBERS) || report1Type.equals(ReportType.ACL_NESTED_GROUP_MEMBERS))) {
						nextLine = nextLine.replace(HTML_ACL_REPORT_NESTED_MEMBER_PERMISSION_COLUMN, HTML_ACL_REPORT_NESTED_MEMBER_PERMISSION_COLUMN_NEW);
					}
					
					// add the new "Difference" <td> to the dynamic beforeRow(....) function (for the SIMPLE PRESENTATION report)
					if (report1IsSimpleTable && (report1Type.equals(ReportType.ACL_DIRECT_GROUP_MEMBERS) || report1Type.equals(ReportType.ACL_NESTED_GROUP_MEMBERS))) {
						nextLine = nextLine.replace(HTML_ACL_REPORT_MEMBER_PERMISSION_COLUMN_SIMPLE_PRESENTATION, HTML_ACL_REPORT_MEMBER_PERMISSION_COLUMN_NEW_SIMPLE_PRESENTATION);
					}
					
					// Replace the title div
					if (nextLine.contains(HTML_REPORT_TITLE_DIV)) {
						nextLine = nextLine.replace(HTML_REPORT_TITLE_DIV, HTML_REPORT_TITLE_DIV + "Comparison: ");
					}
					
					// Replace the report date
					if (nextLine.contains(HTML_REPORT_DATE_DIV)) {
						SimpleDateFormat dateFormatter = new SimpleDateFormat("dd MMMMM yyyy HH:mm"); 
						nextLine = HTML_REPORT_DATE_DIV + dateFormatter.format(new Date()) + "</div>";
					}
					
					// Replace the file count
					if (nextLine.contains(HTML_REPORT_COUNT_DIV)) {
						// we don't know the count as this point, clear it for now
						nextLine = HTML_REPORT_COUNT_DIV + "Number of changes: " + resultCount + "</div>";
					}
					
					if (reportInfoDivFound) {
						// we are now in the line AFTER the report info div, meaning the <h2> heading
						nextLine = nextLine.replace("</h2>", "</h2><br><h3>Comparison report</h3><ul><li><strong>Report 1</strong>: " + file1.getFileName().toString() + "<li><strong>Report 2</strong>: " + file2.getFileName().toString() + "</ul><br>");
						reportInfoDivFound = false;
					}
					
					// Add both the reports to the report description
					if (nextLine.contains(HTML_REPORT_INFO_DIV)) {
						reportInfoDivFound = true;
					}
					
					// copy the report header content to the output file
					writer.write(nextLine);
					writer.newLine();
					
					if (monitor.isCanceled()) {
						return;
					}
				}
			}
		
			
			// send e-mail?
			if (emailAddress != null && !emailAddress.isEmpty()) {
				String msg;
				String subject;
				if (report != null && report.isPolicy()) {
					msg = prefs.getPolicyEmailTemplate();
					//subject = prefs.getPolicyEmailSubject();
				} else {
					msg = prefs.getReportEmailTemplate();
					//subject = prefs.getReportEmailSubject();
				}
				
				monitor.beginTask("Sending e-mail", IProgressMonitor.UNKNOWN);
				
				Report reportForEmail = new Report();
				if (report != null) {
					reportForEmail.setName(report.getName());
					reportForEmail.setDescription(report.getDescription());
				} else {
					reportForEmail.setName(file2.getFileName().toString());
					reportForEmail.setDescription("Comparison of " + file1.getFileName().toString() + " and " + file2.getFileName().toString());
				}
				
				reportForEmail.setLastFilePath(outputFile);
				reportForEmail.setEmailAddress(emailAddress);
				reportForEmail.setEmailThreshold(resultCount);
				
				subject = "Comparison of " + file1.getFileName().toString() + " and " + file2.getFileName().toString();
				EmailHelper.sendMail(subject, msg, reportForEmail);
			}
			
			// keep a history record
			if (report != null) {
				ReportHistory reportHistory = new ReportHistory(report, "OK", outputFile, new Date(), resultCount, true);
				reportHistory.save();
			}
			
			String confirmationMessage;
			if (emailAddress != null && !emailAddress.isEmpty()) {
				confirmationMessage = "The comparison has been built and an e-mail has been sent. Do you want to open the report?";
			} else {
				confirmationMessage = "The comparison has been built. Do you want to open the report?";
			}
			
			if (SyncMessageDialog.showMessage(MessageDialog.QUESTION, "Comparison ready", confirmationMessage, false)) {
				Program.launch(outputFile);								
			}						
			
			monitor.beginTask("Cleaning up database tables", IProgressMonitor.UNKNOWN);
			//DBUtil.truncateTable(ReportComparator1.class);
			//DBUtil.truncateTable(ReportComparator2.class);
			
			monitor.done();
	
		} catch (Exception e) {
			if (e instanceof EmailException) {
				logger.error("Error while e-mailing the report: " + e.getMessage(), e);
				SyncMessageDialog.showMessage(MessageDialog.WARNING, "Warning", "The comparison report has been created, but the e-mail could not be sent: " + e.getMessage());
			} else if (e instanceof SQLException) {
				SQLException sqlEx = (SQLException) e;
				logger.error("Error while comparing both report files: ", sqlEx);
				
				if (sqlEx.getNextException() != null) {
					sqlEx = sqlEx.getNextException();
					logger.error("Error (nested exception) while comparing both report files: " + sqlEx.getMessage(), sqlEx);
				}
				
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while comparing both report files: " + GUILogHandler.getNestedMessage(sqlEx));
			} else if (e instanceof NoSuchFileException) {
				logger.error("Error while comparing both report files. Report file not found: " + e.getMessage(), e);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while comparing both report files. Report file not found: " + e.getMessage());
			} else {
				logger.error("Error while comparing both report files: " + e.getMessage(), e);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while comparing both report files: " + e.getMessage());
			}
			
			// keep a history record
			if (report != null) {
				try {
					String resultMessage;
					if (e instanceof EmailException) {
						resultMessage = "Error while e-mailing the comparison report: " + e.getMessage();
					} else {
						resultMessage = GUILogHandler.getNestedMessage(e);
					}
					
					ReportHistory reportHistory = new ReportHistory(report, resultMessage, outputFile, new Date(), resultCount, true);
					reportHistory.save();
				} catch (DBException e2) {
					// bad luck
					logger.error("Error while updating report status", e2);
				}
			}
						
		} finally {
			DBObject.closeResultSet(rs);
			
			try {
				DBObject.setAutoCommit(true);
			} catch (Exception ex) {
				logger.warn("Error while setting auto commit to true after finishing report compare: " + ex.getMessage());
			}
			
			if (isReader1 != null) {
				try {
					isReader1.close();
				} catch (IOException e) {
					logger.warn("Error while closing reader1 in ReportComparer: " + e.getMessage());
				}
			}
			if (isReader2 != null) {
				try {
					isReader1.close();
				} catch (IOException e) {
					logger.warn("Error while closing reader2 in ReportComparer: " + e.getMessage());
				}
			}
			
			if (writer != null) {
				try {
					writer.close();
				} catch (IOException e) {
					logger.warn("Error while closing writer in ReportComparer: " + e.getMessage());
				}
			}
		}			
	}
	
	
	/**
	 * Takes an existing BufferedReader and finds the next "]" character and returns the String from the current position until the "]" character.
	 * 
	 * @param reader
	 * @return one item in the JSON array of arrays "[[text1, text2, text3][text1, text2, text3][text1, text2, text3]]"
	 * @throws IOException 
	 */
	private static String findNextJsonArray(BufferedReader reader) throws IOException {
		StringBuffer result = new StringBuffer("");
		boolean jsonArrayFound = false;
		boolean characterInDoubleQuote = false;
		int nextCharacter;
		
		while (!jsonArrayFound && (nextCharacter = reader.read()) != -1) {
			// end of entire data block?
			if (((char) nextCharacter == ';') && !characterInDoubleQuote) {
				break;
			}
			
			// end of the json array?
			if ((char) nextCharacter == ']' && !characterInDoubleQuote) {
				if (result.length() > 0) {
					jsonArrayFound = true;
				} else {
					// end of entire data block
					break;
				}
			}
			
			// end of a json value?
			if ((char) nextCharacter == '"') {
				characterInDoubleQuote = !characterInDoubleQuote;
			}
			
			// escape character
			if ((char) nextCharacter == '\\') {
				// means to escape the next character, which doesn't count as a bracket or double quote. Therefore, skip the next character.
				result.append((char)nextCharacter);
				nextCharacter = reader.read();
			}
			
			if (result.length() > 0 || (char)nextCharacter == '[') {
				result.append((char)nextCharacter);
			}
		}
		
		if (jsonArrayFound) {
			return result.toString();
		} else {
			return null;
		}
	}
	
	private static String findJsonArrayItem(String jsonArray, List<Integer> indexes) {
		StringBuffer result = new StringBuffer("");
		int numberOfArrayItemsFound = 0;
		StringBuffer currentArrayValue = new StringBuffer("");
		boolean characterInDoubleQuote = false;
		int currentIndex = 0;
		
		for (int nextCharIndex = 0; nextCharIndex < jsonArray.length(); nextCharIndex++) {
			int nextCharacter = jsonArray.charAt(nextCharIndex);
			int prevCharacter = jsonArray.charAt(nextCharIndex == 0 ? 0 : nextCharIndex - 1);
			
			// end of a json value? (and not an escaped quote)
			if ((char) nextCharacter == '"' && (char) prevCharacter != '\\') {
				if (characterInDoubleQuote) {
					// we reached the end of a value in the array
					if (indexes.contains(currentIndex)) {
						result.append(currentArrayValue.toString());
						numberOfArrayItemsFound++;
						
						if (numberOfArrayItemsFound == indexes.size()) {
							// we have found all the requested indexes
							return result.toString();
						}
					} 
					
					// process the next array key
					currentArrayValue.delete(0, currentArrayValue.length());
					characterInDoubleQuote = false;
					currentIndex++;
					
				} else {
					characterInDoubleQuote = true;
					continue;
				}
			}
			
			if (characterInDoubleQuote) {
				currentArrayValue.append((char) nextCharacter);
			}						
		}

		// should not happen
		logger.warn(MessageFormat.format("Warning, not all requested indexes were found in the jsonArray: {0}, {1}", new Object[]{indexes.toString(), jsonArray}));
		
		return result.toString();
	}
	
	private static String findCsvItem(String csvLine, List<Integer> indexes, String csvSeparator) {
		StringBuffer result = new StringBuffer("");
		int numberOfArrayItemsFound = 0;
		
		String[] splitted = csvLine.split(csvSeparator + "(?=([^\"]*\"[^\"]*\")*[^\"]*$)");
	   
		for (int index = 0; index < splitted.length; index++) {
			if (indexes.contains(index)) {
				result.append(splitted[index]);
				numberOfArrayItemsFound++;
				
				if (numberOfArrayItemsFound == indexes.size()) {
					// we have found all the requested indexes
					return result.toString();
				}
			}					
		}

		// should not happen
		logger.warn(MessageFormat.format("Warning, not all requested indexes were found in the CSV line: {0}, {1}", new Object[]{indexes.toString(), csvLine}));
		
		return result.toString();
	}
	
	private static ReportType determineReportTypeBasedOnMetaData(String metadata) {
		switch (metadata) {
			case "Permission Analyzer effective permissions report": return ReportType.EFFECTIVE;
			case "Permission Analyzer ACL report": return ReportType.ACL;
			case "Permission Analyzer ACL report with the effective permissions of direct group members": return ReportType.ACL_DIRECT_GROUP_MEMBERS;
			case "Permission Analyzer ACL report with the effective permissions of nested group members": return ReportType.ACL_NESTED_GROUP_MEMBERS;
			case "Permission Analyzer user/group permission report": return ReportType.ALL_MEMBERS;
			case "Permission Analyzer group report": return ReportType.MATCHING_GROUPS_AND_DIRECT_MEMBERS;
			case "Permission Analyzer user report": return ReportType.MATCHING_USERS_AND_DIRECT_GROUP_MEMBERSHIPS; //old template, has been changed to the ones below
			case "Permission Analyzer user report with group memberships": return ReportType.MATCHING_USERS_AND_DIRECT_GROUP_MEMBERSHIPS;
			case "Permission Analyzer user report with nested group memberships": return ReportType.MATCHING_USERS_AND_NESTED_GROUP_MEMBERSHIPS;
			case "Permission Analyzer files, folders and users with their effective permissions report": return ReportType.FOLDERS_AND_USERS_EFFECTIVE;
		}
		
		logger.warn("Could not determine HTML report type based on " + metadata);
		
		return null;		
	}
	
	private static ReportType determineReportTypeBasedOnCsvHeader(String csvHeader, String csvSeparator) {
		
		if (csvHeader.charAt(0) == (UTF_8_WITH_BOM_MARKER)) {
			csvHeader = csvHeader.substring(1);
		}
		
		if (csvHeader.equalsIgnoreCase(new String("Is directory,File path,Member count,Owner,Inherit ACL,Permission,Read data,Write data,Append data,Execute,Read attributes,Write attributes,Read extended attributes,Write extended attributes,Delete subfolders and files,Delete,Read permissions,Change permissions,Take ownership").replace(",", csvSeparator))) {
			return ReportType.EFFECTIVE;
		}
		if (csvHeader.equalsIgnoreCase(new String("Is directory,File path,Member count,Owner,Inherit ACL,Is explicit,Is deny,Is group,Member,Permission,Read data,Write data,Append data,Execute,Read attributes,Write attributes,Read extended attributes,Write extended attributes,Delete subfolders and files,Delete,Read permissions,Change permissions,Take ownership,Flags,Inherited from").replace(",", csvSeparator))) {
			return ReportType.ACL;
		}
		if (csvHeader.equalsIgnoreCase(new String("Is directory,File path,Member count,Owner,Inherit ACL,Is explicit,Is deny,Is group,Member,Permission,Read data,Write data,Append data,Execute,Read attributes,Write attributes,Read extended attributes,Write extended attributes,Delete subfolders and files,Delete,Read permissions,Change permissions,Take ownership,Flags,Inherited from, Via group").replace(",", csvSeparator))) {
			return ReportType.ACL_DIRECT_GROUP_MEMBERS;
		}
		if (csvHeader.equalsIgnoreCase(new String("Member,Is directory,File path,ACE count,Is group,ACE member,Is explicit,Is deny,Permission,Read data,Write data,Append data,Execute,Read attributes,Write attributes,Read extended attributes,Write extended attributes,Delete subfolders and files,Delete,Read permissions,Change permissions,Take ownership,Flags,Owner,Inherit ACL").replace(",", csvSeparator))) {
			return ReportType.ALL_MEMBERS;
		}
		if (csvHeader.equalsIgnoreCase(new String("Group,Member").replace(",", csvSeparator))) {
			return ReportType.MATCHING_GROUPS_AND_DIRECT_MEMBERS;
		}
		if (csvHeader.equalsIgnoreCase(new String("User,Group").replace(",", csvSeparator))) {
			return ReportType.MATCHING_USERS_AND_DIRECT_GROUP_MEMBERSHIPS;
		}
		if (csvHeader.equalsIgnoreCase(new String("User,Group,Via group").replace(",", csvSeparator))) {
			return ReportType.MATCHING_USERS_AND_NESTED_GROUP_MEMBERSHIPS;
		}
		if (csvHeader.equalsIgnoreCase(new String("Is directory,File path,User count,Owner,Inherit ACL,Is group,User,Permission,Read data,Write data,Append data,Execute,Read attributes,Write attributes,Read extended attributes,Write extended attributes,Delete subfolders and files,Delete,Read permissions,Change permissions,Take ownership,Via group(s)").replace(",", csvSeparator))) {
			return ReportType.FOLDERS_AND_USERS_EFFECTIVE;
		}
		
		logger.warn("Could not determine CSV report type based on the header " + csvHeader + " with csvSeparator " + csvSeparator);
		
		return null;		
	}
	
	public static void testRun() {
		startReportComparison("C:\\Temp\\sum_folders.html", "C:\\Temp\\sum_folders_new.html", "C:\\Temp\\sum_folders_output.html", null, true, true, true, true, "", null);
	}
	
	public static void main(String[] args) {
		//executeReportComparison("C:\\Temp\\sum_folders.html", "C:\\Temp\\sum_folders_new.html", "C:\\Temp\\sum_folders_output.html", null, true, true, true, "");
		String nextLine = "   <meta name=\"reporttype\" content=\"Permission Analyzer ACL report with the effective permissions of direct group members\">";
		System.out.println(nextLine.trim().substring(HTML_META_REPORT_TYPE_PREFIX.length(), nextLine.trim().length() - 2));
	}
}
