package com.perdemia.permissionanalyzer.application.jface.contentproviders;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILazyTreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.FilterConfig;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.util.FileCache;
import com.perdemia.permissionanalyzer.data.util.MembershipCache;

public class AclPartContentProvider implements ILazyTreeContentProvider {
	private static Logger logger = LoggerFactory.getLogger(AclPartContentProvider.class.getName());
	private List<AclItem> aclItems;
	private TreeViewer viewer;
	private AclPartContentProviderConfig config;
	
	public AclPartContentProvider(TreeViewer viewer) {
		this.viewer = viewer;
	}
	
	/**
	 * Reads the new {@link AclPartContentProviderConfig} and processes the selected {@link FileInfo} object. It builds a List of
	 * {@link AclItem} objects, each {@link aclItem} object represents a row in the table.
	 * 
	 * <p>
	 * The field {@link AclPartContentProviderConfig.selectedFile} contains a List of {@link AclItem} objects which apply
	 * to the selected folder. The field {@link AclPartContentProviderConfig.parentFiles} is used to build a folder tree
	 * to display permission inheritance.
	 * </p>
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		config = (AclPartContentProviderConfig) newInput;
		if (config != null && config.getSelectedFile() != null) {
			
			try { 
				if (config.isApplyFilter()) {
					aclItems = config.getSelectedFile().getAclItems();
				} else {
					// retrieve the full list of ACE's from the database
					aclItems = AclItem.where("fileinfo_id = ?", new Object[]{config.getSelectedFile().getId()}, "member_id");
				}
				
				determineInheritedFromFolderPath(aclItems, config.getSelectedFile());
				
			} catch (DBException ex) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while retrieving the list of ACE's for the selected file: " + ex.getMessage());
			}
			
		} else {
			aclItems = new ArrayList<AclItem>();
		}
	}
	
	@Override
	public void updateElement(Object parent, int index) {
		AclItem aclItemElement = null;
		Member memberElement = null;
		
		if (parent instanceof AclItem || parent instanceof Member) {
			Member group;
			if (parent instanceof AclItem) {
				group = ((AclItem) parent).getMember(true);
			} else {
				group = (Member) parent;
			}
			
			if (!group.getMembers().isEmpty()) {
				memberElement = group.getMembers().get(index);
			} else {
				memberElement = new Member("No matching members found in this group");
			}
		} else {
			aclItemElement = aclItems.get(index);
		}
		
		if (aclItemElement != null) {
			viewer.replace(parent, index, aclItemElement);
			viewer.setHasChildren(aclItemElement, aclItemElement.getMember(true).isGroup());
		} else {
			viewer.replace(parent, index, memberElement);
			viewer.setHasChildren(memberElement, memberElement.isGroup());
		}
	}

	@Override
	public void updateChildCount(Object element, int currentChildCount) {
		ApplicationUtil.setBusyCursor();
		
		Member group = null;
		if (element instanceof AclItem) {
			AclItem aclItem = (AclItem) element;
			group = aclItem.getMember(true);
		} else if (element instanceof Member){
			group = (Member) element;
		}
		
		if (group != null) {
			try {
				FilterConfig filterConfig = FilterHelper.getInstance().getFilterConfig(false);
				if (config.isApplyFilter()) {
					// only select the members that match the filter criteria
					group.setMembers(MembershipCache.getFiltedGroupMembers(group, filterConfig.isIncludeUsers(), filterConfig.isIncludeGroups(), false, false, false, FilterHelper.getInstance()));
				} else {
					// all members of this group
					List<Member> groupMembers = MembershipCache.getGroupMembers(group, filterConfig.isIncludeUsers(), filterConfig.isIncludeGroups(), false);
					group.setMembers(groupMembers);
				}
			} catch (DBException ex) {
				logger.error("Error while getting members for " + group, ex);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting members for " + group + ": " + ex.getMessage());
			}
			
			if (group.getMembers().size() == 0) {
				// although there are no group memers we want to display a child named "No matching members found in this group"
				viewer.setChildCount(element, 1);
			} else {
				viewer.setChildCount(element, group.getMembers().size());
			}
		} else {
			viewer.setChildCount(element, aclItems.size());
		}
		
		ApplicationUtil.setDefaultCursor();
	}
	
	/**
	 * Set the field {@link AclItem#setInheritedFromFolder(String)} using the inheritedFromFolderId of the AclItem.
	 * @param aclItems
	 * @param file
	 */
	public static void determineInheritedFromFolderPath(List<AclItem> aclItems, FileInfo file) {
		
		// DON'T DETERMINE THE FULL PATH BECAUSE IT TAKES A LOT OF SPACE IN THE UI AND THE HTML REPORTS,
		// JUST RETURN THE NAME OF THE FOLDER
		/*
		List<FileInfo> parentFiles = new ArrayList<FileInfo>();
		FileInfo parent = file;
		while (parent.getParent() != null) {
			parentFiles.add(parent.getParent());
			// process next parent
			parent = parent.getParent();
		}
		
		// determine the 'inherited from' folder path, we only have the 'inherited from' folder id
		for (AclItem aclItem : aclItems) {
			aclItem.setFile(file);
			if (aclItem.getInheritedFromFolderId() != null) {
				String folderPath = "";
				boolean folderFound = false;
				//for (int counter = model.getParentFiles().size() -1; counter>=0; counter--) {
				for (int counter = 0; counter < parentFiles.size(); counter++) {
					// start building the path when the right parent is found (the one that matches with aclItem.getInheritedFromFolderId()
					if (folderFound || parentFiles.get(counter).getId().equals(aclItem.getInheritedFromFolderId())) {
						folderFound = true;
						folderPath = parentFiles.get(counter).getName() + (folderPath.equals("") ? "" : java.io.File.separator) + folderPath; 
					}
				}
				
				aclItem.setInheritedFromFolder(folderPath);								
			}
		}*/
		
		if (aclItems != null) {
			for (AclItem aclItem : aclItems) {
				aclItem.setFile(file);
				if (aclItem.getInheritedFromFolderId() != null) {
					try {
						aclItem.setInheritedFromFolder(FileCache.getFile(aclItem.getInheritedFromFolderId()).getName());
					} catch (DBException e) {
						logger.error("Could not determine InheritedFromFolder name", e);
					}
				}
			}
		}
	}
	
	@Override
	public Object getParent(Object element) {
		return null;
	}

	public List<AclItem> getAclItems() {
		return aclItems;
	}

	@Override
	public void dispose() {
		
	}
}