package com.perdemia.permissionanalyzer.application.jface.contentproviders;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILazyTreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.FilterConfig;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper.ObjectType;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.DBObject;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.util.MemberCache;
import com.perdemia.permissionanalyzer.data.util.MembershipCache;

/**
 * <p>
 * This class reacts on the file selection in the main tree and provides the content for the Effective Permissions tab. It's a lazy loading content provider and it first checks if the ACL's of the
 * current folder and all the parent folders has been loaded. Then it will load all the members, depending on the filter settings. If no member filters have been selected and there is no search text entered, then
 * it will only load the direct ACE's of the current file (no avoid expanding groups like Domain Users).  If a search text has been entered or a member filter has been entered, then the provider will load
 * all members that belong to the selected file. That may include group memberships. When a user is expanded, the ACE's for that specific user are loaded using {@link MemberCache#getMemberOf(Member)} and the 
 * {@link FileInfo#getAclItems()} or {@link FileInfo#getAllAclItems()} of all the parent files (and current file) depending on the "Apply filter" checkbox.
 * </p>
 * <p>
 * This content provider differs from the {@link TracePartContentProvider} because this provider will load all members and when requested the ACE's for that member. While the {@link TracePartContentProvider} will load
 * all ACE's for a given member immediately, without lazy loading. The {@link TracePartContentProvider} acts upon a given user/group while this content provider acts on the selected file and loads all members.
 * </p>
 * <p>
 * Other differences:
 * <ul>
 * <li>This provider will always load <b>explicit</b> ACE's, while the trace part may also load inherited ACE's (depending on the checkbox "hide inherited permissions".</li>
 * <li>This provider will set the flag <code>applicableOnCurrentFolder</code> of a AclItem to false when a parent ACE is not found on the current folder, while the trace part will display the ACE on the corresponding parent folder.</li>
 * <li>This provider will display the member name that belongs to the ACE, while the trace part will also set the ACE member to the selected member and it will set the original ACE member in {@link AclItem#setViaMember(Member)}.</li>
 * <li>The trace provider will generate dummy ACE rows for parent files that don't have any ACE's, so that the folder tree will stay consistent.
 * </ul>
 * </p>
 * @author Reinier
 *
 */
public class EffectivePermissionsPartContentProvider implements ILazyTreeContentProvider {
	private static Logger logger = LoggerFactory.getLogger(EffectivePermissionsPartContentProvider.class.getName());	
	private TreeViewer viewer;
	private EffectivePermissionsPartContentProviderConfig config;
	private List<FileInfo> parentFiles;
	private List<Member> directMembers;
				
	public EffectivePermissionsPartContentProvider(TreeViewer viewer){
		this.viewer = viewer;
	}
	
	@Override
	public void dispose() {
	}

	/**
	 * Received an {@link EffectivePermissionsPartContentProviderConfig} object and checks if the ACL list of the selected file and all its parents has been loaded. Depending on the 
	 * {@link EffectivePermissionsPartContentProviderConfig#isApplyFilter()} flag the method will set {@link FileInfo#setAclItems(List)} or {@link FileInfo#setAllAclItems(List)}.
	 * 
	 * It will then decide whether to load the direct ACE's of the file, or loading all members (including group memberships) that belong to the selected file and parents using {@link #searchMembers()}.
	 * When memberships are excluded OR (no member filters have been selected AND the search text is empty), then it will load the direct members on the selected file. Otherwise the complete list is loaded.
	 */
	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		config = (EffectivePermissionsPartContentProviderConfig) newInput;
		
		if (config != null && config.getSelectedFile() != null) {
			// determine the list with parent files and check if the AclItems are loaded
			parentFiles = new ArrayList<FileInfo>();
			FileInfo parent = config.getSelectedFile();
			
			do {
				// in case this selection comes from the TableViewer (where parents are not always loaded because of the flat structure) it might be so that 
				// the ACL items of this parent have not been loaded yet
				// or the checkbox Apply Filter has been unchecked, which means we only have the filted AclItems on a FileInfo object! (while we need the whole list)
				if ((config.isApplyFilter() && parent.getAclItems() == null) || (!config.isApplyFilter() && parent.getAllAclItems() == null)) {
					try {
						if (config.isApplyFilter()) {
							PreparedStatement ps = config.getFilterHelper().buildFilterQuery(FilterHelper.ObjectType.ACL_ITEM, false, false, 0, "ACLITEM.MEMBER_ID", "ACLITEM.FILEINFO_ID = ?", parent.getId());
							parent.setAclItems(AclItem.getByPreparedStatement(ps));
						} else {
							// no filter applied, just retrieve all the ACE of this file
							parent.setAllAclItems(AclItem.where("FILEINFO_ID = ?", new Object[]{parent.getId()}));
						}
					} catch (DBException ex) {
						logger.error("Error while retrieving ACE's for the file/folder " + parent.getName(), ex);
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while retrieving ACE's for the file/folder " + parent.getName() + ": " + ex.getMessage());
					}
				}
				
				parentFiles.add(parent);
				
				// process next parent
				parent = parent.getParent();
				
			} while (parent != null);
			
			boolean noMemberships = !config.getFilterHelper().getFilterConfig(false).isIncludeGroupMembership();
			boolean noMemberFilter = !config.getFilterHelper().isApplyMemberFilter();
			
			// use the ACE's from the selected file to build the list of members, when memberships are not included, or when there's no member filter
			// when there's no member filter we don't want to auto expand groups like Domain User for example
			if (noMemberships || (noMemberFilter && config.getMemberSearch().isEmpty())) {
				directMembers = new ArrayList<Member>();
				List<AclItem> aclMembers;
				if (config.isApplyFilter()) {
					aclMembers = config.getSelectedFile().getAclItems();
				} else {
					aclMembers = config.getSelectedFile().getAllAclItems();
				}
				
				// get the distinct list with members
				for (AclItem aclItem : aclMembers) {
					// already in the list
					boolean memberFound = false;
					for (Member member : directMembers) {
						if (member.getId().equals(aclItem.getMemberId())) {
							memberFound = true;
							break;
						}
					}
					
					if (!memberFound) {
						// does this member match the search criteria?
						String search = config.getMemberSearch();
						search = search.replace("*", "").toLowerCase();
						if (!search.isEmpty()) {							
							if (aclItem.getMember(true).getName(false).toLowerCase().contains(search) || aclItem.getMember(false).getDisplayName().toLowerCase().contains(search)) {
								directMembers.add(aclItem.getMember(true));
							}
						} else {
							// no search text
							directMembers.add(aclItem.getMember(true));
						}
					}
				}
			} else {
				// we have a member filter, display all members
				try {
					directMembers = searchMembers();
				} catch (DBException | SQLException e) {
					logger.error("Error while searching for members.", e);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while searching for members: " + e.getMessage());
					directMembers = new ArrayList<Member>();
				}
			}
		} else {
			directMembers = new ArrayList<Member>();
		}
	}

	@Override
	public Object getParent(Object element) {
//		if (element instanceof FileInfo) {
//			FileInfo parentFile = ((FileInfo) element).getParent();
//			if (parentFile != null) {
//				logger.debug("returning parent with id " + parentFile.getId());
//				return parentFile;
//			} else {
//				logger.debug("returning directMembers as parent");
//				return directMembers;
//			}
//		} else {
//			return directMembers;
//		}
		// we don't store the parent of a Member object, and I don't believe this method is used for the (read-only) tree!
		//=return directMembers;
		return null;
	}

	/**
	 * When the element is a {@link AclItem} then the child size will be 0 and no action will be taken. When the element is a {@link Member} then the memberOf list is loaded
	 * using {@link MemberCache#getMemberOf(Member)} and it is set in {@link Member#setMemberOfList(List)}. Then the method will check what AclItems (of the selected file) match with
	 * the memberOf list. Those AclItems are used to determine the effective permissions for this member, using {@link PermissionUtil#determineEffectivePermissions(List)} and stored 
	 * in {@link Member#setEffectivePermissions(com.perdemia.permissionanalyzer.data.EffectivePermissions)}.
	 * 
	 */
	@Override
	public void updateElement(Object parent, int index) {
		if (logger.isTraceEnabled()){
			logger.trace("Entering updateElement: " + parent + ", " + index);
		}
		
		ApplicationUtil.setBusyCursor();
		
		Member memberElement = null;
		AclItem aclItemElement = null;
		
		try {
			if (parent instanceof Member) {
				// AclItem requested, or a member of a group?
				Member parentMember = (Member) parent;
				if (parentMember.getAclItemList() != null && (index < parentMember.getAclItemList().size() || index == 0)) {
					if (!parentMember.getAclItemList().isEmpty()) {
						aclItemElement = parentMember.getAclItemList().get(index);
					} else {
						aclItemElement = new AclItem();
						aclItemElement.setMember(new Member("No explicit ACE's found"));						
					}
				} else {
					int aclSize = parentMember.getAclItemList() != null ? parentMember.getAclItemList().size() : 0;
					if (aclSize == 0) { 
						// add one for the dummy item "No explicit ACE's found"
						aclSize = 1;
					}
					memberElement = parentMember.getMembers().get(index - aclSize);
				}
			} else {
				memberElement = directMembers.get(index);
			}
					
			if (aclItemElement != null) {
				// just replace the AclItem in the tree and set the child count to 0
				if (viewer != null) {
					viewer.replace(parent, index, aclItemElement);
					viewer.setHasChildren(aclItemElement, false);
				}
			} else {
				// determine the memberOf list for this member
				//TODO apply exclude filter on the list?
				List<Integer> memberOfIdList = MembershipCache.getMemberOf(memberElement);
				memberElement.setMemberOfList(memberOfIdList);
				
				// determine the applicable AclItems for this member on the selected file
				List<AclItem> fileAclItems;
				List<AclItem> memberAclItems = new ArrayList<AclItem>();
				if (config.isApplyFilter()) {
					fileAclItems = config.getSelectedFile().getAclItems();
				} else {
					fileAclItems = config.getSelectedFile().getAllAclItems();
				}
				
				if (fileAclItems != null) {
					for (AclItem aclItem : fileAclItems) {
						// match on the member of one of its group memberships?
						if (aclItem.getMemberId().equals(memberElement.getId()) || memberOfIdList.contains(aclItem.getMemberId())) {
							memberAclItems.add(aclItem);
						}
					}
				}
				
				// determine the effective permissions for this file (not for all the parent folders!)
				memberElement.setEffectivePermissions(PermissionUtil.determineEffectivePermissions(memberAclItems));
				
				if (logger.isTraceEnabled()){
					logger.trace("Replacing tree element with index " + index + ": " + memberElement);
				}
				
				if (viewer != null) {
					viewer.replace(parent, index, memberElement);
					viewer.setHasChildren(memberElement, true);
				}
			}
		} catch (DBException ex) {
			logger.error("Error while updating element of " + parent, ex);
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while updating element of " + parent + ": " + ex.getMessage());
		}
		
		ApplicationUtil.setDefaultCursor();
	}

	/**
	 * <p>
	 * When the given member is a group all group members will be loaded using {@link MemberCache#getFiltedGroupMembers(Member, boolean, boolean, boolean)} or 
	 * {@link MemberCache#getGroupMembers(Member, boolean, boolean, boolean) depending on the flag {@link EffectivePermissionsPartContentProviderConfig#isApplyFilter()}.
	 * </p>
	 * <p>
	 * After the group members have been loaded, all the <b>explicit</b> ACE's on the selected file and its parent folders will determined and stored in {@link Member#getAclItemList()}.
	 * </p>
	 */
	@Override
	public void updateChildCount(Object element, int currentChildCount) {
		if (logger.isTraceEnabled()){
			logger.trace("Entering updateChildCount: " + element + ", " + currentChildCount);
		}
		
		ApplicationUtil.setBusyCursor();
		
		int childSize = 0;
		
		if (element instanceof Member) {
			Member memberElement = (Member) element;
			
			if (memberElement.isGroup()) {
				// load member info
				try {
					FilterConfig filterConfig = config.getFilterHelper().getFilterConfig(false);
					
					if (config.isApplyFilter()) {
						// only select the members that match the filter criteria
						memberElement.setMembers(MembershipCache.getFiltedGroupMembers(memberElement, filterConfig.isIncludeUsers(), filterConfig.isIncludeGroups(), config.isUseNestedGroupMembers(), false, false, config.getFilterHelper()));
					} else {
						// all members of this group
						List<Member> groupMembers = MembershipCache.getGroupMembers(memberElement, filterConfig.isIncludeUsers(), filterConfig.isIncludeGroups(), config.isUseNestedGroupMembers());
						memberElement.setMembers(groupMembers);
					}
					
				} catch (DBException ex) {
					logger.error("Error while getting members for " + memberElement, ex);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting members for " + memberElement + ": " + ex.getMessage());
				}
				childSize = memberElement.getMembers().size();
			}
			
			// loop through all the parent folders (and the current folder/file) to see which aclItems are applicable
			memberElement.setAclItemList(new ArrayList<AclItem>());
			for (FileInfo file : parentFiles) {
				List<AclItem> aclItems;
				if (config.isApplyFilter()) {
					aclItems = file.getAclItems();
				} else {
					aclItems = file.getAllAclItems();
				}
				
				for (AclItem aclItem : aclItems) {
					
					// only pick the aclItems that belong to the selected member
					if (aclItem.getMemberId().equals(memberElement.getId()) || memberElement.getMemberOfList().contains(aclItem.getMemberId())) {
	
						// AclItems with flag INHERIT_ONLY on the current file only apply to sub files and folders (lower than the selected file/folder)
						boolean isOnlyInheritForCurrentFile = aclItem.isFlagInheritOnly() && file.getId().equals(config.getSelectedFile().getId());
						
						// Determine if the AclItem is applicable on the current folder (AclItems that don't apply on the current folder are grayed out)
						boolean applicableOnCurrentFolder = false;
						List<AclItem> currentFileAclItems;
						if (config.isApplyFilter()) {
							currentFileAclItems = config.getSelectedFile().getAclItems();
						} else {
							currentFileAclItems = config.getSelectedFile().getAllAclItems();
						}
						
						for (AclItem currentFolderAclItem : currentFileAclItems) {
							if ((currentFolderAclItem.getInheritedFromFolderId() == null || currentFolderAclItem.getInheritedFromFolderId().equals(file.getId())) && 
									currentFolderAclItem.getMemberId().equals(aclItem.getMemberId()) && currentFolderAclItem.getPermissionId().equals(aclItem.getPermissionId())) {
								applicableOnCurrentFolder = true;
								break;
							}
						}
						
						aclItem.setApplicableOnSelectedFolder(applicableOnCurrentFolder);
						
						// also skip implicit ace's
						if (!isOnlyInheritForCurrentFile && !aclItem.isFlagInheritedAce()) {
							memberElement.getAclItemList().add(0, aclItem);
							aclItem.setFile(file);
							childSize++;	
						}
					}
				}
			}
			
			if (memberElement.getAclItemList().size() == 0) {
				// we always use one dummy child for the text "No explicit ACE's found"
				childSize++;
			}
		} else {
			childSize = directMembers.size();
		}
		
		if (logger.isDebugEnabled()) {
			logger.debug("Setting child count " + childSize + " for element " + element.toString());
		}
		
		if (viewer != null) {
			viewer.setChildCount(element, childSize);
		}
		
		ApplicationUtil.setDefaultCursor();
		
		if (logger.isDebugEnabled()) {
			logger.debug("Exiting updateChildCount");
		}
	}

	public List<Member> getDirectMembers() {
		return directMembers;
	}
	
	/**
	 * Builds the query that is required to find members that match the selected filter criteria in the MemberFilterComposite.
	 * This method is only used when the option "Include nested group memberships" is enabled. Otherwise, we just use the direct members 
	 * from the ACE's found on the selected file.
	 * 
	 * The thing with group memberships is that we may find ACE's that don't belong to the users selected by the filter criteria! The member list in the
	 * effective permissions tab will only show members that match the filter criteria. So, instead of using the direct members from the 
	 * (filtered) ACE's on the selected file, we use the <b>groups</b> that belong to the members on the ACE's (or the members that match the
	 * filter criteria).
	 * 
	 * @return List of members
	 * @see {@link FilterHelper#buildFilterQuery(ObjectType, boolean, int, String, String, Object...)}
	 * @see {@link Member#getByPreparedStatement(PreparedStatement)}
	 */
	public List<Member> searchMembers() throws DBException, SQLException {
		StringBuilder additionalMemberFilterBuilder = new StringBuilder("FILTER_MEMBERSHIP.GROUP_ID IN (");
		List<Object> additionalMemberFilterValues = new ArrayList<Object>();
		Iterator<AclItem> iter;
		
		// on show members for the ACE's of the selected directory
		if (config.isApplyFilter()) {
			iter = config.getSelectedFile().getAclItems().iterator();
		} else {
			iter = config.getSelectedFile().getAllAclItems().iterator();
		}
		
		if (iter.hasNext()) {
			while (iter.hasNext()) {
				additionalMemberFilterBuilder.append("?");
				additionalMemberFilterValues.add(iter.next().getMemberId());
				if (iter.hasNext()) {
					additionalMemberFilterBuilder.append(",");
				}
			}
			
			additionalMemberFilterBuilder.append(")");
			
			// Filter on member name?
			String search = config.getMemberSearch();
			if (!search.isEmpty()) {
				search = search.toLowerCase().replace("*", "%");
				if (!search.endsWith("%")) {
					search += "%";
				}
				
				additionalMemberFilterBuilder.append(" AND (FILTER_MEMBER.NAME_LOWERCASE LIKE ? OR FILTER_MEMBER.DISPLAY_NAME_LOWERCASE LIKE ?)");
				additionalMemberFilterValues.add(search); // account name
				additionalMemberFilterValues.add(search); // display name
			}		
			
			PreparedStatement searchMembersPS;
			if (config.isApplyFilter()) {
				// IncludeUsers and IncludeGroups checkboxes are handled in buildFilterQuery 
				searchMembersPS = config.getFilterHelper().buildFilterQuery(ObjectType.MEMBER, false, false, 0, "NAME_LOWERCASE", additionalMemberFilterBuilder.toString(), additionalMemberFilterValues.toArray());
			} else {
				// don't apply the filter, just search on member name and the ACL ID's of the selected folder
				String query = FilterHelper.MEMBER_BASE_QUERY + " INNER JOIN {0}.MEMBERSHIP FILTER_MEMBERSHIP ON FILTER_MEMBER.ID = FILTER_MEMBERSHIP.MEMBER_ID WHERE " + additionalMemberFilterBuilder.toString();
				query = MessageFormat.format(query, DBObject.SCHEMA);
				DBObject.addLimitAndOrderBy(query, "NAME_LOWERCASE", null, 0);
				searchMembersPS = DBObject.getPreparedStatement(query);
				
				// set the PS parameters
				for (int index = 0; index < additionalMemberFilterValues.size(); index++) {
					searchMembersPS.setObject(index + 1, additionalMemberFilterValues.get(index));
				}
			}
			
			return Member.getByPreparedStatement(searchMembersPS);
		} else {
			// no ACE's
			return new ArrayList<Member>();
		}
	}

	public EffectivePermissionsPartContentProviderConfig getConfig() {
		return config;
	}

	public void setConfig(EffectivePermissionsPartContentProviderConfig config) {
		this.config = config;
	}
}