package com.perdemia.permissionanalyzer.application.jface.contentproviders;

import gnu.trove.list.array.TIntArrayList;

import java.sql.PreparedStatement;
import java.util.List;

import com.perdemia.permissionanalyzer.application.export.ExportHelper;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.util.FileCache;
import com.perdemia.permissionanalyzer.data.util.MemberCache;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILazyContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MainFileTableContentProvider implements ILazyContentProvider  {
	private static Logger logger = LoggerFactory.getLogger(MainFileTableContentProvider.class.getName());	
	private TableViewer viewer;
	private TIntArrayList fileIds;
	
	public MainFileTableContentProvider(TableViewer viewer){
		this.viewer = viewer;
	}
	
	@Override
	public void dispose() {
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		fileIds = (TIntArrayList) newInput;
	}

	@Override
	public void updateElement(int index) {
		try {
			FileInfo file = loadFileInfo(index, FilterHelper.getInstance());
			
			if (file != null) {					
				if (logger.isTraceEnabled()){
					logger.trace("Replacing table element with index " + index + ": " + file);
				}
				
				viewer.replace(file, index);
			}
			
		} catch (DBException ex) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", ex.getMessage());
		}
	}
	
	/**
	 * Loads all the file information when {@link #updateElement(int)} is called. This method is also used when generating a report. 
	 * 
	 * @param index
	 * @param filterHelper instance of the FilterHelper, note that this class is using FilterHelper.getInstance but that the ExportHelper is using its own instance of a FilterHelper containing the report filters!
	 * @throws DBException
	 * 
	 * @see {@link ExportHelper#exportReport(com.perdemia.permissionanalyzer.data.Report)}
	 */
	public FileInfo loadFileInfo(int index, FilterHelper filterHelper) throws DBException {
		if (logger.isTraceEnabled()){
			logger.trace("Entering updateElement");
		}
		
		if (fileIds == null) {
			return null;
		}
		
		FileInfo file = null;
		int fileId = fileIds.getQuick(index);
		
		try {
			file = FileCache.getFile(fileId);
						
			if (file != null) {
								
				// determine all the parent files to build up the complete file path
				FileInfo currentParent = file;
				while (true) {
					if (currentParent.getParentId() != null) {
						// retrieve the next parent from cache or DB
						try {
							FileInfo nextParent = FileCache.getFile(currentParent.getParentId());
							currentParent.setParent(nextParent);
							currentParent = nextParent;
						} catch (DBException ex) {
							logger.warn("Parent ID " + currentParent.getParentId() + " not found while loading file info " + file.getName() + ": " + ex.getMessage());
							currentParent.setParent(new FileInfo("???"));
							break;
						}
					} else {
						break;
					}
				}
				
				// get the owner
				if (file.getOwnerId() != null) {
					file.setOwner(MemberCache.getMember(file.getOwnerId()));
				}
				
				// load the related AclItems for this FileInfo
				if (file.getAclItems() == null) {
					List<AclItem> aclItems;
					PreparedStatement ps = filterHelper.buildFilterQuery(FilterHelper.ObjectType.ACL_ITEM, false, false, 0, "ACLITEM.MEMBER_ID", "ACLITEM.FILEINFO_ID = ?", file.getId());
					//if (ps != null) {
						aclItems = AclItem.getByPreparedStatement(ps);
					//} 
					//else {
						// no filter applied, just retrieve all the ACE of this file
					//	aclItems = AclItem.where("FILEINFO_ID = ?", new Object[]{file.getId()});
					//}
					
					file.setAclItems(aclItems);
				}
				file.setEffectivePermissions(PermissionUtil.determineEffectivePermissions(file.getAclItems()));
				
				if (PreferenceHelper.getInstance().isCalculateOwnershipRatioInOverview()) {
					MainFileTreeContentProvider.determineOwnershipRatio(filterHelper, file);
				}
			} else {
				logger.warn("FileInfo with ID " + fileId + " not found in database while building table list with files.");
			}
		} catch (DBException ex) {
			// add file name and index to the error message
			throw new DBException("Error while retrieving ACE's or file info for the file/folder " + (file != null ? file.getName() : "null") + ", index " + index + ": " + ex.getMessage(), ex);
		}
		
		return file;
	}
}