package com.perdemia.permissionanalyzer.application.jface.contentproviders;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.util.MembershipCache;

public class TracePartContentProvider implements ITreeContentProvider {
	List<AclItem> aclItems;
	
	public Object[] getChildren(Object parentElement) {
		List<AclItem> children = new ArrayList<AclItem>();
		Integer fileId = ((AclItem) parentElement).getFile().getId();
		
		for (AclItem aclItem : aclItems) {
			if (aclItem.getFile().getParentId() != null && aclItem.getFile().getParentId().equals(fileId)) {
				children.add(aclItem);
			}
		}
		return children.toArray();
	}

	public Object getParent(Object element) {
		AclItem aclItem = (AclItem) element;
		if (aclItem.getFile().getParentId() != null) {
			Integer parentFileId = aclItem.getFile().getParentId();
			return getLastAclItem(parentFileId);
		} else {
			return null;
		}
	}

	public boolean hasChildren(Object element) {
		AclItem aclItem = (AclItem) element;

		//only check for children if this is the last row for this folder
		AclItem lastChild = getLastChildAclItem(aclItem.getFile().getParentId());
		
		if (lastChild == null) {
			return false;
		}
		
		if (lastChild != aclItem) {
			return false;
		}
		
		return getChildren(element).length > 0;
	}

	public Object[] getElements(Object elements) {	
		List<AclItem> rootItems = new ArrayList<AclItem>();
				
		for (AclItem aclItem : aclItems) {
			if (aclItem.getFile().getParentId() == null) {
				rootItems.add(aclItem);
			}
		}
		return rootItems.toArray();
	}

	public void dispose() {
		
	}

	/**
	 * Reads the new {@link TracePartContentProviderConfig} and processes the selected {@link FileInfo} object. It builds a List of
	 * {@link AclItem} objects, each {@link aclItem} object represents a row in the detail tree.
	 * 
	 * <p>
	 * The field {@link TracePartContentProviderConfig.selectedFile} contains a List of {@link AclItem} objects which apply
	 * to the selected folder. The field {@link TracePartContentProviderConfig.parentFiles} is used to build a folder tree
	 * to display permission inheritance.
	 * </p>
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		TracePartContentProviderConfig config = (TracePartContentProviderConfig) newInput;
		aclItems = new ArrayList<AclItem>();
		boolean aclItemFound = false;
		
		//build a List of AclItem objects for this content provider, each AclItem object represents a row in the TreeViewer
		if (config != null && config.getSelectedMember() != null && config.getSelectedMember().getId() > 0 && config.getSelectedFile() != null) {
			List<FileInfo> files = new ArrayList<FileInfo>();
			files.add(config.getSelectedFile());
			if (config.getParentFiles() != null) {
				files.addAll(config.getParentFiles());
			}
			
			try {
				// load the group membership of the selected member to determine if ACE's are inherited from another group
				if (config.getSelectedMember().getMemberOfList() == null) {
					config.getSelectedMember().setMemberOfList(MembershipCache.getMemberOf(config.getSelectedMember()));
				}
				
				// loop through all the parent folders (and the current folder/file) to see which aclItems are applicable
				for (FileInfo file : files) {
					boolean aclItemForThisFile = false;
					List<AclItem> fileAclItems = file.getAclItems();
					
					// it might happen that the "Apply filter" in the Effective permissions tab has been unchecked and that a member without
					// permissions that match the filter is being placed in the Trace View. Check for this using getAllAclItems()
					if (file.getAllAclItems() != null) {
						fileAclItems = file.getAllAclItems();
					}
					
					for (AclItem aclItem : fileAclItems) {
						
						// only pick the aclItems that belong to the selected member
						if (aclItem.getMemberId().equals(config.getSelectedMember().getId()) || config.getSelectedMember().getMemberOfList().contains(aclItem.getMemberId())) {
	
							// skip aclItems with flag INHERIT_ONLY if they belong to the current file, those aclItems only apply to sub files and folders (lower than the selected file/folder)
							boolean isOnlyInheritForCurrentFile = aclItem.isFlagInheritOnly() && file.getId().equals(config.getSelectedFile().getId());
							
							// also skip implicit ace's in case the checkbox 'hide inherited permissions' has been checked
							if (!isOnlyInheritForCurrentFile && (!aclItem.isFlagInheritedAce() || !config.isHideInheritedPermissions())) {
								//add aclItem (is new row in the tree viewer)
								AclItem aclItemCopy = aclItem.clone();
								aclItemCopy.setMember(config.getSelectedMember());
								aclItemCopy.setFile(file);
								
								// ACE's inherited via group membership?
								if (!aclItem.getMemberId().equals(config.getSelectedMember().getId())) {
									aclItemCopy.setViaMember(aclItem.getMember(true));
								}
										
								aclItems.add(aclItemCopy);
								aclItemForThisFile = true;
								aclItemFound = true;
							}
						}
					}
					
					if (!aclItemForThisFile) {
						//create an empty row for this file/folder, so the file/folder name is at least mentioned
						AclItem aclItem = new AclItem();
						aclItem.setFile(file);
						aclItems.add(aclItem);
					}
				}
				
				if (!aclItemFound) {
					// remove the file rows with no aclItems because there is none to show
					aclItems.clear();
					
					AclItem dummyAcItem = new AclItem();
					FileInfo dummyFile = new FileInfo();
					dummyFile.setName("No ACE's found for this member");
					dummyAcItem.setFile(dummyFile);
					aclItems.add(dummyAcItem);
				}
				
			} catch (DBException e) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting memberOf list for the member " + config.getSelectedMember() + ": " + e.getMessage());
			}
		}
		
		/*
	
		//build a List of AclItem objects for this content provider, each AclItem object 
		//represents a row in the TreeViewer
		if (model != null) {
			List<FileInfo> files = new ArrayList<FileInfo>();
			files.add(model.getSelectedFile());
			files.addAll(model.getParentFiles());
			for (FileInfo file : files) {
				boolean aclItemForThisFile = false;
				for (Member member : model.getMembers()) {
					for (AclItem aclItem : model.getSelectedFile().getAclItems()) {
						
						// only pick the aclItems that belong to the selected member
						if (aclItem.getMemberId().equals(member.getId())) {
							//does this AclItem belong to the selected file OR is it delegated by the current file?
							//skip aclItems with flag INHERIT_ONLY if they belong to the current file, those aclItems only apply to sub files and folders
							if ((!aclItem.isFlagInheritOnly() && aclItem.isDirectlyAssigned() && file.getId().equals(model.getSelectedFile().getId())) ||
							aclItem.getInheritedFromFolderIds().contains(file.getId())) {
								
								//new AclItem (is new row in the tree viewer)
								AclItem newAclItem = aclItem.clone();
								newAclItem.setFile(file);
								newAclItem.setMember(member);
								aclItems.add(newAclItem);
								aclItemForThisFile = true;
							}
						}
					}
				}
				
				if (!aclItemForThisFile) {
					//create an empty row for this file/folder, so the file/folder name is at least mentioned
					AclItem aclItem = new AclItem();
					aclItem.setFile(file);
					aclItems.add(aclItem);
				}
			}
		}	*/
	}
	
	private AclItem getLastAclItem(Integer fileId){
		//start from the last element
		for (int counter = aclItems.size()-1; counter >= 0; counter--) {
			if (fileId.equals(aclItems.get(counter).getFile().getId())) {
				return aclItems.get(counter);
			}
		}
		return null;
	}
	
	/**
	 * Finds the last AclItem in the list {@link #aclItems} that has a parentFileId equal to the given <code>parentFileId</code>. This method is used because the tree in the Trace View
	 * (AclItem objects) is grouped by FileInfo.
	 * 
	 * @param parentFileId
	 * @return last AclItem
	 */
	private AclItem getLastChildAclItem(Integer parentFileId){
		//start from the last element
		for (int counter = aclItems.size()-1; counter >= 0; counter--) {
			if ((aclItems.get(counter).getFile().getParentId() == null && parentFileId == null) ||
			  (aclItems.get(counter).getFile().getParentId() != null && aclItems.get(counter).getFile().getParentId().equals(parentFileId))) {
				return aclItems.get(counter);
			}
		}
		return null;
	}

	public List<AclItem> getAclItems() {
		return aclItems;
	}
}

