package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.export.ExportHelper;
import com.perdemia.permissionanalyzer.application.export.ReportComparator;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel.ExplainType;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.ReportFilterSet;
import com.perdemia.permissionanalyzer.data.ReportHistory;

public class ReportDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(ReportDialog.class.getName());
	
	/**
	 * This dialog can be used for Reports and Polices, both entities require more or less the same fields
	 */
	private boolean isPolicies;
	
	/**
	 * Indicates if the dialog is used to select a particular report (as Member Filter). If so, the dialog will hide the "Run all" button, and it will
	 * display the buttons "Include selected report" and "Exclude selected report"
	 */
	private boolean isUseToSelectReport;
	
	/** Button ID's */
	public final static int LOAD_REPORT_ID = 3;
	
	/** Button to include a report as member filter, used when {@#isUseToSelectReport} is true. */
	public final static int ADD_TO_INCLUDE_ID = 4;
	
	/** Button to include a report as member filter, used when {@#isUseToSelectReport} is true. */
	public final static int ADD_TO_EXCLUDE_ID = 5;
			
	/** UI controls */
	private TableViewer reportTableViewer;
	
	/** Selected report used when loading a report into the Report perspective. */
	private Report selectedReport = new Report();
		
	/** List of all reports */
	private List<Report> reports;
	
	/** Date format for the last run date */
	private DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.getDefault());
	//private SimpleDateFormat dateFormat = new SimpleDateFormat("MMM dd yyyy HH:mm");
	
	public ReportDialog(Shell parentShell, boolean isPolicies, boolean isUseToSelectReport) {
		super(parentShell);
		this.isPolicies = isPolicies;
		this.isUseToSelectReport = isUseToSelectReport;
	}
	
	@Override
	protected boolean isResizable()  {
	  return true;
	}
	
	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		if (isPolicies) {
			shell.setImage(ImageFactory.getInstance().getImage(AppImage.POLICY_16X16));
		} else {
			shell.setImage(ImageFactory.getInstance().getImage(AppImage.REPORT_16X16));
		}
		shell.setSize(1200, 800);
	}
	
	@Override
	protected Control createDialogArea(final Composite container) {		
		ApplicationUtil.setBusyCursor();
				
		container.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		String title = "";
		Image image = null;
		if (isPolicies) {
			title = "Policies";
			image = ImageFactory.getInstance().getImage(AppImage.POLICY_48X48);
		} else {
			title = "Reports";
			image = ImageFactory.getInstance().getImage(AppImage.REPORT_48X48);
		}

		Composite composite = (Composite) super.createDialogArea(container, title, image);
		container.getShell().setText(title);
						
		// set width, height and layout
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 7;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);			
		
		if (isPolicies) {
			new ExplainLabel(composite, "A policy is a collection of filters that display unwanted permissions. The difference between a policy and a standard report is that a policy defines a combination of filters that should not yield any results. Should you have any search result items that appear as exceptions, then simply raise the threshold value in the details.", ExplainType.POLICIES);
		} else {
			new ExplainLabel(composite, "Current sets of filters can be saved as a new report. Reports can be exported to HTML or CSV files or can be reloaded within the program to change the filters or review results. There are four report types, each of which displays search results differently. Use Permission Analyzer to run reports automatically using the command parameter -allReports or -report \"<report name>\".", ExplainType.REPORTS);
		}
		
		// ------------------------------
		// Reports or Policies list
		// ------------------------------
		// load existing reports from the database 
		try {
			if (isPolicies) {
				reports = Report.where("EMAIL_THRESHOLD > 0", null, "NAME");
			} else {
				reports = Report.where("EMAIL_THRESHOLD = 0 OR EMAIL_THRESHOLD IS NULL", null, "NAME");
			}
		} catch (DBException e1) {
			MessageDialog.openError(null, "Database error", "Error while loading the reports: " + e1.getMessage());			
		}
		
		Label reportsTitle = new Label(composite, SWT.NONE);
		
		if (isPolicies) {
			reportsTitle.setText("Available policies:");
		} else {
			reportsTitle.setText("Available reports:");
		}
		reportsTitle.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		GridData gridData = new GridData();
		gridData.horizontalIndent = 5;
		gridData.verticalIndent = 5;
		reportsTitle.setLayoutData(gridData);
		
		reportTableViewer = new TableViewer(composite, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = reportTableViewer.getTable();
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		table.setLayoutData(gridData);

		TableViewerColumn nameColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		if (isPolicies) {
			nameColumn.getColumn().setWidth(285);
		} else {
			nameColumn.getColumn().setWidth(400);
		}
		nameColumn.getColumn().setText("Name");

		TableViewerColumn policyStatusColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		policyStatusColumn.getColumn().setWidth(150);
		policyStatusColumn.getColumn().setText("Policy status");
		
		TableViewerColumn itemCountColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		itemCountColumn.getColumn().setWidth(80);
		itemCountColumn.getColumn().setText("Result count");
		
		TableViewerColumn policyThresholdColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		policyThresholdColumn.getColumn().setWidth(70);
		policyThresholdColumn.getColumn().setText("Threshold");
				
		TableViewerColumn reportTypeColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		if (isPolicies) {
			reportTypeColumn.getColumn().setWidth(180);
		} else {
			reportTypeColumn.getColumn().setWidth(250);
		}
		reportTypeColumn.getColumn().setText("Report type");
		
		TableViewerColumn fileTypeColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		fileTypeColumn.getColumn().setWidth(150);
		fileTypeColumn.getColumn().setText("File type");
		
		TableViewerColumn targetPathColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		targetPathColumn.getColumn().setWidth(120);
		targetPathColumn.getColumn().setText("Last run");
						
		TableViewerColumn lastRunMessageColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		lastRunMessageColumn.getColumn().setWidth(120);
		lastRunMessageColumn.getColumn().setText("Last run message");
		
		TableViewerColumn targetFilePathColumn = new TableViewerColumn(reportTableViewer, SWT.NONE);
		targetFilePathColumn.getColumn().setWidth(120);
		targetFilePathColumn.getColumn().setText("File path");
		
		if (!isPolicies) {
			// hide the policy columns
			policyStatusColumn.getColumn().setWidth(0);
			policyStatusColumn.getColumn().setResizable(false);
			policyThresholdColumn.getColumn().setWidth(0);
			policyThresholdColumn.getColumn().setResizable(false);
		}
		
		reportTableViewer.setContentProvider(ArrayContentProvider.getInstance());
		reportTableViewer.setLabelProvider(new TableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				Report report = (Report) element;
				switch (columnIndex) {
					case 0: return report.getName();
					case 1:
						// only applicable for policies (not for regular reports)
						if (report.getEmailThreshold() == 0) {
							return "";
						} else if (report.getLastRunDate() == null) {
							return "Unknown";
						} else if (report.getLastStatusMessage() == null || !report.getLastStatusMessage().equalsIgnoreCase("ok")) {
							return "Policy ran with errors";
						} else if (report.getLastCount() >= report.getEmailThreshold()) {
							return "Found permissions";
						} else {
							return "Passed";
						}
					case 2: return String.valueOf(report.getLastCount());
					case 3: return String.valueOf(report.getEmailThreshold());
					case 4: return Report.reportTypeToString(report.getReportType());
					case 5: return Report.fileTypeToString(report.getFileType());
					case 6: return report.getLastRunDate() != null ? dateFormat.format(report.getLastRunDate()) : "";
					case 7: return report.getLastStatusMessage() != null ? report.getLastStatusMessage() : "";
					case 8: return report.getFilePath() != null ? report.getFilePath() : "";
					default: return "";
				}
			}
			
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0){
					if (isPolicies) {
						return ImageFactory.getInstance().getImage(AppImage.POLICY_16X16);
					} else {
						return ImageFactory.getInstance().getImage(AppImage.REPORT_16X16);
					}
				} else if (columnIndex == 1) {
					// only applicable for policies (not for regular reports)
					Report report = (Report) element;
					if (report.getLastRunDate() == null) {
						return ImageFactory.getInstance().getImage(AppImage.POLICY_STATUS_UNKNOWN_16X16);
					} else if (report.getLastStatusMessage() == null || !report.getLastStatusMessage().equalsIgnoreCase("ok")) {
						return ImageFactory.getInstance().getImage(AppImage.STATUS_ERROR_16X16);
					} else if (report.getLastCount() >= report.getEmailThreshold()) {
						return ImageFactory.getInstance().getImage(AppImage.STATUS_WARNING_16X16);
					} else {
						return ImageFactory.getInstance().getImage(AppImage.VALID_16X16);
					}
				} else if (columnIndex == 7) {
					Report report = (Report) element;
					if (report.getLastStatusMessage() == null || report.getLastStatusMessage().isEmpty()) {
						return ImageFactory.getInstance().getImage(AppImage.POLICY_STATUS_UNKNOWN_16X16);
					} else if (report.getLastStatusMessage() != null && report.getLastStatusMessage().equalsIgnoreCase("ok")) {
						return ImageFactory.getInstance().getImage(AppImage.VALID_16X16);
					} else {
						return ImageFactory.getInstance().getImage(AppImage.STATUS_WARNING_16X16);
					}
				} else {
					return null;
				}
			}
		});
		
		reportTableViewer.setInput(reports);
		
		// open edit dialog on double click
		reportTableViewer.addDoubleClickListener(new IDoubleClickListener(){
			@Override
			public void doubleClick(DoubleClickEvent event) {
				final IStructuredSelection selection = (IStructuredSelection) reportTableViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Report report = ((Report) selection.getFirstElement());
					ReportDetailsDialog detailsDialog = new ReportDetailsDialog(getShell(), report);
					detailsDialog.open();
					reportTableViewer.refresh();
				}
			}
		});
		
		//----------------------------------------------
		// context menu for the table
		//----------------------------------------------
		MenuManager memberMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		memberMenuMgr.setRemoveAllWhenShown(true);
		memberMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) reportTableViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Report report = ((Report) selection.getFirstElement());
					final String entity = isPolicies ? "policy" : "report";
					
					// load the filter sets (if applicable and save them in the class variables)
					try {
						report.loadFilterSets();
					} catch (DBException ex) {
						logger.error("Error while loading the report filter sets from the database", ex);
						MessageDialog.openError(null, "Error", "Error while loading the report filter sets from the database: " + ex.getMessage());
					}
					
					// edit action
					Action action = new Action() {
						public void run() {
							ReportDetailsDialog detailsDialog = new ReportDetailsDialog(getShell(), report);
							detailsDialog.open();
							reportTableViewer.refresh();
						}
					};
					action.setText("Edit " + entity + " details");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.EDIT_16X16));
					manager.add(action);
															
					// load into application
					if (!isUseToSelectReport) {
						action = new Action() {
							public void run() {
								selectedReport = report;
								setReturnCode(LOAD_REPORT_ID);
								close();
							}
						};
						action.setText("Load " + entity + " into application");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.TREE_16X16));
						manager.add(action);
					}
					
					// quickly view results
					action = new Action() {
						public void run() {
							try {
								ReportPreviewDialog previewDialog = new ReportPreviewDialog(getShell(), report.getMemberFilters(), report.getPermissionFilters(), report.getFolderFilters(), isPolicies);
								previewDialog.open();
							} catch (DBException ex) {
								logger.error("Error while building the preview", ex);
								MessageDialog.openError(getShell(), "Error", "Error while building the preview: " + ex.getMessage());
							}
						}
					};
					action.setText("Quickly view " + entity + " results");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.SEARCH_16X16));
					manager.add(action);
					
					// open report file
					action = new Action() {
						public void run() {
							String filePath = report.getLastFilePath();
							if (filePath != null && !filePath.isEmpty()) {
								Program.launch(filePath);
							} else {
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "File not present", "The " + entity + " has not yet run and the target file does not exist on the system.");
							}
						}
					};
					action.setText("Open report file");
					if (isPolicies) {
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.POLICY_16X16));
					} else {
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.REPORT_16X16));
					}
					manager.add(action);
					
					// run report					
					action = new Action() {
						public void run() {
							ExportHelper.exportReport(report, null, true, true);
							reportTableViewer.refresh();
						}
					};
					action.setText("Run " + entity);
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.EXPORT_16X16));
					manager.add(action);
					
					// open folder					
					action = new Action() {
						public void run() {
							if (report.getLastFilePath() != null && !report.getLastFilePath().isEmpty()) {
								String filePath = new File(report.getLastFilePath()).getParent();
								Program.launch(filePath);
							} else {
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "File not present", "The " + entity + " has not yet run and the target file does not exist on the system.");
							}
							
							
						}
					};
					action.setText("Open destination folder in Explorer");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.FOLDER_EXPLORER_16X16));
					manager.add(action);
					
					// compare report					
					action = new Action() {
						public void run() {
							ReportComparator.compareLastTwoReportRuns(report, true, true, true, true, true, true, null);
						}
					};
					action.setText("Compare last two runs");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.COMPARE_REPORT_16X16));
					manager.add(action);
					
					// Report history					
					action = new Action() {
						public void run() {
							ReportHistoryDialog reportHistoryDialog = new ReportHistoryDialog(getShell(), report);
							reportHistoryDialog.open();
						}
					};
					action.setText(entity.substring(0,1).toUpperCase() + entity.substring(1) + " history");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.REPORT_HISTORY_16X16));
					manager.add(action);
					
					// delete action
					action = new Action() {
						public void run() {
							try {
								if (SyncMessageDialog.showMessage(MessageDialog.CONFIRM, "Delete report", "Are you sure you want to delete the report " + report.getName())) {
									// first delete all filters of this report
									// apparently the cascade delete doesn't work on MSSQL!
									List<Filter> reportFilters = Filter.where("REPORT_ID = ?", new Object[]{report.getId()});
									for (Filter filter : reportFilters) {
										filter.delete();
									}
									List<ReportFilterSet> reportFilterSets = ReportFilterSet.where("REPORT_ID = ?", new Object[]{report.getId()});
									for (ReportFilterSet filterSet : reportFilterSets) {
										filterSet.delete();
									}
									
									// delete report history
									ReportHistory.delete("REPORT_ID = ?", report.getId());
									
									// delete the report itself
									report.delete();
									reports.remove(reportTableViewer.getTable().getSelectionIndex());
									reportTableViewer.refresh();
								}
							} catch (DBException ex) {
								logger.error("Error while deleting the report in the database", ex);
								MessageDialog.openError(null, "Error", "Error while deleting the report in the database: " + ex.getMessage());
							}
						}
					};
					action.setText("Delete " + entity);
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.DELETE_16X16));
					manager.add(action);
				}
				
			}
		});
		Menu memberMenu = memberMenuMgr.createContextMenu(reportTableViewer.getTable());
		reportTableViewer.getTable().setMenu(memberMenu);
		
		ApplicationUtil.setDefaultCursor();
		
		return composite;
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {		
		GridLayout gridLayout = new GridLayout(6, false);
		gridLayout.marginTop = 10;
		gridLayout.marginBottom = 5;
		
		parent.setLayout(new GridLayout(isUseToSelectReport ? 5: 4, false));
		parent.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		String entity = isPolicies ? "policies" : "reports";
		
		if (isUseToSelectReport) {
			Button includeReportButton = createButton(parent, ADD_TO_INCLUDE_ID, "Include report", false);
			includeReportButton.setImage(ImageFactory.getInstance().getImage(AppImage.INCLUDE_REPORT_16X16));
			((GridData) includeReportButton.getLayoutData()).widthHint = 120;
			((GridData) includeReportButton.getLayoutData()).horizontalIndent = 10;;
			includeReportButton.pack(true);
			
			Button excludeReportButton = createButton(parent, ADD_TO_EXCLUDE_ID, "Exclude report", false);
			excludeReportButton.setImage(ImageFactory.getInstance().getImage(AppImage.EXCLUDE_REPORT_16X16));
			((GridData) excludeReportButton.getLayoutData()).widthHint = 120;
			((GridData) excludeReportButton.getLayoutData()).horizontalIndent = 10;;
			excludeReportButton.pack(true);
			
		} else {
			Button runAllButton = createButton(parent, 99, "Run all " + entity, false);
			runAllButton.setImage(ImageFactory.getInstance().getImage(AppImage.EXPORT_16X16));
			((GridData) runAllButton.getLayoutData()).widthHint = 120;
			((GridData) runAllButton.getLayoutData()).horizontalIndent = 10;;
			runAllButton.pack(true);
			
			runAllButton.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					for (Report report : reports) {
						ExportHelper.exportReport(report, null, true, false);
					}
					
					// refresh the table
					reportTableViewer.refresh();
				}
			});
		}
		
		Composite filler = new Composite(parent, SWT.NONE);
		filler.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		createButton(parent, IDialogConstants.CLOSE_ID, "Close", false);
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		super.buttonPressed(buttonId);
		if (buttonId == IDialogConstants.CLOSE_ID) {
			close();
		} else if (buttonId == ADD_TO_INCLUDE_ID || buttonId == ADD_TO_EXCLUDE_ID){
			final IStructuredSelection selection = (IStructuredSelection) reportTableViewer.getSelection();
			if (selection != null && !selection.isEmpty()) {
				selectedReport = ((Report) selection.getFirstElement());
				setReturnCode(buttonId);
				close();
			} else {
				SyncMessageDialog.showMessage(MessageDialog.WARNING, "No report selected", "Please select a report.");
			}
		}
	}
		
	public Report getSelectedReport() {
		return selectedReport;
	}

	public void setSelectedReport(Report selectedMember) {
		this.selectedReport = selectedMember;
	}
}