package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILazyTreeContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;
import com.perdemia.permissionanalyzer.data.LdapContainer;
import com.perdemia.permissionanalyzer.data.LdapContainer.Type;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnector;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnectorImpl;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapFactory;
import com.unboundid.ldap.sdk.LDAPException;

public class SearchLdapDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(SearchLdapDialog.class.getName());
	
	/** ID's of the custom buttons */
	public final static int ADD_TO_INCLUDE_ID = 3;
	public final static int ADD_TO_EXCLUDE_ID = 4;
	
	/** UI controls */
	private TabFolder tabFolder;
	private TreeViewer ldapTreeViewer;
	private TableViewer ldapTableViewer;
	
	/** Reference to the Filter TableViewer in the FilterMemberPart */
	private TableViewer filterTableViewer;
	
	/** Dialog settings and values */
	private LdapConnector ldapConnector = new LdapConnectorImpl();
	private List<LdapConnectionInfo> ldapConnectionsList;
	private boolean allowOverlappingOUs;
	private LdapConnectionInfo ldapConnection;
	private LdapContainer selectedLdapContainer;
	private boolean useInclude;
	private boolean useExclude;
	
	public SearchLdapDialog(Shell shell, TableViewer filterListViewer, boolean allowOverlappingOUs, LdapConnectionInfo ldapConnection, boolean useInclude, boolean useExclude) {
		super(shell);
		this.filterTableViewer = filterListViewer;
		this.allowOverlappingOUs = allowOverlappingOUs;
		this.ldapConnectionsList = PreferenceHelper.getInstance().getLdapConnectionsList();
		this.ldapConnection = ldapConnection;
		this.useInclude = useInclude;
		this.useExclude = useExclude;
		
		if (ldapConnectionsList.size() == 0) {
			MessageDialog.openError(null, "No LDAP Connection", "No LDAP connection has been configured. Please open the preferences and configure at least one LDAP connection.");
			this.close();
		}
	}

	@Override
	protected boolean isResizable() {
		return true;
	}
	
	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText("Include or exclude members from LDAP containers");
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		shell.setSize(850, 700);
	}

	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Include or exclude members from LDAP containers", ImageFactory.getInstance().getImage(AppImage.LDAP_OU_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
						
		// set width, height and layout
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 7;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
				
		//ldap connection drop down
		Composite ldapConnectionComposite = new Composite(composite, SWT.NONE);
		ldapConnectionComposite.setLayout(new GridLayout(2, false));
		ldapConnectionComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Label ldapLabel = new Label(ldapConnectionComposite, SWT.NONE);
		ldapLabel.setText("LDAP connection: ");
		ldapLabel.setLayoutData(new GridData());
		
		final ComboViewer ldapConnectionCombo = new ComboViewer(ldapConnectionComposite, SWT.BORDER | SWT.READ_ONLY);
		ldapConnectionCombo.setContentProvider(ArrayContentProvider.getInstance());
		ldapConnectionCombo.setLabelProvider(new LabelProvider(){
			@Override
			public String getText(Object element) {
				return ((LdapConnectionInfo) element).getConnectionName();				
			}
			@Override
			public Image getImage(Object element) {
				return ImageFactory.getInstance().getImage(AppImage.CONNECTION_16X16);
			}
		});
		ldapConnectionCombo.getCombo().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapConnectionCombo.setInput(ldapConnectionsList);
		ldapConnectionCombo.addSelectionChangedListener(new ISelectionChangedListener(){
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				LdapConnectionInfo connection = ldapConnectionsList.get(ldapConnectionCombo.getCombo().getSelectionIndex());
				
				try {
					ApplicationUtil.setBusyCursor();
					ldapConnector.closeConnection();
					ldapConnector.openConnection(connection);
					
					// refresh the list with Root DN's
					refreshLdapTreeViewer();
					
					// clear the search list
					ldapTableViewer.setInput(new ArrayList<LdapContainer>());
					
					ApplicationUtil.setDefaultCursor();
					
				} catch (Exception e) {
					ApplicationUtil.setDefaultCursor();
					logger.error("Error while connection to LDAP", e);
					logger.info(connection.toString());
					MessageDialog.openError(null, "Error connecting to LDAP", "Could not connect to LDAP:\r\n\r\n" + LdapFactory.getUserFriendlyLdapError(e.toString()));
					
					// reset the (wrong) password
					if (connection.isOnlyStorePasswordInMemory()) {
						connection.setPassword("");
					}					
				} 
			}			
		});
						
		// tabs
		tabFolder = new TabFolder (composite, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_BOTH);
		tabFolder.setLayoutData(gridData);
		
		//======================================
		//Tab with browse LDAP
		//======================================
		TabItem browseLdapTab = new TabItem (tabFolder, SWT.NONE);
		browseLdapTab.setText ("Browse LDAP");
		Composite browseLdapComposite = new Composite(tabFolder, SWT.NONE);
		browseLdapComposite.setLayout(new GridLayout(1, false));
		browseLdapComposite.setLayoutData(gridData);
		
		// Search result list
		int swtStyleTree;
		if (filterTableViewer != null) {
			// this dialog is being used to add LDAP containers to the include/exclude filter lists
			swtStyleTree = SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.VIRTUAL;
		} else {
			// this dialog is used to browse a single LDAP container
			swtStyleTree = SWT.BORDER | SWT.V_SCROLL | SWT.VIRTUAL;
		}
		ldapTreeViewer = new TreeViewer(browseLdapComposite, swtStyleTree);
		ldapTreeViewer.setUseHashlookup(true);
		
		gridData = new GridData(GridData.FILL_BOTH);
		ldapTreeViewer.getTree().setLayoutData(gridData);
		ldapTreeViewer.setContentProvider(new LdapContentProvider(ldapTreeViewer, ldapConnector));
		ldapTreeViewer.setLabelProvider(new LdapLabelProvider());

		browseLdapTab.setControl(browseLdapComposite);
		
		//======================================
		//Tab with search LDAP
		//======================================
		TabItem searchLdapTab = new TabItem (tabFolder, SWT.NONE);
		searchLdapTab.setText ("Search LDAP");
		Composite searchLdapComposite = new Composite(tabFolder, SWT.NONE);
		searchLdapComposite.setLayout(new GridLayout(2, false));
		searchLdapComposite.setLayoutData(gridData);
		
		// search controls
		Label searchLabel = new Label(searchLdapComposite, SWT.NONE);
		searchLabel.setText("Search LDAP (use * as wildcard, a maximum of 500 items is displayed):");
		gridData = new GridData();
		gridData.verticalAlignment = SWT.BOTTOM;
		gridData.horizontalAlignment = SWT.LEFT;
		gridData.horizontalSpan = 2;
		searchLabel.setLayoutData(gridData);

		// Search textbox
		final Text searchText = new Text(searchLdapComposite, SWT.BORDER);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		searchText.setLayoutData(gridData);
		searchText.addTraverseListener(new TraverseListener() {
			@Override
			public void keyTraversed(TraverseEvent e) {
				if (e.detail == SWT.TRAVERSE_RETURN) {
					searchLdap(searchText.getText());
				}
			}
		});

		// Search button
		Button searchMembersButton = new Button(searchLdapComposite, SWT.PUSH);
		searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILTER_20X20));
		searchMembersButton.setToolTipText("Search LDAP");
		searchMembersButton.setLayoutData(new GridData());
		searchMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchLdap(searchText.getText());
			}
		});
		
		// Search result list
		int swtStyleTable = 0;
		if (filterTableViewer != null) {
			// this dialog is being used to add LDAP containers to the include/exclude filter lists
			swtStyleTable = SWT.BORDER | SWT.MULTI | SWT.V_SCROLL;
		} else {
			// this dialog is used to browse a single LDAP container
			swtStyleTable = SWT.BORDER | SWT.V_SCROLL;
		}		
		ldapTableViewer = new TableViewer(searchLdapComposite, swtStyleTable);
		ldapTableViewer.getTable().setLinesVisible(true);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 2;
		ldapTableViewer.getTable().setLayoutData(gridData);

		ldapTableViewer.setContentProvider(new ArrayContentProvider());
		ldapTableViewer.setLabelProvider(new LdapLabelProvider());
		
		searchLdapTab.setControl(searchLdapComposite);
		
		// Result list buttons
		Composite searchResultButtons = new Composite(composite, SWT.NONE);
		searchResultButtons.setLayout(new GridLayout(1, true));

		if (ldapConnection != null) {
			// use a predefined read-only selection item
			ldapConnectionCombo.setSelection(new StructuredSelection(ldapConnection), true);
			ldapConnectionCombo.getCombo().setEnabled(false);
		} else {
			// let the user select a ldap connection, and preselect the first item
			if (ldapConnectionsList.size() > 0) {
				ldapConnectionCombo.setSelection(new StructuredSelection(ldapConnectionsList.get(0)), true);
			}
		}
		
		return composite;
	}
	
	private void refreshLdapTreeViewer(){
		try {
			List<LdapContainer> rootElements = ldapConnector.getRootDNs(); 
			ldapTreeViewer.getTree().setItemCount(rootElements.size());
			ldapTreeViewer.setInput(rootElements);
		} catch (LDAPException e) {
			MessageDialog.openError(null, "LDAP error", "Error while getting root DNs from LDAP.\r\n\r\n" + e.getMessage());
			logger.error("Error while getting root DNs from LDAP", e);
		}
	}
	
	/**
	 * Refresh the table with search results 
	 */
	private void searchLdap(String searchText) {
		try {
			ApplicationUtil.setBusyCursor();
			List<LdapContainer> containers = ldapConnector.searchContainers(searchText); 
			ldapTableViewer.setInput(containers);
			ApplicationUtil.setDefaultCursor();
		} catch (Exception e) {
			MessageDialog.openError(null, "Error", "Error while searching LDAP.\r\n\r\n" + e.getMessage());
			logger.error("Error while searching LDAP", e);
			ApplicationUtil.setDefaultCursor();
		}
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		if (filterTableViewer != null) {
			// this dialog is being used to add LDAP containers to the include/exclude filter lists
			if (useInclude) {
				Button includeButton = createButton(parent, ADD_TO_INCLUDE_ID, "Include selected containers", false);
				includeButton.setImage(ImageFactory.getInstance().getImage(AppImage.INCLUDE_GROUPS_16X16));
				((GridData) includeButton.getLayoutData()).widthHint = 240;
				includeButton.pack(true);
			}
			
			if (useExclude) {
				Button excludeButton = createButton(parent, ADD_TO_EXCLUDE_ID, "Exclude selected containers", false);
				excludeButton.setImage(ImageFactory.getInstance().getImage(AppImage.EXCLUDE_GROUPS_16X16));
				((GridData) excludeButton.getLayoutData()).widthHint = 240;
				excludeButton.pack(true);
			}
		} else {
			// this dialog is used to browse a single LDAP container
			createButton(parent, IDialogConstants.OK_ID, "OK", false);
		}
		
		createButton(parent, IDialogConstants.CLOSE_ID, "Close", false);
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == ADD_TO_INCLUDE_ID || buttonId == ADD_TO_EXCLUDE_ID) {

			IStructuredSelection selection;
			if (tabFolder.getSelectionIndex() == 0) {
				selection = (IStructuredSelection) ldapTreeViewer.getSelection();
			} else {
				selection = (IStructuredSelection) ldapTableViewer.getSelection();
			}
			for (Iterator<LdapContainer> iterator = selection.iterator(); iterator.hasNext();) {
				LdapContainer ldapContainer = iterator.next();
				Filter filter = new Filter(FilterType.MEMBER_LDAP_OU, buttonId == ADD_TO_INCLUDE_ID, ldapContainer.getDn(), ldapContainer.getDn());

				// does this filter already exists?
				boolean exists = false;
				List<Filter> currentFilters = (java.util.List<Filter>) filterTableViewer.getInput();
				for (Filter curFilter : currentFilters) {
					if (curFilter.getType().equals(FilterType.MEMBER_LDAP_OU) && (curFilter.getValue().equals(ldapContainer.getDn()) ||
							(!allowOverlappingOUs && (curFilter.getValue().startsWith(ldapContainer.getDn()) || ldapContainer.getDn().startsWith(curFilter.getValue()))))) {
						String message;
						if (allowOverlappingOUs) {
							message = "The DN \"" + ldapContainer.getDn() + "\" already exists in the list of filters.";
						} else {
							message = "The DN \"" + ldapContainer.getDn() + "\" already exists in the list of filters or overlaps with an existing filter (" + curFilter.getValue() + ").";
						}
						MessageDialog.openError(null, "Already exists", message);
						exists = true;
						break;
					}
				}

				if (!exists) {
					if (FilterHelper.checkNumberOfMemberFiltersForTrial(currentFilters, true)) {
						currentFilters.add(filter);
						setReturnCode(buttonId);
					}
				}			
			}
			filterTableViewer.refresh();

		} else if (buttonId == IDialogConstants.OK_ID) {

			// ok button
			IStructuredSelection selection;
			if (tabFolder.getSelectionIndex() == 0) {
				selection = (IStructuredSelection) ldapTreeViewer.getSelection();
			} else {
				selection = (IStructuredSelection) ldapTableViewer.getSelection();
			}
			selectedLdapContainer = (LdapContainer)selection.getFirstElement();
			super.buttonPressed(buttonId);

		} else if (buttonId == IDialogConstants.CLOSE_ID) {
			// don't set a return code, because it may already have been set by the include/exclude button
			close();
		} else {
			super.buttonPressed(buttonId);
		}
	}

	public LdapContainer getSelectedLdapContainer() {
		return selectedLdapContainer;
	}
}

class LdapContentProvider implements ILazyTreeContentProvider {
	private static Logger logger = LoggerFactory.getLogger(LdapContentProvider.class.getName());
	private TreeViewer viewer;
	private List<LdapContainer> rootElements;
	private LdapConnector ldapConnector;

	public LdapContentProvider(TreeViewer viewer, LdapConnector ldapConnector) {
		this.viewer = viewer;
		this.ldapConnector = ldapConnector;
	}

	@Override
	public void dispose() {}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		rootElements = (List<LdapContainer>) newInput;
	}

	@Override
	public Object getParent(Object element) {
		LdapContainer ldapContainer = (LdapContainer) element;
		if (ldapContainer.getParent() != null) {
			return ldapContainer.getParent();
		} else {
			return rootElements;
		}
	}

	@Override
	public void updateElement(Object parent, int index) {		
		LdapContainer element;
		if (parent instanceof LdapContainer) {
			element = (LdapContainer) ((LdapContainer) parent).getChildren().get(index);
		} else {
			element = rootElements.get(index);
		}

		if (logger.isTraceEnabled()) {
			logger.trace("Replacing tree element with index " + index + ": " + element);
		}
		viewer.replace(parent, index, element);
		viewer.setHasChildren(element, true);
		//TODO enable?
		//updateChildCount(element, -1);
	}

	@Override
	public void updateChildCount(Object element, int currentChildCount) {
		int length = 0;
		if (element instanceof LdapContainer) {
			LdapContainer ldapContainer = (LdapContainer) element;
			// load the children of this LdapContainer element
			try {
				ApplicationUtil.setBusyCursor();
				ldapContainer.setChildren(ldapConnector.getChildContainers(ldapContainer.getDn()));
			} catch (LDAPException e) {
				MessageDialog.openError(null, "LDAP error", "Error while getting ldap child containers for DN " + ldapContainer.getDn() + ".\r\n\r\n" + e.getMessage());
				logger.error("Error while getting ldap child containers for DN " + ldapContainer.getDn(), e);
			} finally {
				ApplicationUtil.setDefaultCursor();
			}
			length = (ldapContainer.getChildren() != null ? ldapContainer.getChildren().size() : 0);

		} else if (element == rootElements) {
			length = rootElements.size();

		} else {
			logger.warn("Element is not of type LdapContainer or rootElements in LdapContentProvider.updateChildCount: " + element.getClass());
		}

		if (logger.isDebugEnabled()) {
			logger.debug("Setting child count " + length + " for element " + element.toString());
		}
		viewer.setChildCount(element, length);
	}
}

class LdapLabelProvider extends StyledCellLabelProvider {
	@Override
	public void update(ViewerCell cell) {
		Object element = cell.getElement();
		StyledString text = new StyledString();
		LdapContainer ldapContainer = (LdapContainer) element;

		text.append(ldapContainer.getName() != null ? ldapContainer.getName() : "unknown");
		
		if (ldapContainer.getType().equals(Type.CONTAINER)) {
			cell.setImage(ImageFactory.getInstance().getImage(AppImage.LDAP_OU_16X16));
		} else if (ldapContainer.getType().equals(Type.DOMAIN)) {
			cell.setImage(ImageFactory.getInstance().getImage(AppImage.GLOBE_16X16));
		} else {
			cell.setImage(ImageFactory.getInstance().getImage(AppImage.FOLDER_16X16));
		}

		text.append(" (" + ldapContainer.getDn() + ") ", StyledString.COUNTER_STYLER);

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);
	}
}