package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILazyContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.parts.FilterMemberPart;
import com.perdemia.permissionanalyzer.application.parts.EffectivePermissionsPart;
import com.perdemia.permissionanalyzer.application.swt.DelayedKeyListenerText;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Domain;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.util.DomainCache;

/** This dialog is used to search a member in the MemberFilterComposite, or to select a member in the TracePart or to display the matching members of a wild card filter
 * in the MemberFilterComposite. In the last two cases this dialog won't display Include and Exclude buttons.
 */
public class SearchMembersDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(SearchMembersDialog.class.getName());
	
	/** Options for this dialog */
	private boolean useIncludeButton;
	private boolean useExcludeButton;
	private boolean useSelectButton = true;
	private boolean showDomainFilter;
	private boolean multiSelect;
	private boolean onlyShowGroups;
	
	/** Button ID's */
	public final static int ADD_TO_INCLUDE_ID = 3;
	public final static int ADD_TO_EXCLUDE_ID = 4;
	
	/** Filter data */
	private Filter.FilterType filterType;
	private java.util.List<Member> searchResultList = new ArrayList<Member>();
	
	/** UI controls */
	private Text searchText;
	private Text searchMultipleText;
	private TableViewer searchResultTableViewer;
	private Button showUsersCheckbox;
	private Button showGroupsCheckbox;
	private ComboViewer domainCombo;
	private Button expandSearchButton;
	
	/** Reference to the Filter TableViewer in the FilterMemberPart (in order to refresh the TableViewer when a filter is added!) */
	private TableViewer filterTableViewer;
	
	/** PreparedStatement given by the {@link EffectivePermissionsPart} when the checkbox "Include Nested Groupmembership" is selected. */
	private PreparedStatement basePS;
	private int basePSParamCount;
	
	/** List given by the {@link EffectivePermissionsPart} when the checkbox "Include Nested Groupmembership" is NOT selected. */
	private List<Member> baseList;
	
			
	/** Selected member which can be used by the caller after the dialog has been closed. */
	private Member selectedMember;
	
	private String title;
	private String description;
	
	/**
	 * Constructor used in the {@link FilterMemberPart}. This constructor contains a filterTableViewer and filterType which are used
	 * to add Members to a filter list as Include or Exclude items.  
	 * 
	 * @param shell
	 * @param filterTableViewer
	 * @param filterType
	 * @param onlyShowGroups
	 */
	public SearchMembersDialog(Shell shell, TableViewer filterListViewer, Filter.FilterType filterType, boolean onlyShowGroups, boolean useIncludeButton, boolean useExcludeButton){
		super(shell);
		this.filterTableViewer = filterListViewer;
		this.filterType = filterType;
		this.onlyShowGroups = onlyShowGroups;
		this.useIncludeButton = useIncludeButton;
		this.useExcludeButton = useExcludeButton;
		this.multiSelect = true;
		this.showDomainFilter = true;
		
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);
		//setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		//setBlockOnOpen(false);
		
	}

	/**
	 * Constructor used in the {@link EffectivePermissionsPart} when the checkbox "Include Nested Groupmembership" is selected.
	 * 
	 * @param shell
	 * @param basePS query used to select the members. The search expects at least three parameters in this PreparedStatement, one for the search text
	 * and two for the is_group column. When both "Show users" and "Show groups" are selected the same PreparedStatement can be used with filter values
	 * is_group = true or is_group = false.
	 * @param basePSParamCount the number of parameters in the basePS prepared statement. Using <code>basePS.getParameterMetaData().getParameterCount()</code> did
	 *  	not work because it throws an <code>"SQLServerException: Incorrect syntax near ','"</code> for the PS with a query containing <code>WHERE (MEMBERSHIP.GROUP_ID IN (?,?,?,?,?,?,?)</code>. 
	 */
	public SearchMembersDialog(Shell shell, PreparedStatement basePS, int basePSParamCount){
		super(shell);
		this.basePS = basePS;
		this.basePSParamCount = basePSParamCount;
		this.title = "Search a user or group on the selected directory/file";
		this.description = "This search dialog displays all the (filtered) members found on the selected file/directory.\r\n\r\nSearch users and groups (use * as wildcard):";
		this.useIncludeButton = false;
		this.useExcludeButton = false;
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);
	}
	
	/**
	 * Constructor used in the {@link EffectivePermissionsPart} when the checkbox "Include Nested Groupmembership" is NOT selected.
	 * 
	 * @param shell
	 * @param baseList list with Member objects that is used to fill the search result box. Filtering on name is done by looping through the list.
	 */
	public SearchMembersDialog(Shell shell, List<Member> baseList){
		super(shell);
		this.title = "Search a user or group on the selected directory/file";
		this.description = "This search dialog displays all the (filtered) members found on the selected file/directory.\r\n\r\nSearch users and groups (use * as wildcard):";
		this.baseList = baseList;
		this.useIncludeButton = false;
		this.useExcludeButton = false;
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);
	}
		
	@Override
	protected boolean isResizable()  {
	  return true;
	}
	
	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		shell.setSize(1100, 700);
	}
	
	@Override
	protected Control createDialogArea(Composite container) {		
		ApplicationUtil.setBusyCursor();
		
		if (title == null) {
			if (useIncludeButton && useExcludeButton) {
				title = onlyShowGroups ? "Include or exclude members from groups" : "Include or exclude members (users and groups)";
			} else if (useIncludeButton) {
				title = onlyShowGroups ? "Include members from groups" : "Include members (users and groups)";
			} else if (useExcludeButton) {
				title = onlyShowGroups ? "Exclude members from groups" : "Exclude members (users and groups)";
			} else {
				if (useSelectButton) {
					title = "Select a user or group";
				} else {
					title = "Overview of users and groups";
				}
			}
		}
		
		Composite composite = (Composite) super.createDialogArea(container, title, ImageFactory.getInstance().getImage(AppImage.GROUP_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
		container.getShell().setText(title);
				
		// set width, height and layout
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.marginTop = 7;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);			
		
		/*
		if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 3;
			gridData.heightHint = 40;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("The Trial version of Permission Analyzer has a limit of 50 users and 25 groups.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}*/
		
		if (description == null) {
			description = onlyShowGroups ? "Search groups (use * as wildcard):" : "Search users and groups (use * as wildcard):";
		}
		// Search label
		CLabel searchLabel = new CLabel(composite, SWT.NONE);
		searchLabel.setText(description);
		GridData gridData = new GridData();
		gridData.verticalAlignment = SWT.BOTTOM;
		gridData.horizontalAlignment = SWT.LEFT;
		gridData.horizontalSpan = 3;
		searchLabel.setLayoutData(gridData);
					
		// Search textbox
		searchText = DelayedKeyListenerText.create(composite, SWT.BORDER, new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				// because of the delayed keypress search, it might happen that the dialog has been closed!
				if (!showUsersCheckbox.isDisposed()) {
				if (e.keyCode == SWT.ARROW_DOWN) {
					searchResultTableViewer.getTable().setFocus();
				} else {
					searchMembers(e.keyCode == 13);
				}					
			}			
			}			
		});
		searchText.setMessage("search");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		searchText.setLayoutData(gridData);
		searchText.addTraverseListener(new TraverseListener() {
			@Override
			public void keyTraversed(TraverseEvent e) {
				if (e.detail == SWT.TRAVERSE_RETURN) {
					searchMembers(true);
				}
			}
		});
		
		// Multiple search text
		searchMultipleText = new Text(composite, SWT.BORDER | SWT.MULTI);
		searchMultipleText.setText("search multiple members, one per line");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.exclude = true;
		gridData.heightHint = 6 * searchMultipleText.getLineHeight();
		searchMultipleText.setLayoutData(gridData);
		searchMultipleText.setVisible(false);	
				
		// Search button
		Button searchMembersButton = new Button(composite, SWT.PUSH);
		searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILTER_16X16));
		searchMembersButton.setToolTipText("Search members");
		searchMembersButton.setLayoutData(new GridData());
		searchMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchMembers(true);
			}
		});
		
		expandSearchButton = new Button(composite, SWT.TOGGLE);
		expandSearchButton.setImage(ImageFactory.getInstance().getImage(AppImage.MULTIPLE_SEARCH_16X16));
		expandSearchButton.setToolTipText("Search multiple members");
		expandSearchButton.setLayoutData(new GridData());
		expandSearchButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				((GridData)searchText.getLayoutData()).exclude = expandSearchButton.getSelection();
				searchText.setVisible(!expandSearchButton.getSelection());
				((GridData)searchMultipleText.getLayoutData()).exclude = !expandSearchButton.getSelection();
				searchMultipleText.setVisible(expandSearchButton.getSelection());
				searchText.getParent().layout();
			}
		});
		
		// filter checkbox for users
		Composite checkboxes = new Composite(composite, SWT.NONE);
		checkboxes.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		checkboxes.setLayoutData(gridData);
		
		showUsersCheckbox = new Button(checkboxes, SWT.CHECK);
		showUsersCheckbox.setLayoutData(new GridData(GridData.BEGINNING));
		showUsersCheckbox.setText("Show users");
		showUsersCheckbox.setSelection(true);
		showUsersCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.USER_20X20));
		if (onlyShowGroups) {
			showUsersCheckbox.setVisible(false);
			showUsersCheckbox.setSelection(false);
		}
		showUsersCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchMembers(false);
			}
		});
		
		// filter checkbox for groups
		showGroupsCheckbox = new Button(checkboxes, SWT.CHECK);
		showGroupsCheckbox.setLayoutData(new GridData(GridData.BEGINNING));
		showGroupsCheckbox.setText("Show groups");
		showGroupsCheckbox.setSelection(true);
		showGroupsCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_20X20));
		if (onlyShowGroups) {
			showGroupsCheckbox.setVisible(false);
		}
		showGroupsCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchMembers(false);
			}
		});
		
		// Don't use the domain dropdown in case of the TracePart because in that case we get the baseQuery delivered by a PreparedStatement without a filter option 
		// for the domain
		if (showDomainFilter) {
			try {
				domainCombo = new ComboViewer(checkboxes, SWT.BORDER | SWT.READ_ONLY);
				domainCombo.setContentProvider(ArrayContentProvider.getInstance());
				domainCombo.setLabelProvider(new LabelProvider() {
					@Override
					public String getText(Object element) {
						return ((Domain) element).getDomain();
					}
				});
				gridData = new GridData();
				domainCombo.getCombo().setLayoutData(gridData);
				List<Domain> domains = new ArrayList<Domain>();
				domains.addAll(DomainCache.getDomains(false));
				domains.add(0, new Domain("All domains / computers", 0));
				domainCombo.setInput(domains);		
				domainCombo.setSelection(new StructuredSelection(domainCombo.getElementAt(0)), true);
				domainCombo.addSelectionChangedListener(new ISelectionChangedListener() {
					@Override
					public void selectionChanged(SelectionChangedEvent event) {
						searchMembers(false);						
					}					
				});
				
			} catch (DBException ex) {
				logger.error("Error while loading the list of domains in the search member dialog", ex);
			}
		}
				
		// Search result list
		if (multiSelect) {
			// multi select
			searchResultTableViewer = new TableViewer(composite, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.VIRTUAL | SWT.MULTI);
		} else {
			// single select
			searchResultTableViewer = new TableViewer(composite, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.VIRTUAL);
		}
		
		searchResultTableViewer.setUseHashlookup(true);
		searchResultTableViewer.getTable().setHeaderVisible(true);
		searchResultTableViewer.getTable().setLinesVisible(true);
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 3;
		searchResultTableViewer.getTable().setLayoutData(gridData);
		
		// Member name column
		TableViewerColumn column = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		column.getColumn().setText(onlyShowGroups ? "Group name" : "Account name");
		column.getColumn().setWidth(250);
		column.getColumn().setResizable(true);
		
		// Display name column
		column = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		column.getColumn().setText("Display name");
		column.getColumn().setWidth(400);
		column.getColumn().setResizable(true);			
		
		// Domain column
		column = new TableViewerColumn(searchResultTableViewer, SWT.NONE);
		column.getColumn().setText("Domain / computer");
		column.getColumn().setWidth(200);
		column.getColumn().setResizable(true);
		
		searchResultTableViewer.setContentProvider(new ILazyContentProvider(){
			private java.util.List<Member> searchResultList;
			private TableViewer viewer;
			
			@Override
			public void dispose() {}

			@SuppressWarnings("unchecked")
			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
				this.searchResultList = (java.util.List<Member>) newInput;
				this.viewer = (TableViewer)viewer;
			}

			@Override
			public void updateElement(int index) {
				viewer.replace(searchResultList.get(index), index);				
			}
		});
		
		searchResultTableViewer.setLabelProvider(new TableLabelProvider(){
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					if (((Member)element).isGroup()) {
						return ImageFactory.getInstance().getImage(AppImage.GROUP_20X20);
					} else {
						return ImageFactory.getInstance().getImage(AppImage.USER_20X20);
					}
				}
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				Member member = (Member) element;
				switch (columnIndex) {
					case 0: return member.getName(true);
					case 1: return member.getDisplayName(false, true);					
					case 2: return member.getDomain() != null ? member.getDomain().getDomain() : "None";
				}
				return null;
			}
		});
		
		searchResultTableViewer.setItemCount(searchResultList.size());
		searchResultTableViewer.setInput(searchResultList);
				
		// trigger default button at double click 
		searchResultTableViewer.addDoubleClickListener(new IDoubleClickListener() {
			@Override
			public void doubleClick(DoubleClickEvent event) {
				if (useIncludeButton) {
					buttonPressed(ADD_TO_INCLUDE_ID);
				} else if (useExcludeButton) {
					buttonPressed(ADD_TO_EXCLUDE_ID);
				} else {
					buttonPressed(IDialogConstants.OK_ID);
				}
			}			
		});
		
		// Result list buttons
		Composite searchResultButtons = new Composite(composite, SWT.NONE);
		searchResultButtons.setLayout(new GridLayout(1, true));

		// context menu for the Tree
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) searchResultTableViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					Action action = new Action() {
						public void run() {
							Member member = (Member) selection.getFirstElement();
							MemberInfoDialog dialog = new MemberInfoDialog(null, member);
							dialog.open();
						}
					};
					action.setText("Show member info");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
					manager.add(action);
				}
			}
		});
		
		Menu menu = menuMgr.createContextMenu(searchResultTableViewer.getTable());
		searchResultTableViewer.getTable().setMenu(menu);
		
		// pre-load the search result list in case this dialog is provided with a baseList
		if (baseList != null || basePS != null) {
			searchMembers(true);
		}
		
		ApplicationUtil.setDefaultCursor();
		return composite;
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {		
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
		
		// Copy to clipboard button
		Button copyMembersToClipboardButton = createButton(parent, -1, "Copy to clipboard", false);
		copyMembersToClipboardButton.setImage(ImageFactory.getInstance().getImage(AppImage.CLIPBOARD_16X16));
		copyMembersToClipboardButton.setToolTipText("Copy the list with all the members to the clipboard");
		((GridData) copyMembersToClipboardButton.getLayoutData()).widthHint = 200;
		copyMembersToClipboardButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (searchResultList != null) {
					StringBuilder membersText = new StringBuilder("Account name,Display name\r\n");
					for (Member member : searchResultList) {
						membersText.append(member.getName(true));
						membersText.append(",");
						membersText.append(member.getDisplayName());
						membersText.append("\r\n");
					}
					Clipboard clipboard = new Clipboard(Display.getDefault());
					TextTransfer textTransfer = TextTransfer.getInstance();
			        clipboard.setContents(new String[]{membersText.toString()}, new Transfer[]{textTransfer});
			        clipboard.dispose();
			        SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", "Copied " + searchResultList.size() + " members to the clipboard.");
				} else {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "The list with all the members has not been loaded yet, please hit the load button first.");
				}
			}
		});
		
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
				
		if (useIncludeButton) {
			Button includeButton = createButton(parent, ADD_TO_INCLUDE_ID, onlyShowGroups ? "Include selected groups" : "Include selected members", true);
			includeButton.setImage(ImageFactory.getInstance().getImage(onlyShowGroups ? AppImage.INCLUDE_GROUPS_16X16 : AppImage.INCLUDE_USERS_16X16));
			((GridData) includeButton.getLayoutData()).widthHint = 270;
			includeButton.pack(true);
		}
		
		if (useExcludeButton) {
			Button excludeButton = createButton(parent, ADD_TO_EXCLUDE_ID, onlyShowGroups ? "Exclude selected groups" : "Exclude selected members", false);
			excludeButton.setImage(ImageFactory.getInstance().getImage(onlyShowGroups ? AppImage.EXCLUDE_GROUPS_16X16 : AppImage.EXCLUDE_USERS_16X16));
			((GridData) excludeButton.getLayoutData()).widthHint = 270;
			excludeButton.pack(true);
		}
		
		if (!useIncludeButton && !useExcludeButton && useSelectButton) {
			createButton(parent, IDialogConstants.OK_ID, "Select", false);
		}		
		
		createButton(parent, IDialogConstants.CLOSE_ID, "Close", false);
		
	}
	
	@SuppressWarnings("unchecked")
	protected void buttonPressed(int buttonId) {
		if (buttonId == ADD_TO_INCLUDE_ID || buttonId == ADD_TO_EXCLUDE_ID) {
			
			IStructuredSelection selection = (IStructuredSelection) searchResultTableViewer.getSelection();
			for (Iterator<Member> iterator = selection.iterator(); iterator.hasNext();) {
				Member member = iterator.next();
				Filter filter = new Filter(filterType, buttonId == ADD_TO_INCLUDE_ID, String.valueOf(member.getId()), member.getDisplayName(true, false));;					
				filter.setMemberGroup(member.isGroup());
				
				// check for trial limitations
				List<Filter> memberFilters = (List<Filter>) filterTableViewer.getInput();
				if (FilterHelper.checkNumberOfMemberFiltersForTrial(memberFilters, true)) {
					memberFilters.add(filter);
					searchResultList.remove(member);
					setReturnCode(buttonId);
				} else {
					// filter was not allowed because of the trial
					break;
				}
			}
			
			searchResultTableViewer.setItemCount(searchResultList.size());
			searchResultTableViewer.refresh();
			filterTableViewer.refresh();
			
		} else if (buttonId == IDialogConstants.CLOSE_ID) {
			//don't set a return code if the include or exlude button has already been pressed
			if (getReturnCode() != ADD_TO_INCLUDE_ID && getReturnCode() != ADD_TO_INCLUDE_ID) {
				setReturnCode(IDialogConstants.CLOSE_ID);
			}
			close();
		} else if (buttonId == IDialogConstants.OK_ID) {
			// remember the selected member
			IStructuredSelection selection = (IStructuredSelection) searchResultTableViewer.getSelection();
			selectedMember = (Member) selection.getFirstElement();
			super.buttonPressed(buttonId);
		} else {
			super.buttonPressed(buttonId);
		}
	}
	
	/**
	 * Create the Job that refreshes the Member list
	 * 
	 * @param explicitUserSearchAction true in case the user pressed the search button or ENTER in the search text box
	 */
	private void searchMembers(boolean explicitUserSearchAction) {
		if (!showUsersCheckbox.getSelection() && !showGroupsCheckbox.getSelection()) {
			if (explicitUserSearchAction) {
				MessageDialog.openError(Display.getCurrent().getActiveShell(), "Invalid selection", "Select at least one of the checkboxes to show users and/or groups.");
			}
			return;
		}
		
		List<String> searches = new ArrayList<String>();
		
		if (expandSearchButton.getSelection()) {
			String[] multipleSearches = searchMultipleText.getText().split("\r\n");
			for (String search : multipleSearches) {
				searches.add(search.toLowerCase().replace('*', '%'));
			}
		} else {
			String search = searchText.getText().toLowerCase().replace('*', '%');
			if (search.length() > 2 && !search.startsWith("%") || explicitUserSearchAction) {
				if (explicitUserSearchAction && search.isEmpty()) {
					search = "%";
				}
				searches.add(search);
			}
		}
		
		// only search when the user explicit requested the search OR when the length is greater than 2 and doesn't start with % OR the search list is based on the baseList
		if (searches.size() > 0) {
			
			searchResultList.clear();
						
			try {
				ApplicationUtil.setBusyCursor();
						
				int domainId = -1;
				if (showDomainFilter) {	
					domainId = ((Domain) ((StructuredSelection)domainCombo.getSelection()).getFirstElement()).getId();
				}
				
				List<Member> members = new ArrayList<Member>();
				
				// PreparedStatement given? (used in EffectivePermissionsPart)
				if (basePS != null) {
					for (String search : searches) {
						if (!search.endsWith("%")) {
							search += "%";
						}
						
						basePS.setString(basePSParamCount - 2, search);
						basePS.setString(basePSParamCount - 3, search);
						
						// filter on is_group required?
						if (showGroupsCheckbox.getSelection() != showUsersCheckbox.getSelection()) {
							basePS.setBoolean(basePSParamCount - 1, showGroupsCheckbox.getSelection());
							basePS.setBoolean(basePSParamCount, showGroupsCheckbox.getSelection());
						} else {
							// don't filter on is_group, accept both false and true
							basePS.setBoolean(basePSParamCount - 1, false);
							basePS.setBoolean(basePSParamCount, true);
						}
						
						members.addAll(Member.getByPreparedStatement(basePS));
					}
				
				// baseList not null? (used in the EffectivePermissionsPart when the option "Include group membership" is unchecked. In that case the members found in the ACL of a file are used)
				} else if (baseList != null){
					members = new ArrayList<Member>();
					for (String search : searches) {
						search = search.replace("%", "");
						for (Member member : baseList) {
							// match display name or member name?
							if ((member.getDisplayName(false, true).toLowerCase().contains(search) || member.getName(false).toLowerCase().contains(search))
									// filter on groups or users?
									&& ((member.isGroup() && showGroupsCheckbox.getSelection()) || (!member.isGroup() && showUsersCheckbox.getSelection()))) {
								members.add(member);
							}
						}
					}
				
				// no query info has been provided, just use Member.findByName method (used by Select member button in the MemberFilterComposite)
				} else {
					for (String search : searches) {
						// a search with double quotes in the dialog means a exact match
						if (!search.endsWith("%") && !search.startsWith("\"")) {
							search += "%";
						}
						
						// don't use ORDER BY member.name_lowercase because this query might be used with a UNION and Oracle doesn't understand ORDER BY on column name in that case
						members.addAll(Member.findByName(search, showUsersCheckbox.getSelection(), showGroupsCheckbox.getSelection(), domainId, "3", 0, null, null));
					}
				}
				
				// sort the list on account name
				Collections.sort(members, new Comparator<Member>(){
					@Override
					public int compare(Member member1, Member member2) {
						return member1.getName(true).compareTo(member2.getName(true));
					}
					
				});
				
				// remove duplicates and the members that are already added as filter
				boolean membersFound = false;
				Member lastMember = null;
				for (Member member : members) {
					boolean memberAlreadyPresent = false;
					
					if (lastMember != null && lastMember.getId().equals(member.getId())) {
						memberAlreadyPresent = true;
						
					} else if (filterTableViewer != null) {
						java.util.List<Filter> filterList = (java.util.List<Filter>) filterTableViewer.getInput();
						// member already in the list?
						for (Filter filter : filterList) {
							if (filter.getValue().equals(String.valueOf(member.getId())) && filter.getType().equals(filterType)) {
								memberAlreadyPresent = true;
								break;
							}
						}
					}
					
					if (!memberAlreadyPresent) {
						searchResultList.add(member);
						membersFound = true;
					}
				}
				
				if (!membersFound && explicitUserSearchAction){
					String message;
					if (filterTableViewer != null) {
						message = "No members found or the members are already in the filter list.";
					} else {
						message = "No members found.";
					}
					MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "No results", message);
				}
				
			} catch (DBException | SQLException ex) {
				logger.error("Error while searching for members: " + ex.getMessage(), ex);
				MessageDialog.openError(null, "Error", "Error while searching for members: " + ex.getMessage());
			}
			
			searchResultTableViewer.setItemCount(searchResultList.size());
			searchResultTableViewer.refresh();
			
			ApplicationUtil.setDefaultCursor();
		}		
	}

	public Member getSelectedMember() {
		return selectedMember;
	}

	public void setSelectedMember(Member selectedMember) {
		this.selectedMember = selectedMember;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public boolean isUseSelectButton() {
		return useSelectButton;
	}

	public void setUseSelectButton(boolean useSelectButton) {
		this.useSelectButton = useSelectButton;
	}
}
