package com.perdemia.permissionanalyzer.application.logging;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;

import com.perdemia.permissionanalyzer.application.model.StatusListener;
import com.perdemia.permissionanalyzer.data.ScanLogItem;
import com.perdemia.permissionanalyzer.data.ScanLogItem.Severity;

/**
 * This handler encapsulates the log message in {@link StatusMessage} objects so that they can be shown in the GUI
 * using listeners on this class.
 *  
 * @author Reinier
 *
 */
public class GUILogHandler extends Handler {
	private static List<StatusListener> listeners = new ArrayList<StatusListener>();
	private static List<String> excludePackages;
	
	public GUILogHandler(){
        LogManager logmanager = LogManager.getLogManager();
        String level = logmanager.getProperty((new StringBuilder()).append(getClass().getName()).append(".level").toString());
        String excludePackagesProp = logmanager.getProperty((new StringBuilder()).append(getClass().getName()).append(".excludePackages").toString());
        if (excludePackagesProp != null) {
        	excludePackages = Arrays.asList(excludePackagesProp.split(","));
        } else {
        	excludePackages = new ArrayList<String>();
        }
        if (level != null) {
        	setLevel(Level.parse(level));
        }
	}
	
	@Override
	public void publish(LogRecord logrecord) {
		if(!isLoggable(logrecord)) {
			return;
		}
				
		Severity statusLevel;
		switch (logrecord.getLevel().intValue()){
			case 1000: statusLevel = Severity.ERROR; break;
			case 900: statusLevel = Severity.WARNING; break;
			case 800: statusLevel = Severity.INFO; break;
			case 700: statusLevel = Severity.INFO; break; //config
			case 500: statusLevel = Severity.DEBUG; break; //fine
			case 400: statusLevel = Severity.TRACE; break; //finer
			case 300: statusLevel = Severity.TRACE; break; //finest
			default: statusLevel = Severity.TRACE;
		}
		
		//stacktrace?
		String details = "";
		if (logrecord.getThrown() != null) {
			details = getStracktrace(logrecord.getThrown());			
		}
		
		// notify the listener(s)
		for (StatusListener listener : listeners) {
			listener.newStatusMessage(new ScanLogItem(statusLevel, logrecord.getMessage(), details));
		}		
	}
	
	@Override
	public boolean isLoggable(LogRecord logrecord) {
		boolean isLoggable = super.isLoggable(logrecord);
		
		if (!isLoggable) {
			return isLoggable;
		} else {
			for (String excludePackage : excludePackages) {
				if (logrecord.getSourceClassName().startsWith(excludePackage)) {
					return false;
				}
			}
			return true;
		}		
	}

	@Override
	public void flush() {}

	@Override
	public void close() throws SecurityException {}
	
	public static void addListener(StatusListener listener) {
		listeners.add(listener);
	}
	
	public static void removeListener(StatusListener listener) {
		listeners.remove(listener);
	}
	
	public static String getNestedMessage(Exception ex){
		Throwable nestedException = ex;
		if (ex.getCause() != null) {
			for (int counter = 0; counter < 5; counter++) {
				nestedException = nestedException.getCause();
				if (nestedException.getCause() == null) {
					break;
				}
			}
			return nestedException.getMessage();
		} else {
			return ex.getMessage();
		}
	}
	
	public static String getStracktrace(Throwable ex){
		StringWriter stringwriter = new StringWriter();
		PrintWriter printwriter = new PrintWriter(stringwriter);
		ex.printStackTrace(printwriter);
		printwriter.close();
		return stringwriter.toString();
	}
}
