package com.perdemia.permissionanalyzer.application.util;

import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * Provides methodes to encrypt / decrypt messages, to hash a password and to calculate a HMAC hash code to protect the integrity of an
 * encrypted text. The encryption is done using AES (Advanced Encryption Standard) in combination with Cypher Block Chaining (CBC). Cypher
 * Block Chaining splits the encryption algorithm up into blocks of data. It then encrypts one block and passes the running result into the
 * encryption algorithm for the next block. As such, each block encryption depends on the aggregate result of all previously encrypted
 * blocks in the same message.
 * </p>
 *
 * @author Reinier
 */
public final class AESEncryptionHelper extends AbstractEncryptionHelper {
	private static Logger logger = LoggerFactory.getLogger(AESEncryptionHelper.class.getName());

	/** Default encryption method. */
	private static final String DEFAULT_ENCRYPTION_ALGORITHM = "AES/CBC/PKCS5Padding"; // is actually PKCS7Padding underneath because of AES
																						// 128 bit!

	/**
	 * Encrypts a plain text using the given encoded key, iv and algorithm.
	 *
	 * @param plainText plain text
	 * @param encodedPassPhrase the encoded pass phrase
	 * @param iv the iv
	 * @param algorithm the algorithm
	 * @return encoded cipher text
	 * @throws UnsupportedEncodingException 
	 * @throws NoSuchPaddingException 
	 * @throws NoSuchAlgorithmException 
	 * @throws InvalidAlgorithmParameterException 
	 * @throws InvalidKeyException 
	 * @throws BadPaddingException 
	 * @throws IllegalBlockSizeException 
	 * @throws EncryptionException the encryption exception
	 * @see {@link #encode(byte[])}
	 */
	public static String encrypt(String plainText, String encodedPassPhrase, String iv, String algorithm) throws UnsupportedEncodingException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException {
		byte[] bytes = plainText.getBytes(TEXT_ENCODING);

		SecretKey secret = new SecretKeySpec(decode(encodedPassPhrase), "AES");

		// IV
		IvParameterSpec ivSpec = null;
		if (iv != null) {
			ivSpec = new IvParameterSpec(decode(iv));
		}

		Cipher ecipher = Cipher.getInstance(algorithm);
		ecipher.init(Cipher.ENCRYPT_MODE, secret, ivSpec);
		byte[] encrypted = ecipher.doFinal(bytes);

		return encode(encrypted);
	}

	/**
	 * Encrypts a plain text using the given encoded key and iv. This method defaults to the algorithm <code>AES/CBC/PKCS5Padding</code>
	 *
	 * @param plainText plain text
	 * @param encodedPassPhrase the encoded pass phrase
	 * @param iv the iv
	 * @return encoded cipher text
	 * @throws BadPaddingException 
	 * @throws IllegalBlockSizeException 
	 * @throws InvalidAlgorithmParameterException 
	 * @throws NoSuchPaddingException 
	 * @throws NoSuchAlgorithmException 
	 * @throws UnsupportedEncodingException 
	 * @throws InvalidKeyException 
	 * @throws EncryptionException the encryption exception
	 * @see {@link #encode(byte[])}
	 */
	public static String encrypt(String plainText, String encodedPassPhrase, String iv) throws InvalidKeyException, UnsupportedEncodingException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException {
		return encrypt(plainText, encodedPassPhrase, iv, DEFAULT_ENCRYPTION_ALGORITHM);
	}

	/**
	 * Decrypts a text using the encryptedText, key, iv and algorithm.
	 *
	 * @param encryptedText plain text
	 * @param encodedPassPhrase the encoded pass phrase
	 * @param iv the iv
	 * @param algorithm the algorithm
	 * @return plain text
	 * @throws NoSuchPaddingException 
	 * @throws NoSuchAlgorithmException 
	 * @throws BadPaddingException 
	 * @throws IllegalBlockSizeException 
	 * @throws InvalidAlgorithmParameterException 
	 * @throws InvalidKeyException 
	 * @throws UnsupportedEncodingException 
	 * @throws EncryptionException the encryption exception
	 * @see {@link #decode(String)}
	 */
	public static String decrypt(String encryptedText, String encodedPassPhrase, String iv, String algorithm) throws NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidKeyException, InvalidAlgorithmParameterException, UnsupportedEncodingException {
		if (encryptedText == null) {
			throw new NullPointerException();
		}
		byte[] bytes = decode(encryptedText);

		SecretKey secret = new SecretKeySpec(decode(encodedPassPhrase), "AES");

		// IV
		IvParameterSpec ivSpec = null;
		if (iv != null) {
			ivSpec = new IvParameterSpec(decode(iv));
		}

		Cipher dcipher = Cipher.getInstance(algorithm);
		dcipher.init(Cipher.DECRYPT_MODE, secret, ivSpec);

		byte[] decrypted = dcipher.doFinal(bytes);

		return new String(decrypted, TEXT_ENCODING);		
	}

	/**
	 * Decrypts a text using the encryptedText, key, iv and algorithm.
	 *
	 * @param encryptedText plain text
	 * @param encodedPassPhrase the encoded pass phrase
	 * @param iv the iv
	 * @param algorithm the algorithm
	 * @return plain text
	 * @throws NoSuchPaddingException 
	 * @throws NoSuchAlgorithmException 
	 * @throws InvalidAlgorithmParameterException 
	 * @throws InvalidKeyException 
	 * @throws BadPaddingException 
	 * @throws IllegalBlockSizeException 
	 * @throws UnsupportedEncodingException 
	 * @throws EncryptionException the encryption exception
	 * @see {@link #decode(String)}
	 */
	public static String decrypt(byte[] encryptedText, byte[] encodedPassPhrase, byte[] iv, String algorithm) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException, UnsupportedEncodingException {
		if (encryptedText == null) {
			throw new NullPointerException();
		}

		SecretKey secret = new SecretKeySpec(encodedPassPhrase, "AES");

		// IV
		IvParameterSpec ivSpec = null;
		if (iv != null) {
			ivSpec = new IvParameterSpec(iv);
		}

		Cipher dcipher = Cipher.getInstance(algorithm);
		dcipher.init(Cipher.DECRYPT_MODE, secret, ivSpec);

		byte[] decrypted = dcipher.doFinal(encryptedText);

		return new String(decrypted, TEXT_ENCODING);
	}

	/**
	 * Decrypts a text using the encryptedText, key and iv. This method defaults to the algorithm <code>AES/CBC/PKCS5Padding</code>
	 *
	 * @param encryptedText plain text
	 * @param encodedPassPhrase the encoded pass phrase
	 * @param iv the iv
	 * @return decoded plain text
	 * @throws UnsupportedEncodingException 
	 * @throws InvalidAlgorithmParameterException 
	 * @throws BadPaddingException 
	 * @throws IllegalBlockSizeException 
	 * @throws NoSuchPaddingException 
	 * @throws NoSuchAlgorithmException 
	 * @throws InvalidKeyException 
	 * @throws EncryptionException the encryption exception
	 * @see {@link #decode(String)}
	 */
	public static String decrypt(String encryptedText, String encodedPassPhrase, String iv) throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, UnsupportedEncodingException {
		return decrypt(encryptedText, encodedPassPhrase, iv, DEFAULT_ENCRYPTION_ALGORITHM);
	}

	/**
	 * The main method.
	 *
	 * @param args the arguments
	 * @throws Exception the exception
	 */
	public static void main(String[] args) throws Exception {

		System.out.println("KEY: " + generateRandomBytesAsBase64(16));

		// String key = args[0];
		// String data = args[1];
		String key = "HV9DYROgzE850PBuT1yiyg==";
		// String data = "01000089"; // AGB code voor EasyCare Query WS
		String data = "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";

		String iv = "slL9Z72zkRmWyRDUukX_cA==";

		if (args.length > 2) {
			iv = args[2];
		} else {
			//iv = generateRandomPassword(16);
		}

		System.out.println("key: " + key);
		System.out.println("Data: " + data);
		System.out.println("Generated IV: " + iv);

		System.out.println("Result: " + decrypt(data, key, iv));

		//		if (args.length < 4 || args[3].equalsIgnoreCase("encrypt")) {
		//			// encrypt
		//			data = System.currentTimeMillis() + data;
		//			System.out.println("Text with timestamp: " + data);
		//			System.out.println("Result: " + encrypt(data, key, iv));
		//		} else {
		//			// decrypt
		//			System.out.println("Result: " + decrypt(data, key, iv));
		//		}
	}
}