package com.perdemia.permissionanalyzer.application.util;

import java.security.SecureRandom;
import java.util.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The Class AbstractEncryptionHelper.
 */
public abstract class AbstractEncryptionHelper {

	private static Logger logger = LoggerFactory.getLogger(AbstractEncryptionHelper.class.getName());

	/** The Constant secureRandom. */
	protected final static SecureRandom secureRandom = new SecureRandom();

	/** The Constant TEXT_ENCODING. */
	protected final static String TEXT_ENCODING = "UTF8";

	/**
	 * Encodes the given byte array to a base64 String. Besides Base64 the characters '+' and '/' are replaced with '-' and '_' to make the
	 * String suitable for URL's.
	 *
	 * @param byteArray bytes to be encoded
	 * @return Base64 String
	 *
	 * @throws Exception
	 */
	public static String encode(byte[] byteArray) {
		String encoded = Base64.getEncoder().encodeToString(byteArray);
		return encoded.replace('+', '-').replace('/', '_');
	}

	/**
	 * Decodes the given String to a byte array using Base64 decoding. Besides Base64 the characters '-' and '_' are replaced with '+' and '/'
	 *
	 * @param byteArray bytes to be encoded
	 * @return Byte array
	 *
	 * @throws Exception
	 */
	public static byte[] decode(String text) {
		text = text.replace('-', '+').replace('_', '/');
		
		return Base64.getDecoder().decode(text);
	}

	/**
	 * Helper method to create a random password or initialization vector. Random bytes are created and the Base64 encoded (and URL encoded)
	 * representation is returned.
	 *
	 * @param byteLength for example 16 in case of AES 128bit encryption.
	 * @return random String
	 */
	public static String generateRandomPassword(int byteLength) {
		return generateRandomBytesAsBase64(byteLength);
	}

	/**
	 * Helper method to create a random password or initialization vector. Random bytes are created and the Base64 encoded (and URL encoded)
	 * representation is returned.
	 *
	 * @param byteLength for example 16 in case of AES 128bit encryption.
	 * @return random String
	 */
	public static String generateRandomBytesAsBase64(int byteLength) {
		byte[] password = new byte[byteLength];
		secureRandom.nextBytes(password);
		return encode(password);
	}

	/**
	 * Byte array to unsigned ints.
	 *
	 * @param bytes the bytes
	 * @return the string
	 */
	public static String byteArrayToUnsignedInts(byte[] bytes) {
		StringBuilder builder = new StringBuilder();
		for (byte b : bytes) {
			int unsigned = (b & 0xFF);
			builder.append(unsigned);
			builder.append(", ");
		}
		return builder.toString();
	}

	/**
	 * Byte array to hex.
	 *
	 * @param bytes the bytes
	 * @return the string
	 */
	public static String byteArrayToHex(byte[] bytes) {
		StringBuilder sb = new StringBuilder();
		for (byte b : bytes) {
			sb.append(String.format("%02X ", b));
		}
		return sb.toString();
	}
}
