package com.perdemia.permissionanalyzer.application.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.mail.ByteArrayDataSource;
import org.apache.commons.mail.DefaultAuthenticator;
import org.apache.commons.mail.EmailAttachment;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.HtmlEmail;
import org.eclipse.jface.dialogs.MessageDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ContactSupportDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.data.Report;

public final class EmailHelper {
	private static Logger logger = LoggerFactory.getLogger(EmailHelper.class.getName());
	
	public static void sendMail(String host, int port, String username, String password, boolean ssl, boolean startTls, String from, String subject, String msg, String htmlMsg, String to, Attachment attachment, boolean debug) throws EmailException {
		HtmlEmail email = new HtmlEmail();
		
		if (host == null || host.length() == 0) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "No SMTP server configured", "The e-mail could not be sent because there is no SMTP server configured in the preferenes.");
			logger.error("No SMTP server configured!");
			throw new EmailException("No SMTP server configured");
		}
		
		if (attachment != null) {
			//Create the attachment
			ByteArrayOutputStream bos = new ByteArrayOutputStream(); 
		    ZipOutputStream zos = new ZipOutputStream(bos);
		    FileInputStream fis = null;		    
		    byte[] buffer = new byte[1024];
		    
		    try {
		        // create a zip byte stream from the report file
		    	File srcFile = new File(attachment.getPath());
 				fis = new FileInputStream(srcFile);
 
 				// begin writing a new ZIP entry, positions the stream to the start of the entry data
 				zos.putNextEntry(new ZipEntry(srcFile.getName()));
 				
 				int length;
 				while ((length = fis.read(buffer)) > 0) {
					zos.write(buffer, 0, length);
				}

				zos.closeEntry();
				fis.close();
				zos.close();

				ByteArrayDataSource ds = new ByteArrayDataSource(bos.toByteArray(), "application/zip");
				
				// attach the zip
				email.attach(ds, attachment.getName() + ".zip", attachment.getDescription(), EmailAttachment.ATTACHMENT);
				
		    } catch (IOException ex) {
		    	logger.error("Error while zipping mail attachment, file: " + attachment.getPath(), ex);
		    	SyncMessageDialog.showMessage(MessageDialog.WARNING, "Attachment failed", "Error while zipping e-mail attachment: " + GUILogHandler.getNestedMessage(ex) + "\r\n\r\nThe e-mail will be sent without attachment.");
		    } finally {
		    	try {
		    		if (fis != null) fis.close();
					zos.close();
				} catch (IOException e) {}
		    }			
		}
		
		email.setHostName(host);
		email.setSmtpPort(port);
		email.setSslSmtpPort(""+port);
		
		email.setDebug(debug);
		
		if (username != null && username.length() > 0) {
			email.setAuthenticator(new DefaultAuthenticator(username, password));
		}
		
		email.setSSLOnConnect(ssl);
		email.setSSLCheckServerIdentity(false);
		email.setStartTLSEnabled(startTls);
		email.setFrom(from);
		email.setSubject(subject);
		email.setTextMsg(msg);
		email.setHtmlMsg("<html><body>" + htmlMsg.replace("\r\n", "<br>") + "</body></html>");
		email.addTo(to.split(","));
		email.send();
	}
		
	/**
	 * Sends an e-mail based on the SMTP information in the preferences and the given Report.
	 * 
	 * @param subject
	 * @param msg
	 * @param to
	 * @throws EmailException
	 */
	public static void sendMail(String subject, String msg, Report report) throws EmailException {
		PreferenceHelper prefs = PreferenceHelper.getInstance();
		Attachment attachment = null;
		
		if (PreferenceHelper.getInstance().isSmtpAttachReports() && report.getLastFilePath() != null && !report.getLastFilePath().isEmpty()) {
			attachment = new Attachment();
			attachment.setName(report.getName());
			attachment.setDescription(report.getDescription());
			attachment.setPath(report.getLastFilePath());
		}
		
		String htmlMsg = msg;
		if (report != null) {
			msg = msg.replace("[report_name]", report.getName());
			msg = msg.replace("[report_threshold]", String.valueOf(report.getEmailThreshold()));
			msg = msg.replace("[report_description]", report.getDescription());
			
			htmlMsg = msg;
			msg = msg.replace("[report_path]", report.getLastFilePath());
			htmlMsg = htmlMsg.replace("[report_path]", "<a href=\"file://" + report.getLastFilePath() + "\" target=\"_blank\">" + report.getLastFilePath() + "</a>");
						
			subject = subject.replace("[report_name]", report.getName());
			subject = subject.replace("[report_path]", report.getLastFilePath());
			subject = subject.replace("[report_threshold]", String.valueOf(report.getEmailThreshold()));
			subject = subject.replace("[report_description]", report.getDescription());			
		}
		
		sendMail(prefs.getSmtpHost(), prefs.getSmtpPort(), prefs.getSmtpUsername(), prefs.getSmtpPassword(), prefs.getSmtpType() == 1, prefs.getSmtpType() == 2, prefs.getSmtpFrom(), subject, msg, htmlMsg, report.getEmailAddress(), attachment, false);
	}
	
	/**
	 * Sends an e-mail based on the SMTP information in the preferences and a optional Attachment object.
	 * 
	 * @param subject
	 * @param msg
	 * @param to
	 * @throws EmailException
	 * @see {@link ContactSupportDialog#buttonPressed(Integer)}
	 */
	public static void sendMail(String subject, String msg, String from, String to, Attachment attachment) throws EmailException {
		PreferenceHelper prefs = PreferenceHelper.getInstance();
		
		sendMail(prefs.getSmtpHost(), prefs.getSmtpPort(), prefs.getSmtpUsername(), prefs.getSmtpPassword(), prefs.getSmtpType() == 1, prefs.getSmtpType() == 2, from, subject, msg, msg, to, attachment, false);
	}
}
