package com.perdemia.permissionanalyzer.application.util;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.Activator;

/***************************************************************************************
 * Maintains all the images using a Factory pattern and the <code>ImageRegistry</code> of JFace.
 * 
 * @author Reinier L'abee
 * @see ImageRegistry
 * @see http://codeandme.blogspot.co.at/2012/07/reusing-platform-images.html
 ***************************************************************************************/

public class ImageFactory {
	private static Logger logger = LoggerFactory.getLogger(ImageFactory.class.getName());
	
	/** The singleton instance. */
	private static ImageFactory instance;

	/** The image registry. */
	private ImageRegistry imageRegistry;
	
	/** Mapping of file extension to AppImage name (only extension part of the name) */
	public final static Map<String, String> FILE_EXTENSIONS;
	
	/**
	 * Disables public constructor (Singleton pattern).
	 */
	private ImageFactory() {
		init();
	}

	/**
	 * Controls the access to this class (Singleton pattern)
	 * 
	 * @return Instance of this class. If it doesn't exist, it will be created.
	 */
	public static ImageFactory getInstance() {
		if (instance == null)
			instance = new ImageFactory();
		return instance;
	}

	/** The enum that defines all the images. */
	public static enum AppImage {
		DENY_20X20, DENY_16X16, ALLOW_20X20, ALLOW_16X16, TEST, USER_INFO_16X16, USER_FILTER_16X16, USER_FILTER_48X48, USER_INFO_48X48, USER_FILTER_HEADER_40X32, SEARCH_16X16, INCLUDE_USERS_16X16, EXCLUDE_USERS_16X16, INCLUDE_GROUPS_16X16, EXCLUDE_GROUPS_16X16,  
		REMOVE_16X16, DELETE_16X16, SAVE_16X16, SELECTION_16X16, SELECTION_48X48, FILTER_20X20, FILTER_16X16, ADD_WILDCARD_16X16, FOLDER_16X16, FOLDER_USER_16X16, FOLDER_DISABLED_16X16, FOLDER_24X24, FOLDER_48X48,
		FOLDER_20X20, FOLDER_NONE_20X20, FOLDER_NONE_16X16, FOLDER_READ_20X20, FOLDER_EXECUTE_20X20, FOLDER_LIST_FOLDER_CONTENT_20X20, FOLDER_WRITE_20X20, FOLDER_DELETE_20X20, FOLDER_MODIFY_20X20, 
		FOLDER_FULL_20X20, USER_16X16, USER_20X20, GROUP_16X16, GROUP_20X20, GROUP_24X24, GROUP_48X48, LDAP_OU_16X16, LDAP_OU_24X24, LDAP_OU_48X48, INCLUDE_WILDCARD_16X16, EXCLUDE_WILDCARD_16X16, INCLUDE_LDAP_OU_16X16, EXCLUDE_LDAP_OU_16X16,
		PERM_READ_DATA_16X16, PERM_WRITE_DATA_16X16, PERM_APPEND_DATA_16X16, PERM_EXECUTE_16X16, PERM_DELETE_CHILD_16X16, PERM_DELETE_16X16, PERM_READ_ATTR_16X16, 
		PERM_WRITE_ATTR_16X16, PERM_READ_EXT_ATTR_16X16, PERM_WRITE_EXT_ATTR_16X16, PERM_READ_ACL_16X16, PERM_FILTER_HEADER_40X32, PERM_WRITE_ACL_16X16, PERM_TAKE_OWNERSHIP_16X16,
		PERM_READ_DATA_20X20, PERM_WRITE_DATA_20X20, PERM_APPEND_DATA_20X20, PERM_EXECUTE_20X20, PERM_DELETE_CHILD_20X20, PERM_DELETE_20X20, PERM_READ_ATTR_20X20, 
		PERM_WRITE_ATTR_20X20, PERM_READ_EXT_ATTR_20X20, PERM_WRITE_EXT_ATTR_20X20, PERM_READ_ACL_20X20, PERM_WRITE_ACL_20X20, PERM_TAKE_OWNERSHIP_20X20,
		PERM_READ_DATA_LIGHT_20X20, PERM_WRITE_DATA_LIGHT_20X20, PERM_APPEND_DATA_LIGHT_20X20, PERM_EXECUTE_LIGHT_20X20, PERM_DELETE_CHILD_LIGHT_20X20, PERM_DELETE_LIGHT_20X20, PERM_READ_ATTR_LIGHT_20X20, 
		PERM_WRITE_ATTR_LIGHT_20X20, PERM_READ_EXT_ATTR_LIGHT_20X20, PERM_WRITE_EXT_ATTR_LIGHT_20X20, PERM_READ_ACL_LIGHT_20X20, PERM_WRITE_ACL_LIGHT_20X20, PERM_TAKE_OWNERSHIP_LIGHT_20X20,
		EXPAND_ONE_LEVEL_20X20, COLLAPSE_ONE_LEVEL_20X20, EXPAND_ALL_20X20, COLLAPSE_ALL_20X20, TRACE_16X16, ACL_20X20, ACL_48X48, GLOBE_16X16, NETWORK_32X32, INTERNET_32X32, CONNECTION_24X24, CONNECTION_48X48,
		CONNECTION_16X16, CONNECTION_DISABLED_16X16, UPDATE_SERVICE_24X24, SETTINGS_16X16, EDIT_16X16, ADD_16X16, APPLY_16X16, TEST_16X16, SECURITY_32X32, DATABASE_32X32, DATABASE_24X24, ACCESS_CONTROL_24X24, ACCESS_CONTROL_16X16, ACCESS_CONTROL_48X48,
		BROWSE_16X16, FILL_DATABASE_16X16, FILL_DATABASE_48X48, STATUS_INFO_16X16, STATUS_TRACE_16X16, STATUS_WARNING_16X16, STATUS_ERROR_16X16, ANALYZE_FOLDERS_48X48_1, ANALYZE_FOLDERS_48X48_2, ANALYZE_FOLDERS_48X48_3, ANALYZE_FOLDERS_48X48_4,
		ANALYZE_USERS_48X48_1, ANALYZE_USERS_48X48_2, ANALYZE_USERS_48X48_3, ANALYZE_USERS_48X48_4, ANALYZE_DONE_48X48, ERROR_32X32, WARNING_32X32, OK_32X32, FOLDER_EXPLORER_16X16,
		FOLDER_READ_16X16, FOLDER_EXECUTE_16X16, FOLDER_LIST_FOLDER_CONTENT_16X16, FOLDER_WRITE_16X16, FOLDER_DELETE_16X16, FOLDER_MODIFY_16X16, FOLDER_FULL_16X16,
		INCLUDE_PERMISSIONS_16X16, EXCLUDE_PERMISSIONS_16X16, SELECT_PERMISSIONS_INTERSECT_16X16, SELECT_PERMISSIONS_UNION_16X16, CANCEL_SCAN_16X16, GROUP_MEMBERSHIP_20X20, GROUP_MEMBERSHIP_16X16,
		EXPORT_16X16, INHERITED_PERMISSIONS_20X20, INCLUDE_FOLDER_16X16, EXCLUDE_FOLDER_16X16, FOLDER_FILTER_HEADER_40X32, PREFERENCES_16X16, PREFERENCES_48X48,
		REPORT_16X16, REPORT_24X24, REPORT_48X48, CREATE_REPORT_48X48, TREE_16X16, LICENSE_16X16, LICENSE_48X48, VALID_16X16, REGISTRATION_24X24, NETWORK_SHARE_16X16, NETWORK_SHARE_48X48,
		INFO_48X48, APP_ICON_16X16, APP_ICON_48X48, LOGO, ADVANCED_32X32, ANALYZE_24X24, FIRST_USE_1, FIRST_USE_2, FIRST_USE_3, FIRST_USE_4, FIRST_USE_5, FIRST_USE_6, FIRST_USE_7, FIRST_USE_8, FIRST_USE_9, FIRST_USE_10,
		EMAIL_32X32, EMAIL_TEMPLATE_24X24, SERVER_24X24, RUN_16X16, RUN_48X48, BUILTIN_16X16, BUILTIN_20X20, SEARCH_USER_24X24, SEARCH_USER_16X16, FOLDER_LINK_16X16, LOAD_LIST_16X16, HELP_16X16, HELP_48X48,
		INHERITED_ALLOW_20X20, EXPLICIT_ALLOW_20X20, INHERITED_DENY_20X20, EXPLICIT_DENY_20X20, CLIPBOARD_16X16, TRASH_16X16, LOG_FILE_16X16, LOG_FILE_48X48, EXCLUDE_FILTER_20X20, MULTIPLE_SEARCH_16X16, INFO_16X16,
		CLOSE_EXPLAIN_LABEL_16X16, POLICY_16X16, POLICY_48X48, CREATE_POLICY_48X48, DATE_16X16, POLICY_STATUS_UNKNOWN_16X16, IMPORT_48X48, POWER_SHELL_16X16, POWER_SHELL_DISABLED_16X16, SERVER_16X16, SERVER_DISABLED_16X16,
		INCLUDE_OWNER_16X16, EXCLUDE_OWNER_16X16, AD_GROUPS_USERS_16X16, AD_GROUPS_USERS_20X20, SERVER_48X48, POWER_SHELL_48X48, TEXT_FILE_48X48, DISABLED_USER_20X20, INVERT_FOLDERS_24X24, BLOCK_ACL_INHERITANCE_24X24,
		EFFECTIVE_PERMISSIONS_16X16, INCLUDE_REPORT_16X16, EXCLUDE_REPORT_16X16, COMPARE_REPORT_16X16, COMPARE_REPORT_32X32, COMPARE_REPORT_48X48, REPORT_HISTORY_16X16, REPORT_HISTORY_32X32, REPORT_HISTORY_48X48,
		
		// FILE PERMISSIONS
		FILE_AUDIO_DELETE_20X20, FILE_AUDIO_EXECUTE_20X20, FILE_AUDIO_FULL_20X20, FILE_AUDIO_MODIFY_20X20, FILE_AUDIO_NONE_20X20, FILE_AUDIO_WRITE_20X20, FILE_AUDIO_READ_20X20, FILE_BIN_DELETE_20X20, 
		FILE_BIN_EXECUTE_20X20, FILE_BIN_FULL_20X20, FILE_BIN_MODIFY_20X20, FILE_BIN_NONE_20X20, FILE_BIN_WRITE_20X20, FILE_BIN_READ_20X20, FILE_CFC_DELETE_20X20, FILE_CFC_EXECUTE_20X20, FILE_CFC_FULL_20X20, 
		FILE_CFC_MODIFY_20X20, FILE_CFC_NONE_20X20, FILE_CFC_WRITE_20X20, FILE_CFC_READ_20X20, FILE_CFM_DELETE_20X20, FILE_CFM_EXECUTE_20X20, FILE_CFM_FULL_20X20, FILE_CFM_MODIFY_20X20, FILE_CFM_NONE_20X20, 
		FILE_CFM_WRITE_20X20, FILE_CFM_READ_20X20, FILE_CHM_DELETE_20X20, FILE_CHM_EXECUTE_20X20, FILE_CHM_FULL_20X20, FILE_CHM_MODIFY_20X20, FILE_CHM_NONE_20X20, FILE_CHM_WRITE_20X20, FILE_CHM_READ_20X20, FILE_CODE_DELETE_20X20, 
		FILE_CODE_EXECUTE_20X20, FILE_CODE_FULL_20X20, FILE_CODE_MODIFY_20X20, FILE_CODE_NONE_20X20, FILE_CODE_WRITE_20X20, FILE_CODE_READ_20X20, FILE_CONF_DELETE_20X20, FILE_CONF_EXECUTE_20X20, 
		FILE_CONF_FULL_20X20, FILE_CONF_MODIFY_20X20, FILE_CONF_NONE_20X20, FILE_CONF_WRITE_20X20, FILE_CONF_READ_20X20, FILE_CSV_DELETE_20X20, FILE_CSV_EXECUTE_20X20, FILE_CSV_FULL_20X20, FILE_CSV_MODIFY_20X20, 
		FILE_CSV_NONE_20X20, FILE_CSV_WRITE_20X20, FILE_CSV_READ_20X20, FILE_DOC_DELETE_20X20, FILE_DOC_EXECUTE_20X20, FILE_DOC_FULL_20X20, FILE_DOC_MODIFY_20X20, FILE_DOC_NONE_20X20, FILE_DOC_WRITE_20X20, FILE_DOC_READ_20X20,
		FILE_EML_DELETE_20X20, FILE_EML_EXECUTE_20X20, FILE_EML_FULL_20X20, FILE_EML_MODIFY_20X20, FILE_EML_NONE_20X20, FILE_EML_WRITE_20X20, FILE_EML_READ_20X20, FILE_FILE_DELETE_20X20, FILE_FILE_EXECUTE_20X20, 
		FILE_FILE_FULL_20X20, FILE_FILE_MODIFY_20X20, FILE_FILE_NONE_20X20, FILE_FILE_WRITE_20X20, FILE_FILE_READ_20X20, FILE_HLP_DELETE_20X20, FILE_HLP_EXECUTE_20X20, FILE_HLP_FULL_20X20, FILE_HLP_MODIFY_20X20, 
		FILE_HLP_NONE_20X20, FILE_HLP_WRITE_20X20, FILE_HLP_READ_20X20, FILE_HTML_DELETE_20X20, FILE_HTML_EXECUTE_20X20, FILE_HTML_FULL_20X20, FILE_HTML_MODIFY_20X20, FILE_HTML_NONE_20X20, FILE_HTML_WRITE_20X20, FILE_HTML_READ_20X20,
		FILE_IMAGE_DELETE_20X20, FILE_IMAGE_EXECUTE_20X20, FILE_IMAGE_FULL_20X20, FILE_IMAGE_MODIFY_20X20, FILE_IMAGE_NONE_20X20, FILE_IMAGE_WRITE_20X20, FILE_IMAGE_READ_20X20, FILE_ISO_DELETE_20X20, 
		FILE_ISO_EXECUTE_20X20, FILE_ISO_FULL_20X20, FILE_ISO_MODIFY_20X20, FILE_ISO_NONE_20X20, FILE_ISO_WRITE_20X20, FILE_ISO_READ_20X20, FILE_ODC_DELETE_20X20, FILE_ODC_EXECUTE_20X20, FILE_ODC_FULL_20X20, 
		FILE_ODC_MODIFY_20X20, FILE_ODC_NONE_20X20, FILE_ODC_WRITE_20X20, FILE_ODC_READ_20X20, FILE_ODF_DELETE_20X20, FILE_ODF_EXECUTE_20X20, FILE_ODF_FULL_20X20, FILE_ODF_MODIFY_20X20, FILE_ODF_NONE_20X20,
		FILE_ODF_WRITE_20X20, FILE_ODF_READ_20X20, FILE_ODG_DELETE_20X20, FILE_ODG_EXECUTE_20X20, FILE_ODG_FULL_20X20, FILE_ODG_MODIFY_20X20, FILE_ODG_NONE_20X20, FILE_ODG_WRITE_20X20, FILE_ODG_READ_20X20, FILE_ODI_DELETE_20X20, 
		FILE_ODI_EXECUTE_20X20, FILE_ODI_FULL_20X20, FILE_ODI_MODIFY_20X20, FILE_ODI_NONE_20X20, FILE_ODI_WRITE_20X20, FILE_ODI_READ_20X20, FILE_ODP_DELETE_20X20, FILE_ODP_EXECUTE_20X20, FILE_ODP_FULL_20X20, 
		FILE_ODP_MODIFY_20X20, FILE_ODP_NONE_20X20, FILE_ODP_WRITE_20X20, FILE_ODP_READ_20X20, FILE_ODS_DELETE_20X20, FILE_ODS_EXECUTE_20X20, FILE_ODS_FULL_20X20, FILE_ODS_MODIFY_20X20, FILE_ODS_NONE_20X20, 
		FILE_ODS_WRITE_20X20, FILE_ODS_READ_20X20, FILE_ODT_DELETE_20X20, FILE_ODT_EXECUTE_20X20, FILE_ODT_FULL_20X20, FILE_ODT_MODIFY_20X20, FILE_ODT_NONE_20X20, FILE_ODT_WRITE_20X20, FILE_ODT_READ_20X20, FILE_PDF_DELETE_20X20, 
		FILE_PDF_EXECUTE_20X20, FILE_PDF_FULL_20X20, FILE_PDF_MODIFY_20X20, FILE_PDF_NONE_20X20, FILE_PDF_WRITE_20X20, FILE_PDF_READ_20X20, FILE_PPT_DELETE_20X20, FILE_PPT_EXECUTE_20X20, FILE_PPT_FULL_20X20, 
		FILE_PPT_MODIFY_20X20, FILE_PPT_NONE_20X20, FILE_PPT_WRITE_20X20, FILE_PPT_READ_20X20, FILE_PS_DELETE_20X20, FILE_PS_EXECUTE_20X20, FILE_PS_FULL_20X20, FILE_PS_MODIFY_20X20, FILE_PS_NONE_20X20, 
		FILE_PS_WRITE_20X20, FILE_PS_READ_20X20, FILE_RAR_DELETE_20X20, FILE_RAR_EXECUTE_20X20, FILE_RAR_FULL_20X20, FILE_RAR_MODIFY_20X20, FILE_RAR_NONE_20X20, FILE_RAR_WRITE_20X20, FILE_RAR_READ_20X20, FILE_SQL_DELETE_20X20,
		FILE_SQL_EXECUTE_20X20, FILE_SQL_FULL_20X20, FILE_SQL_MODIFY_20X20, FILE_SQL_NONE_20X20, FILE_SQL_WRITE_20X20, FILE_SQL_READ_20X20, FILE_SWF_DELETE_20X20, FILE_SWF_EXECUTE_20X20, FILE_SWF_FULL_20X20,
		FILE_SWF_MODIFY_20X20, FILE_SWF_NONE_20X20, FILE_SWF_WRITE_20X20, FILE_SWF_READ_20X20, FILE_SXC_DELETE_20X20, FILE_SXC_EXECUTE_20X20, FILE_SXC_FULL_20X20, FILE_SXC_MODIFY_20X20, FILE_SXC_NONE_20X20, 
		FILE_SXC_WRITE_20X20, FILE_SXC_READ_20X20, FILE_SXD_DELETE_20X20, FILE_SXD_EXECUTE_20X20, FILE_SXD_FULL_20X20, FILE_SXD_MODIFY_20X20, FILE_SXD_NONE_20X20, FILE_SXD_WRITE_20X20, FILE_SXD_READ_20X20, FILE_SXI_DELETE_20X20, 
		FILE_SXI_EXECUTE_20X20, FILE_SXI_FULL_20X20, FILE_SXI_MODIFY_20X20, FILE_SXI_NONE_20X20, FILE_SXI_WRITE_20X20, FILE_SXI_READ_20X20, FILE_SXW_DELETE_20X20, FILE_SXW_EXECUTE_20X20, FILE_SXW_FULL_20X20, 
		FILE_SXW_MODIFY_20X20, FILE_SXW_NONE_20X20, FILE_SXW_WRITE_20X20, FILE_SXW_READ_20X20, FILE_TAR_DELETE_20X20, FILE_TAR_EXECUTE_20X20, FILE_TAR_FULL_20X20, FILE_TAR_MODIFY_20X20, FILE_TAR_NONE_20X20, 
		FILE_TAR_WRITE_20X20, FILE_TAR_READ_20X20, FILE_TXT_DELETE_20X20, FILE_TXT_EXECUTE_20X20, FILE_TXT_FULL_20X20, FILE_TXT_MODIFY_20X20, FILE_TXT_NONE_20X20, FILE_TXT_NONE_16X16, FILE_TXT_WRITE_20X20, FILE_TXT_READ_20X20, FILE_TXT_READ_16X16, FILE_VCF_DELETE_20X20, 
		FILE_VCF_EXECUTE_20X20, FILE_VCF_FULL_20X20, FILE_VCF_MODIFY_20X20, FILE_VCF_NONE_20X20, FILE_VCF_WRITE_20X20, FILE_VCF_READ_20X20, FILE_VIDEO_DELETE_20X20, FILE_VIDEO_EXECUTE_20X20, FILE_VIDEO_FULL_20X20, 
		FILE_VIDEO_MODIFY_20X20, FILE_VIDEO_NONE_20X20, FILE_VIDEO_WRITE_20X20, FILE_VIDEO_READ_20X20, FILE_VSD_DELETE_20X20, FILE_VSD_EXECUTE_20X20, FILE_VSD_FULL_20X20, FILE_VSD_MODIFY_20X20, FILE_VSD_NONE_20X20, 
		FILE_VSD_WRITE_20X20, FILE_VSD_READ_20X20, FILE_XLS_DELETE_20X20, FILE_XLS_EXECUTE_20X20, FILE_XLS_FULL_20X20, FILE_XLS_MODIFY_20X20, FILE_XLS_NONE_20X20, FILE_XLS_WRITE_20X20, FILE_XLS_READ_20X20, FILE_XML_DELETE_20X20,
		FILE_XML_EXECUTE_20X20, FILE_XML_FULL_20X20, FILE_XML_MODIFY_20X20, FILE_XML_NONE_20X20, FILE_XML_WRITE_20X20, FILE_XML_READ_20X20, FILE_ZIP_DELETE_20X20, FILE_ZIP_EXECUTE_20X20, FILE_ZIP_FULL_20X20, 
		FILE_ZIP_MODIFY_20X20, FILE_ZIP_NONE_20X20, FILE_ZIP_WRITE_20X20, FILE_ZIP_READ_20X20
	}

	/**
	 * initialize the image descriptor.
	 */
	private void init() {
		imageRegistry = new ImageRegistry();
		
		/** General icons */
		imageRegistry.put(AppImage.DENY_20X20.toString(), getDescriptorFromFile("images/png/Deny20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.DENY_16X16.toString(), getDescriptorFromFile("images/png/Deny16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ALLOW_20X20.toString(), getDescriptorFromFile("images/png/Allow20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ALLOW_16X16.toString(), getDescriptorFromFile("images/png/Allow16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.TEST.toString(), getDescriptorFromFile("images/test.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_INFO_16X16.toString(), getDescriptorFromFile("images/png/UserInfo16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_FILTER_16X16.toString(), getDescriptorFromFile("images/png/UserFilter16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_FILTER_48X48.toString(), getDescriptorFromFile("images/png/UserFilter48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_INFO_48X48.toString(), getDescriptorFromFile("images/png/UserInfo48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_FILTER_HEADER_40X32.toString(), getDescriptorFromFile("images/png/UserFilterHeader40x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_FILTER_HEADER_40X32.toString(), getDescriptorFromFile("images/png/FolderFilterHeader40x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SEARCH_16X16.toString(), getDescriptorFromFile("images/png/Search16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_USERS_16X16.toString(), getDescriptorFromFile("images/png/IncludeUsers16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_USERS_16X16.toString(), getDescriptorFromFile("images/png/ExcludeUsers16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_GROUPS_16X16.toString(), getDescriptorFromFile("images/png/IncludeGroups16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_GROUPS_16X16.toString(), getDescriptorFromFile("images/png/ExcludeGroups16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REMOVE_16X16.toString(), getDescriptorFromFile("images/png/Remove16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.DELETE_16X16.toString(), getDescriptorFromFile("images/png/Delete16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SAVE_16X16.toString(), getDescriptorFromFile("images/png/Save16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SELECTION_16X16.toString(), getDescriptorFromFile("images/png/Selection16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SELECTION_48X48.toString(), getDescriptorFromFile("images/png/Selection48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILTER_20X20.toString(), getDescriptorFromFile("images/png/Filter20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILTER_16X16.toString(), getDescriptorFromFile("images/png/Filter16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ADD_WILDCARD_16X16.toString(), getDescriptorFromFile("images/png/AddWildcard16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_16X16.toString(), getDescriptorFromFile("images/png/User16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.USER_20X20.toString(), getDescriptorFromFile("images/png/User20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GROUP_16X16.toString(), getDescriptorFromFile("images/png/Group16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GROUP_20X20.toString(), getDescriptorFromFile("images/png/Group20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GROUP_24X24.toString(), getDescriptorFromFile("images/png/Group24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GROUP_48X48.toString(), getDescriptorFromFile("images/png/Group48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_WILDCARD_16X16.toString(), getDescriptorFromFile("images/png/IncludeWildcard16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_WILDCARD_16X16.toString(), getDescriptorFromFile("images/png/ExcludeWildcard16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LDAP_OU_16X16.toString(), getDescriptorFromFile("images/png/LdapOU16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LDAP_OU_24X24.toString(), getDescriptorFromFile("images/png/LdapOU24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LDAP_OU_48X48.toString(), getDescriptorFromFile("images/png/LdapOU48x48.png")); //$NON-NLS-1$		
		imageRegistry.put(AppImage.INCLUDE_LDAP_OU_16X16.toString(), getDescriptorFromFile("images/png/IncludeLdapOU16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_LDAP_OU_16X16.toString(), getDescriptorFromFile("images/png/ExcludeLdapOU16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXPAND_ONE_LEVEL_20X20.toString(), getDescriptorFromFile("images/png/ExpandOneLevel20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.COLLAPSE_ONE_LEVEL_20X20.toString(), getDescriptorFromFile("images/png/CollapseOneLevel20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXPAND_ALL_20X20.toString(), getDescriptorFromFile("images/png/ExpandAll20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.COLLAPSE_ALL_20X20.toString(), getDescriptorFromFile("images/png/CollapseAll20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.TRACE_16X16.toString(), getDescriptorFromFile("images/png/Trace16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ACL_20X20.toString(), getDescriptorFromFile("images/png/Acl20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ACL_48X48.toString(), getDescriptorFromFile("images/png/Acl48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GLOBE_16X16.toString(), getDescriptorFromFile("images/png/Globe16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.NETWORK_32X32.toString(), getDescriptorFromFile("images/png/Network32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INTERNET_32X32.toString(), getDescriptorFromFile("images/png/Internet32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CONNECTION_24X24.toString(), getDescriptorFromFile("images/png/Connection24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CONNECTION_16X16.toString(), getDescriptorFromFile("images/png/Connection16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CONNECTION_48X48.toString(), getDescriptorFromFile("images/png/Connection48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CONNECTION_DISABLED_16X16.toString(), getDescriptorFromFile("images/png/ConnectionDisabled16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.UPDATE_SERVICE_24X24.toString(), getDescriptorFromFile("images/png/UpdateService24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SETTINGS_16X16.toString(), getDescriptorFromFile("images/png/Settings16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EDIT_16X16.toString(), getDescriptorFromFile("images/png/Edit16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ADD_16X16.toString(), getDescriptorFromFile("images/png/Add16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.APPLY_16X16.toString(), getDescriptorFromFile("images/png/Apply16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.TEST_16X16.toString(), getDescriptorFromFile("images/png/Test16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SECURITY_32X32.toString(), getDescriptorFromFile("images/png/Security32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.DATABASE_32X32.toString(), getDescriptorFromFile("images/png/Database32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.DATABASE_24X24.toString(), getDescriptorFromFile("images/png/Database24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ACCESS_CONTROL_24X24.toString(), getDescriptorFromFile("images/png/AccessControl24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ACCESS_CONTROL_16X16.toString(), getDescriptorFromFile("images/png/AccessControl16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ACCESS_CONTROL_48X48.toString(), getDescriptorFromFile("images/png/AccessControl48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.BROWSE_16X16.toString(), getDescriptorFromFile("images/png/Browse16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILL_DATABASE_16X16.toString(), getDescriptorFromFile("images/png/FillDatabase16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILL_DATABASE_48X48.toString(), getDescriptorFromFile("images/png/FillDatabase48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.STATUS_INFO_16X16.toString(), getDescriptorFromFile("images/png/StatusInfo16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.STATUS_TRACE_16X16.toString(), getDescriptorFromFile("images/png/StatusTrace16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.STATUS_WARNING_16X16.toString(), getDescriptorFromFile("images/png/StatusWarning16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.STATUS_ERROR_16X16.toString(), getDescriptorFromFile("images/png/StatusError16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_FOLDERS_48X48_1.toString(), getDescriptorFromFile("images/png/AnalyzeFolders48x48_1.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_FOLDERS_48X48_2.toString(), getDescriptorFromFile("images/png/AnalyzeFolders48x48_2.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_FOLDERS_48X48_3.toString(), getDescriptorFromFile("images/png/AnalyzeFolders48x48_3.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_FOLDERS_48X48_4.toString(), getDescriptorFromFile("images/png/AnalyzeFolders48x48_4.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_USERS_48X48_1.toString(), getDescriptorFromFile("images/png/AnalyzeUsers48x48_1.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_USERS_48X48_2.toString(), getDescriptorFromFile("images/png/AnalyzeUsers48x48_2.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_USERS_48X48_3.toString(), getDescriptorFromFile("images/png/AnalyzeUsers48x48_3.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_USERS_48X48_4.toString(), getDescriptorFromFile("images/png/AnalyzeUsers48x48_4.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_DONE_48X48.toString(), getDescriptorFromFile("images/png/AnalyzeDone48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ERROR_32X32.toString(), getDescriptorFromFile("images/png/Error32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.WARNING_32X32.toString(), getDescriptorFromFile("images/png/Warning32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.OK_32X32.toString(), getDescriptorFromFile("images/png/OK32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_EXPLORER_16X16.toString(), getDescriptorFromFile("images/png/FolderExplorer16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_PERMISSIONS_16X16.toString(), getDescriptorFromFile("images/png/IncludePermissions16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_PERMISSIONS_16X16.toString(), getDescriptorFromFile("images/png/ExcludePermissions16x16.png")); //$NON-NLS-1$		
		imageRegistry.put(AppImage.SELECT_PERMISSIONS_INTERSECT_16X16.toString(), getDescriptorFromFile("images/png/SelectPermissionsIntersect16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SELECT_PERMISSIONS_UNION_16X16.toString(), getDescriptorFromFile("images/png/SelectPermissionsUnion16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CANCEL_SCAN_16X16.toString(), getDescriptorFromFile("images/png/CancelScan16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GROUP_MEMBERSHIP_20X20.toString(), getDescriptorFromFile("images/png/GroupMembership20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.GROUP_MEMBERSHIP_16X16.toString(), getDescriptorFromFile("images/png/GroupMembership16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXPORT_16X16.toString(), getDescriptorFromFile("images/png/Export16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INHERITED_PERMISSIONS_20X20.toString(), getDescriptorFromFile("images/png/InheritedPermissions20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_FOLDER_16X16.toString(), getDescriptorFromFile("images/png/IncludeFolder16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_FOLDER_16X16.toString(), getDescriptorFromFile("images/png/ExcludeFolder16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PREFERENCES_16X16.toString(), getDescriptorFromFile("images/png/Preferences16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PREFERENCES_48X48.toString(), getDescriptorFromFile("images/png/Preferences48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REPORT_16X16.toString(), getDescriptorFromFile("images/png/Report16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REPORT_24X24.toString(), getDescriptorFromFile("images/png/Report24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REPORT_48X48.toString(), getDescriptorFromFile("images/png/Report48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CREATE_REPORT_48X48.toString(), getDescriptorFromFile("images/png/CreateReport48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.POLICY_16X16.toString(), getDescriptorFromFile("images/png/Policy16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.POLICY_48X48.toString(), getDescriptorFromFile("images/png/Policy48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CREATE_POLICY_48X48.toString(), getDescriptorFromFile("images/png/CreatePolicy48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.TREE_16X16.toString(), getDescriptorFromFile("images/png/Tree16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LICENSE_16X16.toString(), getDescriptorFromFile("images/png/License16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LICENSE_48X48.toString(), getDescriptorFromFile("images/png/License48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.VALID_16X16.toString(), getDescriptorFromFile("images/png/Valid16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REGISTRATION_24X24.toString(), getDescriptorFromFile("images/png/Registration24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.NETWORK_SHARE_16X16.toString(), getDescriptorFromFile("images/png/NetworkShare16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.NETWORK_SHARE_48X48.toString(), getDescriptorFromFile("images/png/NetworkShare48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INFO_48X48.toString(), getDescriptorFromFile("images/png/Info48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.APP_ICON_48X48.toString(), getDescriptorFromFile("images/png/AppIcon48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.APP_ICON_16X16.toString(), getDescriptorFromFile("images/png/AppIcon16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LOGO.toString(), getDescriptorFromFile("images/png/Logo.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ADVANCED_32X32.toString(), getDescriptorFromFile("images/png/Advanced32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.ANALYZE_24X24.toString(), getDescriptorFromFile("images/png/Analyze24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_1.toString(), getDescriptorFromFile("images/png/FirstUse1.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_2.toString(), getDescriptorFromFile("images/png/FirstUse2.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_3.toString(), getDescriptorFromFile("images/png/FirstUse3.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_4.toString(), getDescriptorFromFile("images/png/FirstUse4.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_5.toString(), getDescriptorFromFile("images/png/FirstUse5.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_6.toString(), getDescriptorFromFile("images/png/FirstUse6.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_7.toString(), getDescriptorFromFile("images/png/FirstUse7.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_8.toString(), getDescriptorFromFile("images/png/FirstUse8.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_9.toString(), getDescriptorFromFile("images/png/FirstUse9.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FIRST_USE_10.toString(), getDescriptorFromFile("images/png/FirstUse10.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EMAIL_32X32.toString(), getDescriptorFromFile("images/png/Email32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EMAIL_TEMPLATE_24X24.toString(), getDescriptorFromFile("images/png/EmailTemplate24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SERVER_24X24.toString(), getDescriptorFromFile("images/png/Server24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.RUN_16X16.toString(), getDescriptorFromFile("images/png/Run16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.RUN_48X48.toString(), getDescriptorFromFile("images/png/Run48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.BUILTIN_16X16.toString(), getDescriptorFromFile("images/png/BuiltIn16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.BUILTIN_20X20.toString(), getDescriptorFromFile("images/png/BuiltIn20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SEARCH_USER_24X24.toString(), getDescriptorFromFile("images/png/SearchUser24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SEARCH_USER_16X16.toString(), getDescriptorFromFile("images/png/SearchUser16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_LINK_16X16.toString(), getDescriptorFromFile("images/png/FolderLink16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LOAD_LIST_16X16.toString(), getDescriptorFromFile("images/png/LoadList16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.HELP_16X16.toString(), getDescriptorFromFile("images/png/Help16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.HELP_48X48.toString(), getDescriptorFromFile("images/png/Help48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CLIPBOARD_16X16.toString(), getDescriptorFromFile("images/png/Clipboard16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.TRASH_16X16.toString(), getDescriptorFromFile("images/png/Trash16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LOG_FILE_16X16.toString(), getDescriptorFromFile("images/png/LogFile16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.LOG_FILE_48X48.toString(), getDescriptorFromFile("images/png/LogFile48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_FILTER_20X20.toString(), getDescriptorFromFile("images/png/ExcludeFilter20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.MULTIPLE_SEARCH_16X16.toString(), getDescriptorFromFile("images/png/MultipleSearch16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INFO_16X16.toString(), getDescriptorFromFile("images/png/Info16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.CLOSE_EXPLAIN_LABEL_16X16.toString(), getDescriptorFromFile("images/png/CloseExplainLabel16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.DATE_16X16.toString(), getDescriptorFromFile("images/png/Date16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.POLICY_STATUS_UNKNOWN_16X16.toString(), getDescriptorFromFile("images/png/PolicyStatusUnknown16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.IMPORT_48X48.toString(), getDescriptorFromFile("images/png/Import48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.POWER_SHELL_16X16.toString(), getDescriptorFromFile("images/png/PowerShell16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.POWER_SHELL_DISABLED_16X16.toString(), getDescriptorFromFile("images/png/PowerShellDisabled16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_OWNER_16X16.toString(), getDescriptorFromFile("images/png/IncludeOwner16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_OWNER_16X16.toString(), getDescriptorFromFile("images/png/ExcludeOwner16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SERVER_16X16.toString(), getDescriptorFromFile("images/png/Server16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SERVER_DISABLED_16X16.toString(), getDescriptorFromFile("images/png/ServerDisabled16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.AD_GROUPS_USERS_16X16.toString(), getDescriptorFromFile("images/png/ADGroupsAndUsers16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.AD_GROUPS_USERS_20X20.toString(), getDescriptorFromFile("images/png/ADGroupsAndUsers20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.SERVER_48X48.toString(), getDescriptorFromFile("images/png/Server48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.POWER_SHELL_48X48.toString(), getDescriptorFromFile("images/png/PowerShell48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.TEXT_FILE_48X48.toString(), getDescriptorFromFile("images/png/TextFile48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.DISABLED_USER_20X20.toString(), getDescriptorFromFile("images/png/DisabledUser20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INVERT_FOLDERS_24X24.toString(), getDescriptorFromFile("images/png/InvertFolders24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.BLOCK_ACL_INHERITANCE_24X24.toString(), getDescriptorFromFile("images/png/BlockAclInheritance24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EFFECTIVE_PERMISSIONS_16X16.toString(), getDescriptorFromFile("images/png/EffectivePermissions16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INCLUDE_REPORT_16X16.toString(), getDescriptorFromFile("images/png/IncludeReport16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXCLUDE_REPORT_16X16.toString(), getDescriptorFromFile("images/png/ExcludeReport16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.COMPARE_REPORT_16X16.toString(), getDescriptorFromFile("images/png/CompareReport16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.COMPARE_REPORT_32X32.toString(), getDescriptorFromFile("images/png/CompareReport32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.COMPARE_REPORT_48X48.toString(), getDescriptorFromFile("images/png/CompareReport48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REPORT_HISTORY_16X16.toString(), getDescriptorFromFile("images/png/ReportHistory16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REPORT_HISTORY_32X32.toString(), getDescriptorFromFile("images/png/ReportHistory32x32.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.REPORT_HISTORY_48X48.toString(), getDescriptorFromFile("images/png/ReportHistory48x48.png")); //$NON-NLS-1$
		
		
		/** special permissions 16X16 */
		imageRegistry.put(AppImage.PERM_READ_DATA_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/ReadData16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_DATA_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/WriteData16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_APPEND_DATA_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/AppendData16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_EXECUTE_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/Execute16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_DELETE_CHILD_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/DeleteChild16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_DELETE_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/Delete16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_ATTR_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/ReadAttributes16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_ATTR_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/WriteAttributes16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_EXT_ATTR_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/ReadExtendedAttributes16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_EXT_ATTR_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/WriteExtendedAttributes16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_ACL_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/ReadAcl16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_ACL_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/WriteAcl16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_TAKE_OWNERSHIP_16X16.toString(), getDescriptorFromFile("images/png/Permissions/16x16/TakeOwnership16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_FILTER_HEADER_40X32.toString(), getDescriptorFromFile("images/png/PermissionFilterHeader40x32.png")); //$NON-NLS-1$
		
		/** special permissions 20X20 */
		imageRegistry.put(AppImage.PERM_READ_DATA_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/ReadData20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_DATA_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/WriteData20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_APPEND_DATA_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/AppendData20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/Execute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_DELETE_CHILD_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/DeleteChild20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_DELETE_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/Delete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_ATTR_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/ReadAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_ATTR_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/WriteAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_EXT_ATTR_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/ReadExtendedAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_EXT_ATTR_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/WriteExtendedAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_ACL_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/ReadAcl20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_ACL_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/WriteAcl20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_TAKE_OWNERSHIP_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/TakeOwnership20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INHERITED_ALLOW_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/InheritedAllow20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXPLICIT_ALLOW_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/ExplicitAllow20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.INHERITED_DENY_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/InheritedDeny20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.EXPLICIT_DENY_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/ExplicitDeny20x20.png")); //$NON-NLS-1$
			
		/** special permissions 20X20 with a transparancy of 40% (used for implicit permissions) */
		imageRegistry.put(AppImage.PERM_READ_DATA_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/ReadData20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_DATA_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/WriteData20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_APPEND_DATA_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/AppendData20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_EXECUTE_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/Execute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_DELETE_CHILD_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/DeleteChild20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_DELETE_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/Delete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_ATTR_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/ReadAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_ATTR_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/WriteAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_EXT_ATTR_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/ReadExtendedAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_EXT_ATTR_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/WriteExtendedAttributes20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_READ_ACL_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/ReadAcl20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_WRITE_ACL_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/WriteAcl20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.PERM_TAKE_OWNERSHIP_LIGHT_20X20.toString(), getDescriptorFromFile("images/png/Permissions/20x20/light/TakeOwnership20x20.png")); //$NON-NLS-1$
		
		/** Folder icons */
		imageRegistry.put(AppImage.FOLDER_20X20.toString(), getDescriptorFromFile("images/png/Folder20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_16X16.toString(), getDescriptorFromFile("images/png/Folder16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_24X24.toString(), getDescriptorFromFile("images/png/Folder24x24.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_48X48.toString(), getDescriptorFromFile("images/png/Folder48x48.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_DISABLED_16X16.toString(), getDescriptorFromFile("images/png/FolderDisabled16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_USER_16X16.toString(), getDescriptorFromFile("images/png/FolderUser16x16.png")); //$NON-NLS-1$
		
		
		/** Folder permissions */
		imageRegistry.put(AppImage.FOLDER_NONE_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_READ_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_LIST_FOLDER_CONTENT_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderListFolderContent20x20.png")); //$NON-NLS-1$		
		imageRegistry.put(AppImage.FOLDER_WRITE_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_DELETE_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_FULL_20X20.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_READ_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderRead16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_EXECUTE_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderExecute16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_LIST_FOLDER_CONTENT_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderListFolderContent16x16.png")); //$NON-NLS-1$		
		imageRegistry.put(AppImage.FOLDER_WRITE_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderWrite16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_DELETE_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderDelete16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_MODIFY_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderModify16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_FULL_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderFull16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FOLDER_NONE_16X16.toString(), getDescriptorFromFile("images/png/Folder_permissions/FolderNone16x16.png")); //$NON-NLS-1$
		
		/** File permissions */
		imageRegistry.put(AppImage.FILE_AUDIO_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_AUDIO_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_AUDIO_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_AUDIO_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_AUDIO_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_AUDIO_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_AUDIO_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/audioRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_BIN_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/binRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFC_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfcRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CFM_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/cfmRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CHM_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/chmRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CODE_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/codeRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CONF_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/confRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_CSV_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/csvRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_DOC_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/docRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_EML_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/emlRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_FILE_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/fileRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HLP_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/hlpRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_HTML_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/htmlRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_IMAGE_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/imageRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ISO_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/isoRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODC_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odcRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODF_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odfRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODG_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odgRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODI_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odiRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODP_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odpRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODS_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odsRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ODT_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/odtRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PDF_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pdfRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PPT_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/pptRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_PS_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/psRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_RAR_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/rarRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SQL_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sqlRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SWF_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/swfRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXC_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxcRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXD_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxdRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXI_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxiRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_SXW_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/sxwRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TAR_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/tarRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_NONE_16X16.toString(), getDescriptorFromFile("images/png/File_permissions/txtNone16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/txtRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_TXT_READ_16X16.toString(), getDescriptorFromFile("images/png/File_permissions/txtRead16x16.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VCF_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vcfRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VIDEO_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/videoRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_VSD_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/vsdRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XLS_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xlsRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_XML_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/xmlRead20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_DELETE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipDelete20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_EXECUTE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipExecute20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_FULL_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipFull20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_MODIFY_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipModify20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_NONE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipNone20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_WRITE_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipWrite20x20.png")); //$NON-NLS-1$
		imageRegistry.put(AppImage.FILE_ZIP_READ_20X20.toString(), getDescriptorFromFile("images/png/File_permissions/zipRead20x20.png")); //$NON-NLS-1$
		
	}
	
	static {
		FILE_EXTENSIONS = new HashMap<String, String>();
		// images
		FILE_EXTENSIONS.put("jpg", "image");
		FILE_EXTENSIONS.put("jpeg", "image");
		FILE_EXTENSIONS.put("png", "image");
		FILE_EXTENSIONS.put("gif", "image");
		FILE_EXTENSIONS.put("bmp", "image");
		FILE_EXTENSIONS.put("tiff", "image");
		FILE_EXTENSIONS.put("dib", "image");
		FILE_EXTENSIONS.put("jpe", "image");
		FILE_EXTENSIONS.put("jfif", "image");
		FILE_EXTENSIONS.put("tif", "image");
		FILE_EXTENSIONS.put("ico", "image");
		
		// movies
		FILE_EXTENSIONS.put("avi", "video");
		FILE_EXTENSIONS.put("divx", "video");
		FILE_EXTENSIONS.put("mpg", "video");
		FILE_EXTENSIONS.put("mpeg", "video");
		FILE_EXTENSIONS.put("mov", "video");
		FILE_EXTENSIONS.put("xvid", "video");
		FILE_EXTENSIONS.put("wmv", "video");
		FILE_EXTENSIONS.put("mkv", "video");
		FILE_EXTENSIONS.put("mp4", "video");
		FILE_EXTENSIONS.put("vob", "video");
		FILE_EXTENSIONS.put("rec", "video");
		FILE_EXTENSIONS.put("vob", "video");
				
		// audio files
		FILE_EXTENSIONS.put("mp3", "audio");
		FILE_EXTENSIONS.put("wav", "audio");
		FILE_EXTENSIONS.put("aac", "audio");
		FILE_EXTENSIONS.put("ac3", "audio");
		FILE_EXTENSIONS.put("wma", "audio");
		
		// zip archives
		FILE_EXTENSIONS.put("7z", "tar");
		FILE_EXTENSIONS.put("tar", "tar");
		FILE_EXTENSIONS.put("tgz", "tar");
		FILE_EXTENSIONS.put("bz2", "tar");
		FILE_EXTENSIONS.put("gz", "tar");
		FILE_EXTENSIONS.put("jar", "tar");
		FILE_EXTENSIONS.put("war", "tar");
		FILE_EXTENSIONS.put("ear", "tar");
		FILE_EXTENSIONS.put("msi", "tar");
		FILE_EXTENSIONS.put("zip", "zip");
		FILE_EXTENSIONS.put("cab", "zip");
		FILE_EXTENSIONS.put("rar", "rar");
		
		// office formats
		FILE_EXTENSIONS.put("doc", "doc");
		FILE_EXTENSIONS.put("docx", "doc");
		FILE_EXTENSIONS.put("dot", "doc");
		FILE_EXTENSIONS.put("xls", "xls");
		FILE_EXTENSIONS.put("xlsx", "xls");
		FILE_EXTENSIONS.put("ppt", "ppt");
		FILE_EXTENSIONS.put("pptx", "ppt");
		FILE_EXTENSIONS.put("ptt", "ppt");
		FILE_EXTENSIONS.put("pttx", "ppt");
		FILE_EXTENSIONS.put("vsd", "vsd");
		FILE_EXTENSIONS.put("vsdx", "vsd");
		
		// open office
		FILE_EXTENSIONS.put("odp", "odp");
		FILE_EXTENSIONS.put("ods", "ods");
		FILE_EXTENSIONS.put("odt", "odt");
		FILE_EXTENSIONS.put("odc", "odc");
		FILE_EXTENSIONS.put("odf", "odf");
		FILE_EXTENSIONS.put("odg", "odg");
		FILE_EXTENSIONS.put("odi", "odi");

		FILE_EXTENSIONS.put("sxd", "sxd");
		FILE_EXTENSIONS.put("sxi", "sxi");
		FILE_EXTENSIONS.put("sxw", "sxw");
		FILE_EXTENSIONS.put("sxd", "sxd");
		FILE_EXTENSIONS.put("sxc", "sxc");
		
		// code files
		FILE_EXTENSIONS.put("java", "code");
		FILE_EXTENSIONS.put("php", "code");
		FILE_EXTENSIONS.put("pl", "code");
		FILE_EXTENSIONS.put("py", "code");
		FILE_EXTENSIONS.put("cs", "code");
		FILE_EXTENSIONS.put("c", "code");
		FILE_EXTENSIONS.put("js", "code");
		FILE_EXTENSIONS.put("cpp", "code");
		FILE_EXTENSIONS.put("cpp", "code");
		FILE_EXTENSIONS.put("dll", "code");
		FILE_EXTENSIONS.put("cmd", "code");
		FILE_EXTENSIONS.put("bat", "code");
		FILE_EXTENSIONS.put("reg", "code");
		FILE_EXTENSIONS.put("vbs", "code");
		FILE_EXTENSIONS.put("vb", "code");
		
		// config files
		FILE_EXTENSIONS.put("sys", "conf");
		FILE_EXTENSIONS.put("ocx", "conf");
		FILE_EXTENSIONS.put("cpl", "conf");
		FILE_EXTENSIONS.put("db", "conf");
		FILE_EXTENSIONS.put("msc", "conf");
		FILE_EXTENSIONS.put("inf", "conf");
		FILE_EXTENSIONS.put("config", "conf");
				
		// binary files
		FILE_EXTENSIONS.put("bin", "bin");
		FILE_EXTENSIONS.put("exe", "bin");
		FILE_EXTENSIONS.put("class", "bin");
		FILE_EXTENSIONS.put("dat", "bin");
		FILE_EXTENSIONS.put("com", "conf");
		FILE_EXTENSIONS.put("tlb", "conf");
		
		// text files
		FILE_EXTENSIONS.put("txt", "txt");
		FILE_EXTENSIONS.put("log", "txt");
		FILE_EXTENSIONS.put("ini", "txt");
		FILE_EXTENSIONS.put("properties", "txt");
		FILE_EXTENSIONS.put("manifest", "txt");
		FILE_EXTENSIONS.put("rtf", "txt");
		
		// other extensions
		FILE_EXTENSIONS.put("pdf", "pdf");
		FILE_EXTENSIONS.put("xml", "xml");
		FILE_EXTENSIONS.put("dtd", "xml");
		FILE_EXTENSIONS.put("xsd", "xml");
		FILE_EXTENSIONS.put("vcf", "vcf");
		FILE_EXTENSIONS.put("ps", "ps");
		FILE_EXTENSIONS.put("sql", "sql");
		FILE_EXTENSIONS.put("ddl", "sql");
		FILE_EXTENSIONS.put("swf", "swf");
		FILE_EXTENSIONS.put("iso", "iso");
		FILE_EXTENSIONS.put("eml", "eml");
		FILE_EXTENSIONS.put("hlp", "hlp");
		FILE_EXTENSIONS.put("html", "html");
		FILE_EXTENSIONS.put("htm", "html");
		FILE_EXTENSIONS.put("csv", "csv");
		FILE_EXTENSIONS.put("cfc", "cfc");
		FILE_EXTENSIONS.put("cfm", "cfm");
		FILE_EXTENSIONS.put("chm", "chm");		
	}

	/**
	 * @param path
	 * @return
	 */
	private ImageDescriptor getDescriptorFromFile(String path) {
		return ImageDescriptor.createFromURL(FileLocator.find(Activator.getContext().getBundle(), new Path(path), null));
	}

	/**
	 * Through using the key, get the image.
	 * 
	 * @param key
	 *            through the key, the image can be gotten from the image registry
	 * @return Image the wanted image
	 */
	public Image getImage(AppImage key) {
		return imageRegistry.get(key.toString());
	}
	
	/**
	 * Through using the key, get the image descriptor.
	 * 
	 * @param key
	 *            through the key, the image descriptor can be gotten from the image registry
	 * @return ImageDescriptor
	 */
	public ImageDescriptor getDescriptor(AppImage key) {
		return imageRegistry.getDescriptor(key.toString());
	}
	
	public Image mergeImages(AppImage image1, AppImage image2) {
		// load source images
		//Image imageSource1 = getImage(image1);
		//Image imageSource2 = getImage(image2);
		
		//BufferedImage image = ImageIO.read(new File(path, "image.png"));
		//BufferedImage overlay = ImageIO.read(new File(path, "overlay.png"));

		// create the new image, canvas size is the max. of both image sizes
		//int w = Math.max(imageSource1.getBounds().width, imageSource2.getBounds().width);
		//int h = Math.max(imageSource1.getBounds().height, imageSource2.getBounds().height);
		//BufferedImage combined = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);

		// paint both images, preserving the alpha channels
		//Graphics g = combined.getGraphics();
		//g.drawImage(imageSource1, 0, 0, null);
		//g.drawImage(imageSource2, 0, 0, null);
		
		//OverlayIcon resultIcon = new OverlayIcon(getDescriptor(image1), getDescriptor(image2), new Point(16, 16));   
	    //return resultIcon.createImage();
	    
		// Save as new image
		//ImageIO.write(combined, "PNG", new File(path, "combined.png"));
		
		return null;
	}
}