package com.perdemia.permissionanalyzer.application.util;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import jcifs.smb.NtlmPasswordAuthentication;
import jcifs.smb.SmbFile;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.NetworkSharePasswordDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.model.DirectoryInfo;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnectorImpl;

public final class NetworkShareHelper {
	private static Logger logger = LoggerFactory.getLogger(NetworkShareHelper.class.getName());
	
	private static int passwordDialogResult;
	private static NtlmPasswordAuthentication auth = null;
		
	/**
	 * Returns the network shares based on the given server name. It will first try to use the current Windows Credentials, if that fails it will
	 * ask for custom credentials and retrieves the shares using SMB protocol.
	 * 
	 * @param server
	 * @param dirInfo dirInfo to apply excludes
	 * @return
	 */
	public static List<String> getNetworkShares(String server, DirectoryInfo dirInfo, boolean useCustomAuthentication) {
		String domain = LdapConnectorImpl.getDomainName();
		String username = LdapConnectorImpl.getCurrentUsername();
		String password = null;
		
		List<String> shares = new ArrayList<String>();
		
		try {									
			if (useCustomAuthentication || (dirInfo != null && dirInfo.getServerLoginUsername() != null && !dirInfo.getServerLoginUsername().isEmpty())) {
				if (auth == null) {
					
					if (useCustomAuthentication) {
						final NetworkSharePasswordDialog passwordDialog = new NetworkSharePasswordDialog(null, domain, username);
						Display.getDefault().syncExec(new Runnable(){
							@Override
							public void run() {
								passwordDialogResult = passwordDialog.open();
							}
						});
						
						if (passwordDialogResult == IDialogConstants.OK_ID) {
							username = passwordDialog.getUser();
							password = passwordDialog.getPassword();
							domain = passwordDialog.getDomain();
							
							if (dirInfo != null) {
								dirInfo.setServerLoginUsername(username);
								dirInfo.setServerLoginPassword(password);
								dirInfo.setServerLoginDomain(domain);
								PreferenceHelper.getInstance().storePreferences(false);
							}
						} else {
							return shares;
						}
					} else if (dirInfo != null) {
						username = dirInfo.getServerLoginUsername();
						password = dirInfo.getServerLoginPassword();
						domain = dirInfo.getServerLoginDomain();
					}
					
					auth = new NtlmPasswordAuthentication(domain, username, password);
				}
				
				SmbFile file = new SmbFile("smb://" + server, auth);
				shares = new ArrayList<String>(Arrays.asList(file.list()));
			} else {
				// try the native Windows credentials
				Process process = Runtime.getRuntime().exec("net view \\\\" + server + " /all");

	            // Get input streams
	            BufferedReader stdInput = new BufferedReader(new InputStreamReader(process.getInputStream()));
				BufferedReader stdError = new BufferedReader(new InputStreamReader(process.getErrorStream()));            
	            		
	            // Read command standard output
	            String line;
	            int shareNameLength = 0;
	            
	            // skip the first line
	            stdInput.readLine();
	            		
	            while ((line = stdInput.readLine()) != null) {
	            	// ignore the first few lines
	            	if (line.startsWith("Share") || shareNameLength > 0 && !line.isEmpty() && !line.contains("-------------")) {
	            			if (shareNameLength == 0) {
            				// we now have the header of the table, determine the length of the share name column
	            				for (shareNameLength = 10; line.charAt(shareNameLength) == ' '; shareNameLength++);
	            				
	            			} else {
	            				// process this share
	            				if (line.length() >= shareNameLength) {
	            					shares.add(line.substring(0, shareNameLength).trim());
	            				} else {
	            					// may happen at the last line, where the message appears that the command has been executed
	            					// this item will be removed at the end of this loop!
	            					shares.add("NO SHARE");
	            				}
	            			}
	            		}
	            }
	            
	            // errors?
	            StringBuilder errors = new StringBuilder("");
	            while ((line = stdError.readLine()) != null) {
	            	errors.append(line).append(" ");
	            }
	            if (errors.length() > 0) {
	            	throw new Exception("Error while reading network shares for server " + server + ": " + errors.toString());
	            	//SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while reading network shares for server " + server + ": " + errors.toString());
	            }
	            
	            // always remove the last one because it's the message that the command has been executed
	            if (shares.size() > 0) {
	            	shares.remove(shares.size() - 1);
	            }
			}
		} catch (Exception ex) {
			logger.error("Error while scanning the server " + server + " for network shares.", ex);
			
			// try with custom credentials?
			if (!useCustomAuthentication) {
				// try again
				return getNetworkShares(server, dirInfo, true);
				
			} else {
				int result = SyncMessageDialog.showQuestionWithCancelMessage("Error", "Error while scanning the server " + server + " for network shares: \r\n\r\n" + GUILogHandler.getNestedMessage(ex) + ".\r\nSee the log file for more details.\r\n\r\nDo you want to skip this server?"
						, "Yes, skip this server", "No, ask for new username");
				if (result == IDialogConstants.NO_ID) {
					// try again
					auth = null;
					return getNetworkShares(server, dirInfo, true);
				}
			}
		}
		
		// apply excludes?
		if (dirInfo != null) {
			List<String> excludeList = Arrays.asList(dirInfo.getExcludeShares().toUpperCase().replace(", ", ",").split(","));
			Iterator<String> iter = shares.iterator();
			while (iter.hasNext()) {
				String share = iter.next();
				
				boolean isHiddenDrive = share.endsWith("$") && share.length() == 2;
				boolean includeHiddenDrives = ProgramParameters.isIncludeSharedDrives() || dirInfo.isIncludeHiddenSharedDrives();
				boolean excludeBecauseHiddenDrive = !includeHiddenDrives && isHiddenDrive;
				boolean excludeHiddenShare = !ProgramParameters.isIncludeHiddenShares() && !dirInfo.isIncludeHiddenShares() && share.endsWith("$") && !(isHiddenDrive && includeHiddenDrives);
				boolean isInExcludeList = ProgramParameters.getExcludeShare().contains(share.toUpperCase()) || excludeList.contains(share.toUpperCase());
				
				if (excludeBecauseHiddenDrive || excludeHiddenShare || isInExcludeList) {
					logger.warn("Skipping share: " + "\\\\" + server + "\\" + share);
					iter.remove();
				}
			}
		}
		
		return shares;
	}
}
