package com.perdemia.permissionanalyzer.application.util;

import java.io.IOException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;

/**
 * The Class RSAPrivateKeyHelper.
 */
public final class RSAPrivateKeyHelper {

	/** The Constant KEY_FILE_HEADER. */
	private static final String KEY_FILE_HEADER = "-----BEGIN RSA PRIVATE KEY-----";

	/** The Constant KEY_FILE_FOOTER. */
	private static final String KEY_FILE_FOOTER = "-----END RSA PRIVATE KEY-----";

	/**
	 * Instantiates a new RSA private key helper.
	 */
	private RSAPrivateKeyHelper() {
		// no instances needed
	}

	/**
	 * Creates the private key from PKCS#1.
	 *
	 * @param privateKey the private key
	 * @return the private key
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws NoSuchAlgorithmException the no such algorithm exception
	 * @throws InvalidKeySpecException the invalid key spec exception
	 * @throws NoSuchProviderException the no such provider exception
	 */
	public static PrivateKey createFromPKCS1(String privateKey) throws IOException, NoSuchAlgorithmException, InvalidKeySpecException, NoSuchProviderException {
		String privateKeyPem = privateKey.replace(KEY_FILE_HEADER, "");
		privateKeyPem = privateKeyPem.replace(KEY_FILE_FOOTER, "");

		return readPkcs1PrivateKey(Base64.getDecoder().decode(privateKeyPem));

	}

	/**
	 * Read pkcs 1 private key.
	 *
	 * @param pkcs1Bytes the pkcs 1 bytes
	 * @return the private key
	 * @throws NoSuchAlgorithmException the no such algorithm exception
	 * @throws NoSuchProviderException the no such provider exception
	 */
	private static PrivateKey readPkcs1PrivateKey(byte[] pkcs1Bytes) throws NoSuchAlgorithmException, NoSuchProviderException {
		// We can't use Java internal APIs to parse ASN.1 structures, so we build a PKCS#8 key Java can understand
		int pkcs1Length = pkcs1Bytes.length;
		int totalLength = pkcs1Length + 22;
		byte[] pkcs8Header = new byte[] {
				0x30, (byte) 0x82, (byte) ((totalLength >> 8) & 0xff), (byte) (totalLength & 0xff), // Sequence + total length
				0x2, 0x1, 0x0, // Integer (0)
				0x30, 0xD, 0x6, 0x9, 0x2A, (byte) 0x86, 0x48, (byte) 0x86, (byte) 0xF7, 0xD, 0x1, 0x1, 0x1, 0x5, 0x0, // Sequence: 1.2.840.113549.1.1.1, NULL
				0x4, (byte) 0x82, (byte) ((pkcs1Length >> 8) & 0xff), (byte) (pkcs1Length & 0xff) // Octet string + length
		};
		byte[] pkcs8bytes = join(pkcs8Header, pkcs1Bytes);
		return readPkcs8PrivateKey(pkcs8bytes);
	}

	/**
	 * Read pkcs 8 private key.
	 *
	 * @param pkcs8Bytes the pkcs 8 bytes
	 * @return the private key
	 * @throws NoSuchAlgorithmException the no such algorithm exception
	 * @throws NoSuchProviderException the no such provider exception
	 */
	private static PrivateKey readPkcs8PrivateKey(byte[] pkcs8Bytes) throws NoSuchAlgorithmException, NoSuchProviderException {
		KeyFactory keyFactory = KeyFactory.getInstance("RSA");
		PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(pkcs8Bytes);
		try {
			return keyFactory.generatePrivate(keySpec);
		} catch (InvalidKeySpecException e) {
			throw new IllegalArgumentException("Unexpected key format!", e);
		}
	}

	/**
	 * Join.
	 *
	 * @param byteArray1 the byte array 1
	 * @param byteArray2 the byte array 2
	 * @return the byte[]
	 */
	private static byte[] join(byte[] byteArray1, byte[] byteArray2) {
		byte[] bytes = new byte[byteArray1.length + byteArray2.length];
		System.arraycopy(byteArray1, 0, bytes, 0, byteArray1.length);
		System.arraycopy(byteArray2, 0, bytes, byteArray1.length, byteArray2.length);
		return bytes;
	}
}
