package com.perdemia.permissionanalyzer.data;

import java.nio.file.Path;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.data.util.FileCache;
import com.perdemia.permissionanalyzer.data.util.TableCountCache;

public class FileInfo extends DBObject implements Comparable<FileInfo> {
	private final static Logger logger = LoggerFactory.getLogger(FileInfo.class.getName());
	
	/** Table attributes */
	private final static String[] tableAttributes = new String[]{"NAME", "NAME_LOWERCASE", "PARENT_ID", "IS_DIRECTORY", "PARENT_FILE_IDS", "IS_DACL_PROTECTED", "OWNER_ID"};
	
	/** Table fields */
	private String name;
	private Integer parentId;
	private boolean directory;
	private String parentFileIds;
	private boolean daclProtected;
	private Integer ownerId;
	
	/** Helper fields */
	private List<FileInfo> children;
	private FileInfo parent;
	private List<AclItem> aclItems;
	private String fullFilePath;
	private Member owner;
	
	/** The count of all members on this folder that match the filter criteria 
	 * Used to calculate the ownership ratio.
	 * @see {@link MainFileTreeContentProvider#determineOwnershipRatio}
	 */
	private int allMatchingMembersOnFolder;
	
	/** The count of all (nested) members on this folder without using the filter criteria 
	 * Used to calculate the ownership ratio.
	 * @see {@link MainFileTreeContentProvider#determineOwnershipRatio}
	 */ 
	private int allMembersOnFolder;
	
	/** Unfiltered list of AclItems, used in the Effective permissions tab */
	private List<AclItem> allAclItems;
	private Path path;
	private List<String> inheritableAclEntries = new ArrayList<String>();
	private EffectivePermissions effectivePermissions;
	
	/** List of unique direct members, stored after is has been determined in {@link MainFileTreeContentProvider} */
	private List<Member> directMembers;
	
	/** List of unique direct members without applying the user filters, stored after is has been determined in {@link MainFileTreeContentProvider} */
	private List<Member> allDirectMembers;
	
	/**
	 * The last ID that was used for an INSERT of this class in the database. If 0 is returned then the
	 * {@link DBObject#insert()} method will retrieve the max ID from the database.
	 */
	private static int lastId;
	
	/**
	 * Instantiates a new file info.
	 */
	public FileInfo(){}
	
	/**
	 * Instantiates a new file info.
	 *
	 * @param name the name
	 */
	public FileInfo(String name) {
		this.name = name;
	}
	
	//========================================
	// GENERAL DB ACTIONS
	//========================================
	
	public static FileInfo getById(Integer id) throws DBException {
		return (FileInfo) getById(FileInfo.class, id);
	}
		
	@SuppressWarnings("unchecked")
	public static List<FileInfo> where(String whereClause, Object[] parameters) throws DBException {
		return where(FileInfo.class, whereClause, parameters, null, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<FileInfo> where(String whereClause, Object[] parameters, String orderBy) throws DBException {
		return where(FileInfo.class, whereClause, parameters, orderBy, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<FileInfo> where(String whereClause, Object[] parameters, String orderBy, int limit) throws DBException {
		return where(FileInfo.class, whereClause, parameters, orderBy, limit);
	}
	
	public static FileInfo findFirst(String whereClause, Object... parameters) throws DBException {
		return (FileInfo) findFirst(FileInfo.class, whereClause, parameters);
	}
	
	@SuppressWarnings("unchecked")
	public static List<FileInfo> getByPreparedStatement(PreparedStatement ps) throws DBException {
		return getByPreparedStatement(FileInfo.class, ps);
	}
	
	public static int delete(Integer id) throws DBException {
		return delete(FileInfo.class, id);
	}
	
	public static int delete(String whereClause, Object...parameters) throws DBException {
		return delete(FileInfo.class, whereClause, parameters);
	}
	
	@Override
	protected String[] getTableAttributes(){
		return tableAttributes;
	}
	
	@Override
	protected void buildObject(ResultSet rs) throws SQLException {
		id = (rs.getInt("id"));
		name = rs.getString(tableAttributes[0]);
		//don't load name_lowercase in memory because it's only used for filtering in the database!
		parentId = getNullableInteger(rs, tableAttributes[2]);
		directory = rs.getBoolean(tableAttributes[3]);
		parentFileIds = rs.getString(tableAttributes[4]);
		daclProtected = rs.getBoolean(tableAttributes[5]);
		ownerId = getNullableInteger(rs, tableAttributes[6]);
	}
	
	@Override
	protected void setQueryParameters(PreparedStatement ps) throws SQLException {
		ps.setString(1, name);
		ps.setString(2, name != null ? name.toLowerCase() : null);
		setNullableInteger(ps, 3, parentId);
		ps.setBoolean(4, directory);
		
		// add the id of the current file to the list of parent ids, so it appears in the result list when the
		// FilterHelper loads all the applicable files for a given Filter
		if (parentFileIds == null || parentFileIds.isEmpty()) {
			parentFileIds = String.valueOf(id);
		} else if (!parentFileIds.endsWith("," + id) && !parentFileIds.equals(String.valueOf(id))) {
			parentFileIds += "," + id;
		}
		
		ps.setString(5, parentFileIds);
		ps.setBoolean(6, daclProtected);
		setNullableInteger(ps, 7, ownerId);
	}
	
	public static int getCount() throws DBException {
		if (TableCountCache.getCount(FileInfo.class) == null) {
			TableCountCache.cacheCount(FileInfo.class, getCount(FileInfo.class));
		}
		return TableCountCache.getCount(FileInfo.class);
	}
	
	//========================================
	// SPECIFIC DB ACTIONS
	//========================================
	@SuppressWarnings("unchecked")
	public static List<FileInfo> getByParentId(Integer parentId) throws DBException {
		if (parentId != null) {
			return where(FileInfo.class, "parent_id = ?", new Object[]{parentId}, null, 0);
		} else {
			return where(FileInfo.class, "parent_id is null", null, null, 0);
		}
	}
	
	/**
	 * Determines the full file path and assumes that the {@link #getParent()} of each parent is loaded.
	 * 
	 * @return file path
	 */
	public String determineFullPath() {
		if (fullFilePath == null) {
			// get all the parent objects to determine the full folder path
			FileInfo nextFile = this;
			while (nextFile.getParentId() != null) {
				FileInfo nextParentFileInfo = nextFile.getParent();
				if (nextParentFileInfo == null) {
					try {
						nextParentFileInfo = FileCache.getFile(nextFile.getParentId());
						nextFile.setParent(nextParentFileInfo);
					} catch (DBException ex) {
						logger.warn("Error while determining the full file path for " + getName() + "(id: " + getId() + "): " + ex.getMessage());
						//SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while determining the full file path for " + getName() + "(id: " + getId() + "): " + ex.getMessage());
						nextParentFileInfo = new FileInfo();
						nextParentFileInfo.setName("???");
						nextFile.setParent(nextParentFileInfo);
					}
				}
				nextFile = nextParentFileInfo;						
			}
			
			// determine file path
			StringBuilder filePath = new StringBuilder();
			FileInfo parent = this;
			while (parent != null) {
				if (filePath.length() > 0) {
					filePath.insert(0, java.io.File.separator);
				}
				
				String parentName = parent.getName();
				
				// for local paths we added the server name to the root directory
				// like c:\projects\data (dataserver01)
				// we remove this suffix to get a real absolute file path
				if (parent.getParent() == null && !parentName.startsWith("\\\\") && parentName.endsWith(")") && parentName.indexOf("(") > -1) {
					parentName = parentName.substring(0, parentName.lastIndexOf("(") - 1);
				}
				filePath.insert(0, parentName);
				parent = parent.getParent();
			}
			
			if (filePath.toString().endsWith(":")) {
				filePath.append("\\");
			}
			
			fullFilePath = filePath.toString();
			
			if (fullFilePath.startsWith("/")) { // unix path?
				fullFilePath = fullFilePath.replace("\\", "/");
			}			
		}
		return fullFilePath;
	}
	
	@Override
	public int compareTo(FileInfo file2) {
		// sort on directory/file and then on name
		if (file2.isDirectory() != isDirectory()) {
			return file2.isDirectory() ? 1 : -1;
		} else {
			return getName().toLowerCase().compareTo(file2.getName().toLowerCase());
		}
	}
		
	//========================================
	// GETTERS AND SETTERS
	//========================================
	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		//this.name = CompactString.convert(name);
		this.name = name;
	}
	public Integer getId() {
		return id;
	}
	public void setId(Integer id) {
		this.id = id;
	}
	public List<FileInfo> getChildren() {
		return children;
	}
	public void setChildren(List<FileInfo> children) {
		this.children = children;
	}
	public FileInfo getParent() {
		return parent;
	}
	public void setParent(FileInfo parent) {
		if (parent != null) {
			this.parentId = parent.getId();
		}
		this.parent = parent;
	}
	public Integer getParentId() {
		return parentId;
	}
	public void setParentId(Integer parentId) {
		this.parentId = parentId;
	}
	public boolean isDirectory() {
		return directory;
	}
	public void setDirectory(boolean directory) {
		this.directory = directory;
	}
	public List<AclItem> getAclItems() {
		return aclItems;
	}
	public void setAclItems(List<AclItem> aclItems) {
		this.aclItems = aclItems;
	}

	public Path getPath() {
		return path;
	}

	public void setPath(Path path) {
		this.path = path;
	}

	public List<String> getInheritableAclEntries() {
		return inheritableAclEntries;
	}

	public void setInheritableAclEntries(List<String> inheritableAclEntries) {
		this.inheritableAclEntries = inheritableAclEntries;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("FileInfo [name=");
		builder.append(name);
		builder.append(", parentId=");
		builder.append(parentId);
		builder.append(", directory=");
		builder.append(directory);
		builder.append(", children=");
		builder.append(children);
		builder.append(", aclItems=");
		builder.append(aclItems);
		builder.append(", path=");
		builder.append(path);
		builder.append(", inheritableAclEntries=");
		builder.append(inheritableAclEntries);
		builder.append(", effectivePermissions=");
		builder.append(effectivePermissions);
		builder.append(", parentFileIds=");
		builder.append(parentFileIds);
		builder.append(", id=");
		builder.append(id);
		builder.append("]");
		return builder.toString();
	}

	public String getParentFileIds() {
		return parentFileIds;
	}

	public void setParentFileIds(String parentFileIds) {
		this.parentFileIds = parentFileIds;
	}

	public EffectivePermissions getEffectivePermissions() {
		return effectivePermissions;
	}

	public void setEffectivePermissions(EffectivePermissions effectivePermissions) {
		this.effectivePermissions = effectivePermissions;
	}

	public boolean isDaclProtected() {
		return daclProtected;
	}

	public void setDaclProtected(boolean daclProtected) {
		this.daclProtected = daclProtected;
	}

	public List<AclItem> getAllAclItems() {
		return allAclItems;
	}

	public void setAllAclItems(List<AclItem> allAclItems) {
		this.allAclItems = allAclItems;
	}
	
	@Override
	public int getLastId() {
		return lastId;
	}

	@Override
	public void setLastId(int lastId) {
		FileInfo.lastId = lastId;
	}

	public Integer getOwnerId() {
		return ownerId;
	}

	public void setOwnerId(Integer ownerId) {
		this.ownerId = ownerId;
	}

	public Member getOwner() {
		return owner;
	}

	public void setOwner(Member owner) {
		this.ownerId = owner.id;
		this.owner = owner;
	}

	public int getAllMatchingMembersOnFolder() {
		return allMatchingMembersOnFolder;
	}

	public void setAllMatchingMembersOnFolder(int allMatchingMembersOnFolder) {
		this.allMatchingMembersOnFolder = allMatchingMembersOnFolder;
	}

	public int getAllMembersOnFolder() {
		return allMembersOnFolder;
	}

	public void setAllMembersOnFolder(int allMembersOnFolder) {
		this.allMembersOnFolder = allMembersOnFolder;
	}

	public List<Member> getDirectMembers() {
		return directMembers;
	}

	public void setDirectMembers(List<Member> directMembers) {
		this.directMembers = directMembers;
	}

	public List<Member> getAllDirectMembers() {
		return allDirectMembers;
	}

	public void setAllDirectMembers(List<Member> allDirectMembers) {
		this.allDirectMembers = allDirectMembers;
	}	
}
