package com.perdemia.permissionanalyzer.data;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;

/**
 * Represents a message that is generated during the scan process. These message are displayed in the Scan View.
 * 
 * @author Reinier
 *
 */
public class ScanLogItem extends DBObject {
	private final static Logger logger = LoggerFactory.getLogger(ScanLogItem.class.getName());
	
	/** Table attributes */
	private final static String[] tableAttributes = new String[]{"SERVER", "LOG_DATE", "SEVERITY", "MESSAGE", "DETAILS"};
	
	/** Table fields */
	private String server;
	private Date logDate;
	private Severity severity;
	private String message;
	private String details;
	
	public static enum Severity {
		TRACE, DEBUG, INFO, WARNING, ERROR
	};
	
	/**
	 * The last ID that was used for an INSERT of this class in the database. If 0 is returned then the
	 * {@link DBObject#insert()} method will retrieve the max ID from the database.
	 */
	private static int lastId;
	
	public ScanLogItem(){}
	
	public ScanLogItem(Severity severity, String message, String details) {
		this.server = AclScanner.getLocalMachineName();
		this.logDate = new Date();
		this.severity = severity;
		this.message = message;
		if (details != null && details.length() >= 3000) {
			this.details = details.substring(0,  2998); 
		} else {
			this.details = details;
		}
	}
	
	//========================================
	// GENERAL DB ACTIONS
	//========================================
	
	@SuppressWarnings("unchecked")
	public static List<ScanLogItem> getAll() throws DBException {
		return (List<ScanLogItem>) getAll(ScanLogItem.class, "SERVER, LOG_DATE", 0);
	}
	
	public static ScanLogItem getById(Integer id) throws DBException {
		return (ScanLogItem) getById(ScanLogItem.class, id);
	}
		
	@SuppressWarnings("unchecked")
	public static List<ScanLogItem> where(String whereClause, Object[] parameters) throws DBException {
		return where(ScanLogItem.class, whereClause, parameters, null, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<ScanLogItem> where(String whereClause, Object[] parameters, String orderBy) throws DBException {
		return where(ScanLogItem.class, whereClause, parameters, orderBy, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<ScanLogItem> where(String whereClause, Object[] parameters, String orderBy, int limit) throws DBException {
		return where(ScanLogItem.class, whereClause, parameters, orderBy, limit);
	}
	
	public static ScanLogItem findFirst(String whereClause, Object... parameters) throws DBException {
		return (ScanLogItem) findFirst(ScanLogItem.class, whereClause, parameters);
	}
	
	public static int delete(Integer id) throws DBException {
		return delete(ScanLogItem.class, id);
	}
	
	public static int delete(String whereClause, Object...parameters) throws DBException {
		return delete(ScanLogItem.class, whereClause, parameters);
	}
	
	@Override
	protected String[] getTableAttributes(){
		return tableAttributes;
	}
	
	@Override
	protected void buildObject(ResultSet rs) throws SQLException {
		id = (rs.getInt("id"));
		server = rs.getString(tableAttributes[0]);
		logDate = rs.getTimestamp(tableAttributes[1]);
		severity = Severity.values()[rs.getInt(tableAttributes[2])];
		message = rs.getString(tableAttributes[3]);
		details = rs.getString(tableAttributes[4]);
	}
	
	@Override
	protected void setQueryParameters(PreparedStatement ps) throws SQLException {
		ps.setString(1, server);
		ps.setTimestamp(2, new Timestamp(logDate.getTime()));
		ps.setInt(3, severity.ordinal());
		ps.setString(4, message);
		ps.setString(5, details);
		
	}

	//========================================
	// GETTERS AND SETTERS
	//========================================
			
	public Date getLogDate() {
		return logDate;
	}

	public void setLogDate(Date logDate) {
		this.logDate = logDate;
	}

	public Severity getSeverity() {
		return severity;
	}

	public void setSeverity(Severity severity) {
		this.severity = severity;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public String getDetails() {
		return details;
	}

	public void setDetails(String details) {
		this.details = details;
	}

	public String getServer() {
		return server;
	}

	public void setServer(String server) {
		this.server = server;
	}

	@Override
	public int getLastId() {
		return lastId;
	}

	@Override
	public void setLastId(int lastId) {
		ScanLogItem.lastId = lastId;
	}	
}
