package com.perdemia.permissionanalyzer.data.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Domain;
import com.perdemia.permissionanalyzer.data.Rdn;

public class RdnCache {
	private static final Logger logger = LoggerFactory.getLogger(RdnCache.class.getName());
	
	/** Maps for rdn cache */
	private static final Map<String, Rdn> rdnCacheByName = new HashMap<String, Rdn>(100);
	private static final Map<Integer, Rdn> rdnCacheById = new HashMap<Integer, Rdn>(100);
	
	/** Cache settings */
	private static final int CACHE_SIZE = 5000;
	
	/**
	 * Clears the Map of the cache and forces a refresh from the database whenever the cache is used.
	 */
	public static void clearCache() {
		rdnCacheByName.clear();
		rdnCacheById.clear();
	}
		
	/** Checks if the rdn can be found in the {@link #rdnCacheByName} cache. If not, if will check the database and if it's found
	 * the rdn will be added to the cache. If it's not found in the database it will be created in the database!
	 * 
	 * @param rdn
	 * 
	 * @return Rdn object, either from cache or from database, or created in the database
	 * 
	 * @throws DBException
	 */
	public static Rdn getRdn(String rdn) throws DBException {
		Rdn rdnObject = rdnCacheByName.get(rdn);
		
		if (rdnObject != null && logger.isTraceEnabled()) {
			logger.trace("Returning rdn from cache: " + rdnObject.toString());
		}
		
		if (rdnObject == null) {
			// try to get it from the database
			rdnObject = Rdn.findFirst("rdn = ?", rdn);
			
			if (rdnObject != null) {
				// add to cache
				cacheRdn(rdnObject);
			} else {
				// not in the database yet, create a new Rdn record and add it to the cache
				rdnObject = new Rdn(rdn);
				rdnObject.save();
				rdnCacheByName.put(rdn, rdnObject);
			}
		}
		
		return rdnObject;
	}
	
	/** Checks if the rdn can be found in the {@link #rdnCacheByName} cache. If not, if will check the database and if it's found
	 * the rdn will be added to the cache.
	 * 
	 * @param rdn
	 * 
	 * @return null in case the rdn cannot be found in the cache and in the database
	 * 
	 * @throws DBException
	 */
	public static Rdn getRdn(int id) throws DBException {
		Rdn rdnObject = rdnCacheById.get(id);
		
		if (rdnObject != null && logger.isTraceEnabled()) {
			logger.trace("Returning rdn from cache: " + rdnObject.toString());
		}
		
		if (rdnObject == null) {
			// try to get it from the database
			rdnObject = Rdn.getById(id);
			
			if (rdnObject != null) {
				// add to cache
				cacheRdn(rdnObject);
			}
		}
		
		return rdnObject;
	}
		
	public static void cacheRdn(Rdn rdn) {
		if (rdnCacheByName.size() > CACHE_SIZE) {
			if (logger.isTraceEnabled()) {
				logger.trace("Rdn name cache full, clear cache");
			}			
			cleanupCache(rdnCacheByName);			
		}
		
		if (rdnCacheById.size() > CACHE_SIZE) {
			if (logger.isTraceEnabled()) {
				logger.trace("Rdn id cache full, clear cache");
			}			
			cleanupCache(rdnCacheById);			
		}
		rdnCacheById.put(rdn.getId(), rdn);
	}
		
	/**
	 * Cleans up a cache when the maximum number of items has been reached. It removes half of the entries in the Map.
	 * 
	 * @param map Map to clean
	 */
	private static void cleanupCache(Map map) {
		logger.debug("Clean up half of the Rdn cache, max size reached: " + CACHE_SIZE);
		
		// clear half of the cache
		int clearCount = (CACHE_SIZE / 2);
		int count = 0;
		Iterator iter = map.keySet().iterator();
		
		while (iter.hasNext() && count < clearCount) {
			iter.next();
			iter.remove();
			count++;
		}
	}
}