package com.perdemia.permissionanalyzer.scanner.acl;

import java.nio.file.AccessDeniedException;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.scanner.groups.LocalGroupHelper;
import com.sun.jna.platform.win32.WinNT;

public abstract class AceReader {
	
	public abstract List<AclItem> getAclList(Path path, boolean followLinks, FileInfo file, WinNT.SECURITY_DESCRIPTOR_RELATIVE secDescriptor) throws AccessDeniedException;
	
	/**
	 * Only implemented by the {@link EmcIsilonAceReader} reader. Other ACE readers will throw a NotImplemented exception, since the DACL information is read from the file system instead of the txt import file.
	 *  
	 * @return true in case the directory has ACL inheritance blocked
	 * @throws AccessDeniedException
	 */
	public abstract boolean isCurrentFileDaclProtected ();
	
	public static Map<String, String> builtInAccounts;	
	
	/** Corresponding permissions for each generic permission */
	public static final List<Integer> GENERIC_ALL_PERMISSIONS = new ArrayList<Integer>(Arrays.asList(new Integer[]{0,1,2,3,4,5,6,7,8,9,10,11,12})); 
	public static final List<Integer> GENERIC_EXECUTE_PERMISSIONS = new ArrayList<Integer>(Arrays.asList(new Integer[]{5,7,10}));
	public static final List<Integer> GENERIC_WRITE_PERMISSIONS = new ArrayList<Integer>(Arrays.asList(new Integer[]{1,2,4,8,10}));
	public static final List<Integer> GENERIC_READ_PERMISSIONS = new ArrayList<Integer>(Arrays.asList(new Integer[]{0,3,7,10}));
	
	static {
		builtInAccounts = new HashMap<String, String>();
		
		builtInAccounts.put("S-1-5-32-544", "Administrators");
		builtInAccounts.put("S-1-5-32-545", "Users");
		builtInAccounts.put("S-1-5-32-546", "Guests");
		builtInAccounts.put("S-1-5-32-547", "Power Users");
		builtInAccounts.put("S-1-5-32-548", "Account operators");
		builtInAccounts.put("S-1-5-32-549", "Server operators");
		builtInAccounts.put("S-1-5-32-550", "Print operators");
		builtInAccounts.put("S-1-5-32-551", "Backup operators");
		builtInAccounts.put("S-1-5-32-552", "Replicators");
		builtInAccounts.put("S-1-5-83-0", "NT VIRTUAL MACHINE\\Virtual Machines");
		builtInAccounts.put("S-1-5-32-554", "Pre-Windows 2000 Compatible Access");
		builtInAccounts.put("S-1-5-32-555", "Remote Desktop Users");
		builtInAccounts.put("S-1-5-32-556", "Network Configuration Operators");
		builtInAccounts.put("S-1-5-32-557", "Incoming Forest Trust Builders");
		builtInAccounts.put("S-1-5-32-558", "Performance Monitor Users");
		builtInAccounts.put("S-1-5-32-559", "Performance Log Users");
		builtInAccounts.put("S-1-5-32-560", "Windows Authorization Access Group");
		builtInAccounts.put("S-1-5-32-561", "Terminal Server License Servers");
		builtInAccounts.put("S-1-5-32-562", "Distributed COM Users");
		builtInAccounts.put("S-1-5-32-573", "Event Log Readers");
					
		builtInAccounts.put("S-1-4", "Non-unique Authority");
		builtInAccounts.put("S-1-5", "NT Authority");
		builtInAccounts.put("S-1-5-1", "Dialup");
		builtInAccounts.put("S-1-5-2", "Network");
		builtInAccounts.put("S-1-5-3", "Batch");
		builtInAccounts.put("S-1-5-4", "Interactive");
		builtInAccounts.put("S-1-5-6", "Service");
		builtInAccounts.put("S-1-5-7", "Anonymous");
		builtInAccounts.put("S-1-5-8", "Proxy");
		builtInAccounts.put("S-1-5-9", "Enterprise Domain Controllers");
		builtInAccounts.put("S-1-1-0", "Everyone");
		builtInAccounts.put("S-1-5-10", "Principal Self");
		builtInAccounts.put("S-1-5-11", "Authenticated Users");
		builtInAccounts.put("S-1-5-12", "Restricted Code");
		builtInAccounts.put("S-1-5-13", "Terminal Server Users");
		builtInAccounts.put("S-1-5-14", "Remote Interactive Logon");
		builtInAccounts.put("S-1-5-15", "This Organization");
		builtInAccounts.put("S-1-5-17", "This Organization");
		builtInAccounts.put("S-1-5-18", "Local system");
		builtInAccounts.put("S-1-5-19", "NT Authority");
		builtInAccounts.put("S-1-5-20", "NT Authority");
		builtInAccounts.put("S-1-5-80", "NT Service");
		builtInAccounts.put("S-1-5-84-0-0-0-0-0", "User-mode driver");
		builtInAccounts.put("S-1-5-113", "All local accounts group");
		builtInAccounts.put("S-1-5-114", "All local accounts and admins group");
		builtInAccounts.put("S-1-5-1000", "All users from another organization");
		builtInAccounts.put("S-1-15-2-1", "Application package context");
		builtInAccounts.put("S-1-16-0", "Untrusted integrity level");
		builtInAccounts.put("S-1-16-4096", "Low integrity level");
		builtInAccounts.put("S-1-16-8192", "Medium integrity level");
		builtInAccounts.put("S-1-16-8448", "Medium-plus integrity level");
		builtInAccounts.put("S-1-16-12288", "High integrity level");
		builtInAccounts.put("S-1-16-16384", "System integrity level");
		builtInAccounts.put("S-1-16-20480", "Protected-process integrity level");
		builtInAccounts.put("S-1-18-1", "Asserted by an authentication authority");
		builtInAccounts.put("S-1-18-2", "Asserted by a service");
		builtInAccounts.put("S-1-3-0", "Creator Owner");
		builtInAccounts.put("S-1-5-4", "Interactive");
		builtInAccounts.put("S-1-5-7", "Anonymous");
	}
	
	/**
	 * Returns true in case the given sid is in the list of known built-in sids for a local computer.
	 * This is done to prevent multiple groups like BUILT-IN\Administrators when multiple servers are scanned. In case this
	 * method returns true, the "BUILT-IN" prefix is replaced by the server name.
	 * 
	 * @param sid
	 * @return true in case built-in
	 */
	public static boolean isLocalBuiltinAccount(String sid) {
		return builtInAccounts.containsKey(sid);
	}
	
	/**
	 * Returns true in case the given account name is known as built-in account. This doesn't work very reliable because it depends on the language of 
	 * the operating system. This method is used when retrieving the local groups of a server, because we don't have the sid of a local group, 
	 * only the account name.
	 * 
	 * @param sid
	 * @return true in case built-in
	 * @see {@link LocalGroupHelper#processLocalGroups(String, com.perdemia.permissionanalyzer.application.model.StatusListener)}
	 */
	public static boolean isLocalBuiltinAccountByName(String name) {
		return builtInAccounts.containsValue(name);
	}
		
	public static String getBuiltinAccountName(String sid) {
		return builtInAccounts.get(sid);
	}
	
	public static String getMachineNameFromPath(String path) {
		String server;
		
		if (path.startsWith("\\\\")) { //escaped \\
			int firstBackslash = path.substring(2).indexOf("\\"); //skip the first two backslashes
			if (firstBackslash != -1) {
				server = path.substring(2, firstBackslash + 2); //plus two for the first two backslashes
			} else {
				server = path.substring(2);
			}
		} else {
			server = AclScanner.getLocalMachineName();
		}
		
		return server;
	}
}
