package com.perdemia.permissionanalyzer.scanner.acl;

import static com.sun.jna.platform.win32.WinNT.SE_DACL_PROTECTED;
import static java.nio.file.FileVisitResult.CONTINUE;
import static java.nio.file.FileVisitResult.TERMINATE;

import java.io.IOException;
import java.lang.reflect.Field;
import java.nio.file.AccessDeniedException;
import java.nio.file.FileSystemLoopException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.NoSuchFileException;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.UserPrincipal;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.model.StatusListener;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.Member.GroupType;
import com.perdemia.permissionanalyzer.data.util.MemberCache;
import com.sun.jna.platform.win32.Advapi32Util;
import com.sun.jna.platform.win32.WinNT;

public class AclFileVisitor extends SimpleFileVisitor<Path> {
	/** The AceReader to be used.
	 * 
	 * @see {@link JnaAceReader}
	 * @see {@link JavaNioAceReader}
	 * @see {@link EmcIsilonAceReader}
	 */
	private AceReader aceReader;
	
	/** Map containing pairs of aclEntryToString(entry) and folder ids (List), meaning: [acl entry string] inherited from [folder list] */
	private Map<String, List<Integer>> inheritedAclEntryFromFolderIds = new HashMap<String, List<Integer>>();

	/** List of current parent folders */
	private LinkedList<FileInfo> parentFolders = new LinkedList<FileInfo>();

	/** Folder being processed, set in preVisitDirectory method */
	private FileInfo previousFile;

	/** Logger to log information to the StatusListeners */
	private static Logger logger = LoggerFactory.getLogger(AclFileVisitor.class.getName());

	/** Listener used to display messages in the GUI */
	private StatusListener statusListener;
	
	/** Flag to indicate if files need to be scanned */
	private boolean includeFiles;
	
	/** Flag to indicate if inherited ACE's need to be stored in the database */
	private boolean includeInheritedPermissions;
	
	/** Flag to indicate if symbolic links should be followed */
	private boolean followLinks;
	
	/** The server name is added to the root directories in case it is not a network share */
	private String serverName;
			
	/** statistics */
	private long numberOfDirectories;
	private long numberOfFiles;
	private long numberOfUsers;
	private long numberOfGroups;
	
	/** Reflection field to get the SID from a UserPrinciple */
	private static Field sidStringField;
	
	/**
	 * Initialize fields and methods of the JDK protected packages using reflection
	 */
	static {
		try {
			//Reflection fields
			sidStringField = Class.forName("sun.nio.fs.WindowsUserPrincipals$User").getDeclaredField("sidString");
			sidStringField.setAccessible(true);			
		} catch (Exception ex) {
			logger.warn("Exception while initializing NativeAccessController: " + ex.getMessage(), ex);
		}
	}
	
	/**
	 * Constructor
	 * 
	 * @param statusListener listener to receive status messages during the scan
	 */
	public AclFileVisitor(StatusListener statusListener, String serverName, AceReader aceReader) {
		this.statusListener = statusListener;
		this.serverName = serverName;
		this.aceReader = aceReader;
	}
	
	/**
	 * Resets values of a last scan
	 */
	public void reset(){
		numberOfDirectories = 0;
		numberOfFiles = 0;
		previousFile = null;
		inheritedAclEntryFromFolderIds = new HashMap<String, List<Integer>>();
		parentFolders = new LinkedList<FileInfo>();
	}
	
	@Override
	public FileVisitResult visitFile(Path file, BasicFileAttributes attr) throws IOException {
		if (logger.isTraceEnabled()) {
			logger.trace("Start scanning file: " + file);
		}

		FileVisitResult result = FileVisitResult.CONTINUE;	
		result = scanItem(file, attr.isDirectory(), true);
				
		if (result.ordinal() == FileVisitResult.TERMINATE.ordinal()) {
			//cancelled by user
			return result;
		}
		
		if (attr.isDirectory()) {
			// when a directory contains no sub directories then this visitFile method is called
			// instead of preVisitDirectory and postVisitDirectory. Call the postDirectory manually:
			postVisitDirectory(file, null);

		}

		return result;
	}
	
	@Override
	public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
		return scanItem(dir, attrs.isDirectory(), true);
	}

	@Override
	public FileVisitResult postVisitDirectory(Path dir, IOException exc) {
		FileInfo leavingFolder;
		
		if (logger.isTraceEnabled()) {
			logger.trace("PostVisitDirectoryStart: " + dir);
		}
		
		// are we leaving a parent folder?
		if (getCurrentParent() != null && getCurrentParent().getPath().equals(dir)) {
			if (logger.isDebugEnabled()) {
				logger.debug("Removing parent from the queue: " + dir);
			}
			leavingFolder = getCurrentParent();
			// remove the current parent and use to the previous parent
			parentFolders.removeLast();
		} else {
			leavingFolder = previousFile;
		}
		
		// remove any inheritances set by this folder
		Iterator<Entry<String, List<Integer>>> iter = inheritedAclEntryFromFolderIds.entrySet().iterator();
		while (iter.hasNext()) {
			Entry<String, List<Integer>> entry = iter.next();
			int index = entry.getValue().indexOf(leavingFolder.getId());
			if (index >= 0) {
				if (logger.isDebugEnabled()) {
					logger.debug("Removing inheritance for folder " + leavingFolder.getPath() + " and aclEntry " + entry.getKey());
				}

				entry.getValue().remove(index);
				if (entry.getValue().size() == 0) {
					if (logger.isDebugEnabled()) {
						logger.debug("No inheritance folders left for acl entry " + entry.getKey());
					}
					// remove this aclEntry item
					iter.remove();
				}
			}
		}
		
		// forget the previousFile, since we're leaving a parent folder
		previousFile = null;

		return CONTINUE;
	}

	/**
	 * Returns the current parent by getting the top element of the parentFolders Queue.
	 * 
	 * @return current parent Folder
	 */
	public FileInfo getCurrentParent() {
		if (parentFolders.size() > 0) {
			return parentFolders.getLast();
		} else {
			return null;
		}
	}
	
	/**
	 * Scans a file or folder for ACL information. Saves new members in the database and all the permissions for the given file/folder.
	 * 
	 * @param filePath
	 * @param readSecurityDescriptor Indicates of this file visitor should read the security descriptor from the file system (and give it to the AceReader).
	 * 		It might be the case the this AclFileVisitor is used to read a text file, then the sec. descriptor of those paths should not be read.
	 */
	public FileVisitResult scanItem(Path filePath, boolean isDirectory, boolean readSecurityDescriptor) {
		if (!isDirectory && !includeFiles) {
			return CONTINUE;
		}
		
		try {
			if (logger.isTraceEnabled()) {
				logger.trace("Start scanning path: " + filePath);
			}
						
			//statistics
			if (isDirectory) {
				numberOfDirectories++;
			} else {
				numberOfFiles++;
			}
			
			if (statusListener != null) {
				statusListener.updateStatus("Scanning file system (" + (numberOfDirectories + numberOfFiles) + "): " + filePath.toString());
						
				// cancelled by user?
				if (statusListener.isCancelled()) {
					return TERMINATE;
				}
			}
			
			// has this folder another parent than the current parent? OR is this the second round and don't we have a parent yet? (previousFile != null && parentFolders.size() == 0)
			if (previousFile != null && previousFile.isDirectory() && (parentFolders.size() == 0 || !getCurrentParent().getPath().equals(filePath.getParent()))) {
				
				if (!filePath.getParent().equals(previousFile.getPath())) {
					logger.warn(MessageFormat.format("The file {0} does not belong to parent {1}", new Object[]{filePath.getParent().toString(), previousFile.getPath().toString()}));
				}
				
				//add the last folder (previousFile) to the list of parents
				parentFolders.add(previousFile);

				if (logger.isDebugEnabled()) {
					logger.debug("Adding new parent: " + previousFile.getPath());
				}
			}
			
			// save the current file name in DB
			previousFile = new FileInfo();
			if (parentFolders.size() == 0) {
				// use the full path for the first parent
				String fullPath = filePath.toString();
				if (fullPath.endsWith("\\") || fullPath.endsWith("/")) {
					fullPath = fullPath.substring(0, fullPath.length() - 1);
				}
				
				// Add the server name as suffix in case it a local file, meaning the path doesn't start with
				// '\\' (network share) or '\' a unix import file where the /opt/.. is replaced by Path to \opt\... 
				if (!fullPath.startsWith("\\\\") && !fullPath.startsWith("\\") && serverName != null) {
					fullPath += " (" + serverName + ")";
				}
				previousFile.setName(fullPath);
			} else {
				// just use the file name
				if (filePath.getFileName() != null) {
					previousFile.setName(filePath.getFileName().toString());
				} else {
					previousFile.setName(filePath.toFile().getName());
				}
			}
			if (getCurrentParent() != null) {
				previousFile.setParentId(getCurrentParent().getId());
			}
			previousFile.setPath(filePath);
			previousFile.setDirectory(isDirectory);
			
			if (readSecurityDescriptor) {
				UserPrincipal ownerPrinciple = Files.getOwner(filePath);
				if (ownerPrinciple != null) {
					Member owner = new Member();
					String sid = String.valueOf(sidStringField.get(ownerPrinciple)).toUpperCase();
					
					// we now check for BUILT-IN accounts. We convert those account names to <server>\\Administrators instead of BUILT-IN\Administrators.
					owner.setName(ownerPrinciple.getName(), sid, null, AceReader.getMachineNameFromPath(filePath.toString()));
	
					if (ownerPrinciple.getClass().getSimpleName().equals("Group")) {
						// when the group is saved in the database (and not found in the DB), then it is probably a domain group.
						owner.setGroupType(GroupType.GLOBAL);
						owner.setGroup(true);
					}
					
					// owner in cache?
					Member member = MemberCache.getMemberByName(owner.getName(false), owner.getDomainId());
					if (member != null) {
						if (!member.isBuiltIn() && owner.isBuiltIn()) {
							// at this point we have found a local group. In the LocalGroupHelper, which is used while scanning for local groups,
							// we didn't have a SID and we can only determine built-in groups by their name. At this point we have used the 
							// JnaAceReader which has a SID to check if the member is a built-in. So now we update the cache with the
							// SID information found on the file system.
							member.setBuiltIn(true);
							member.save();
						}
						
						// update the owner object in the fileinfo
						previousFile.setOwner(member);
											
						if (logger.isDebugEnabled()) {
							logger.debug("Found owner in cache: " + member.getName(false));
						}					
					} else {
						
						// save this new owner in DB
						owner.save();
						numberOfUsers++;
						statusListener.usersScanned(1);
						
						previousFile.setOwner(owner);
						
						// save in cache
						MemberCache.cacheMemberByName(owner);
					}
				}
			}
			
			if (logger.isDebugEnabled()){
				logger.debug("Saving new folder in DB: " + previousFile.toString());
			}
			
			StringBuilder parentFileIds = new StringBuilder();
			for (int counter = 0; counter < parentFolders.size(); counter++) {
				FileInfo parent = parentFolders.get(counter);
				if (counter > 0) {
					parentFileIds.append(",");
				}
				parentFileIds.append(parent.getId());
			}
			
			previousFile.setParentFileIds(parentFileIds.toString());
			
						
			// get the security descriptor for this file
			// NOTE: The Advapi32Util.getFileSecurityDescriptor method has been modified! In case the path startsWith \\ and 
			// the length is longer than 248 characters it will convert to path to a short notation (~1 etc). Otherwise the 
			// getNamedSecurityInfo will throw an error "The filename, directory name, or volume label syntax is incorrect."
			WinNT.SECURITY_DESCRIPTOR_RELATIVE secDescriptor = null;
			
			if (readSecurityDescriptor) {
				secDescriptor = Advapi32Util.getFileSecurityDescriptor(filePath.toFile(), false);
				previousFile.setDaclProtected((secDescriptor.Control & SE_DACL_PROTECTED) != 0);
			} else {
				previousFile.setDaclProtected(aceReader.isCurrentFileDaclProtected());
			}
			
			previousFile.save(); // generate id
			
			// Process the ACE's of this file/directory
			List<AclItem> aclItems = aceReader.getAclList(filePath, followLinks, previousFile, secDescriptor);
			//List<AclItem> aclItems = JnaAceReader.getAclList(filePath, followLinks);

			for (AclItem aclItem : aclItems) {
				if (logger.isTraceEnabled()) {
					logger.trace("Processing next AclEntry: " + aclItem.toString());
				}
				
				if (!includeInheritedPermissions && aclItem.isFlagInheritedAce()) {
					if (logger.isTraceEnabled()) {
						logger.trace("Skipping inherited ACE");
					}
					continue;
				}
					
				// member in cache?
				Member member = MemberCache.getMemberByName(aclItem.getMember(false).getName(false), aclItem.getMember(false).getDomainId());
				if (member != null) {
					if (!member.isBuiltIn() && aclItem.getMember(false).isBuiltIn()) {
						// at this point we have found a built-in group. In the LocalGroupHelper, which is used while scanning for local groups,
						// we didn't have a SID and we can only determine built-in groups by their name. At this point we have used the 
						// JnaAceReader which has a SID to check if the member is a built-in. So now we update the cache with the
						// SID information found on the file system.
						member.setBuiltIn(true);
						member.save();
					}
					
					// update the member object in the aclItem
					aclItem.setMember(member);
										
					if (logger.isDebugEnabled()) {
						logger.debug("Found member in cache: " + member.getName(false));
					}					
				} else {
					
					// save this new member in DB
					aclItem.getMember(false).save();
					
					// It may occur that the license limit of the number of users/groups has been reached
					// in that case skip this ACE and process the next one.
					if (aclItem.getMember(false).getId() == null) {
						continue; // next ACE
					}
					
					if (aclItem.getMember(false).isGroup()) {
						statusListener.groupsScanned(1);
						numberOfGroups++;
					} else {
						statusListener.usersScanned(1);
						numberOfUsers++;
					}
					
					// forward the auto generated member id to the aclItem
					aclItem.setMemberId(aclItem.getMember(false).getId());
				
					// save in cache
					MemberCache.cacheMemberByName(aclItem.getMember(false));
				}

				// retrieve inheritance information from its parents
				String aclString = aclItemToString(aclItem);
				List<Integer> inheritedFromFolders = inheritedAclEntryFromFolderIds.get(aclString);
				
				if (inheritedFromFolders == null) {
					inheritedFromFolders = new ArrayList<Integer>();
				}
				
				// remember inheritance set by this folder so we can later on determine the 'inherited from folder' value
				// don't save the inheritance information in case the NO_PROPAGATE flag is set, because they only apply to the
				// direct children. The array inheritedFromFolders is meant for inheritance in the whole tree of directories
				// and don't save inheritance in case this ACE has been inherited by a parent
				if (previousFile.isDirectory() && (aclItem.isFlagDirectoryInherit() || aclItem.isFlagFileInherit()) &&
						!aclItem.isFlagInheritedAce()) {
					
					// the inheritable aclEntries are saved to check if the children (sub folders) have inherited their ace from
					// a parent ace with the flag NO_PROPAGATE
					previousFile.getInheritableAclEntries().add(aclString);
					
					if (!aclItem.isFlagNoPropagateInherit()) {
						// aclEntry not assigned to one of the parents?
						if (inheritedFromFolders.size() == 0) {
							// new aclEntry for inheritance
							inheritedFromFolders.add(previousFile.getId());
							inheritedAclEntryFromFolderIds.put(aclString, inheritedFromFolders);
							
							if (logger.isDebugEnabled()) {
								logger.debug("Added new acl entry for inheritance: " + aclItem.toString() + ", folder: " + previousFile.getPath() + ", folder id: " + previousFile.getId());
							}
						} else {
							// this aclEntry is already assigned to a parent folder
							// BUT, perhaps the aclEntry is not present on the direct parent, which means that this folder has the
							// aclEntry directly assigned to it
							if (!aclItem.isFlagInheritedAce()) {
								
								// It appears that folders (like the ones in Recyle bin) can have multiple ACE's which are exactly the same. In that case we don't we to add
								// to folder id to the list of inheritedFromFolders for this aclString. Check for it using contains(..)
								if (!inheritedFromFolders.contains(previousFile.getId())) {
									inheritedFromFolders.add(previousFile.getId());
								}
	
								if (logger.isDebugEnabled()) {
									logger.debug("Extended existing acl entry with new inheritance folder: " + aclItem.toString() + ", currentFolder: " + previousFile.getPath());
								}
							} else {
								if (logger.isTraceEnabled()) {
									logger.trace("Don't add inheritance, inherited from a parent: " + (aclItem != null ? aclItem.toString() : "null") + ", currentFolder: " + (previousFile != null ? previousFile.getPath() : "null"));
								}
							}
						}
					}
				}
			
				// in case the ACE has been inherited determine the 'inherited from' folder id
				if (aclItem.isFlagInheritedAce()) {
					if (inheritedFromFolders.size() == 0) {
												
						/*
						At this point there is no matching ACE found in one of the parent folders, while this ACE does have the INHERITED_ACE flag. This means one of the following:
						
						- its a File, Windows removes the ACE flags from files because files don't have inheritance.
						- the same ACE on the parent has the flag NO_PROPAGATE set. In that case this ACE was inherited from its parent, but Windows removed all the flags 
						  so it only applies to this folder (which is the purpose of NO_PROPAGATE).
						- A generic permission has been inherited, in which case Windows creates two ACE's:
						 	  generic_all - inherited_ace - propagate to folder and files
						 	  full 		  - inherited_ace - apply only to this folder
						*/
						
						/* Check for the NO_PROPAGATE flag by adding:
						  - "DFP" (directory and file inherit plus no propagate) to the aclString for directories or files
						  - "DP" (directory inherit + no propagate) only for directories
						  - "FP" (file inherit + no propagate) only for files
						 */
						if (getCurrentParent() != null && (
						getCurrentParent().getInheritableAclEntries().contains(aclString + "DFP") ||
						(isDirectory && getCurrentParent().getInheritableAclEntries().contains(aclString + "DP")) ||
						(!isDirectory && getCurrentParent().getInheritableAclEntries().contains(aclString + "FP")))) {
							
							if (logger.isTraceEnabled()) {
								logger.trace("ACE has been inherited from parent with NO_PROPAGATE flag: " + previousFile.toString());
							}
							// inherited from a parent with the NO_PROPAGATE flag
							inheritedFromFolders = new ArrayList<Integer>();
							inheritedFromFolders.add(getCurrentParent().getId());
						}
						// File flags are always removed by Windows, so it might be that this file ACE has been inherited from a previous parent
						// Same for generic permissions inherited by a parent, the flags are changed by Windows as explained above
						//} else if (!attr.isDirectory() || generic) {
							else if (inheritedAclEntryFromFolderIds.containsKey(aclString + "F")) {
								inheritedFromFolders = inheritedAclEntryFromFolderIds.get(aclString + "F");
							
							} else if (inheritedAclEntryFromFolderIds.containsKey(aclString + "D")) {
								inheritedFromFolders = inheritedAclEntryFromFolderIds.get(aclString + "D");
									
							} else if (inheritedAclEntryFromFolderIds.containsKey(aclString + "DF")) {
								inheritedFromFolders = inheritedAclEntryFromFolderIds.get(aclString + "DF");
							}
						//}
					}
					
					if (inheritedFromFolders.size() > 0) {
						aclItem.setInheritedFromFolderId(inheritedFromFolders.get(inheritedFromFolders.size() - 1));
					}
				}

				// finish the AclItem
				aclItem.setFileId(previousFile.getId());
												
				// save aclItem in DB
				aclItem.save();
				
				if (logger.isTraceEnabled()) {
					logger.trace("AclItem saved in DB: " + aclItem.toString());
				}
				
				//TODO use executeBatch with prepared insert statement and call clearBatch() after each executeBatch()!!!
				//TODO use more prepared statements 
			}
		} catch (Exception e) {
			logger.error("Error while scanning path " + filePath + ": " + e.getMessage(), e);
			
			// try to save the file anyway (perhaps we received an access denied while reading the owner)
			if (previousFile != null) {
				try {
					previousFile.save();
				} catch (DBException e1) {
					logger.error("Error while saving path " + filePath + ": " + e.getMessage(), e);
				}
			}
		}
		
		return CONTINUE;
	}

	// If there is some error accessing the file, let the user know.
	// If you don't override this method and an error occurs, an IOException is thrown.
	@Override
	public FileVisitResult visitFileFailed(Path file, IOException e) {
		//TODO handle more types of IO exceptions?
		if (e instanceof FileSystemLoopException) {
			logger.warn("Symbolic link cycle detected, skipping file: " + file);
		} else if (e instanceof AccessDeniedException)  {
			logger.warn("Error while processing path " + file + ": Access denied", e);
		} else if (e instanceof NoSuchFileException) {
			//apparently it might happen that the Visitor visits a directory that doesn't exist, like
			// "\\localhost\Users\Default\Documents\My Music"
			// Just ignore the exception because the folder/file doesn't exist
			if (logger.isDebugEnabled()) {
				logger.debug("NoSuchFileException exception occured for file " + file);
			}
		} else {
			logger.error("Error while processing path " + file, e);
		}
		return CONTINUE;
	}
	
	public void addParent(FileInfo fileInfo) {
		parentFolders.add(fileInfo);
	}

	/**
	 * Creates a compact version of the default AclEntry.toString() without using the flag INHERIT_ONLY. When ACL inheritance is used in combination with the INHERIT_ONLY flag, then only
	 * the first (parent) folder has the INHERIT_ONLY flag while all the children folders and files have only the DIRECTORY_INHERIT or/and FILE_INHERTI flags. We still want to match those children
	 * with the parent folder, that is why the INHERIT_ONLY flag is not included in this String.  
	 * 
	 * @param entry
	 * @return String describing the entry
	 */
	private String aclItemToString(AclItem aclItem) {
		StringBuilder aclString = new StringBuilder();
		
		aclString.append(aclItem.getMember(true).getId());
		aclString.append(aclItem.getPermissionId());	

		if (aclItem.isDeny()) {
			aclString.append("N");
		}
		if (aclItem.isFlagDirectoryInherit()) {
			aclString.append("D");
		}
		if (aclItem.isFlagFileInherit()) {
			aclString.append("F");
		}
		if (aclItem.isFlagNoPropagateInherit()) {
			aclString.append("P");
		}
		
		return aclString.toString();
	}
	
	public long getNumberOfDirectories() {
		return numberOfDirectories;
	}

	public void setNumberOfDirectories(long numberOfDirectories) {
		this.numberOfDirectories = numberOfDirectories;
	}

	public long getNumberOfFiles() {
		return numberOfFiles;
	}

	public void setNumberOfFiles(long numberOfFiles) {
		this.numberOfFiles = numberOfFiles;
	}

	public boolean isIncludeFiles() {
		return includeFiles;
	}

	public void setIncludeFiles(boolean includeFiles) {
		this.includeFiles = includeFiles;
	}

	public boolean isFollowLinks() {
		return followLinks;
	}

	public void setFollowLinks(boolean followLinks) {
		this.followLinks = followLinks;
	}

	public boolean isIncludeInheritedPermissions() {
		return includeInheritedPermissions;
	}

	public void setIncludeInheritedPermissions(boolean includeInheritedPermissions) {
		this.includeInheritedPermissions = includeInheritedPermissions;
	}

	public String getServerName() {
		return serverName;
	}

	public void setServerName(String serverName) {
		this.serverName = serverName;
	}

	public AceReader getAceReader() {
		return aceReader;
	}

	public void setAceReader(AceReader aceReader) {
		this.aceReader = aceReader;
	}

	public long getNumberOfUsers() {
		return numberOfUsers;
	}

	public void setNumberOfUsers(long numberOfUsers) {
		this.numberOfUsers = numberOfUsers;
	}

	public long getNumberOfGroups() {
		return numberOfGroups;
	}

	public void setNumberOfGroups(long numberOfGroups) {
		this.numberOfGroups = numberOfGroups;
	}

	public FileInfo getPreviousFile() {
		return previousFile;
	}

	public LinkedList<FileInfo> getParentFolders() {
		return parentFolders;
	}

	public void setPreviousFile(FileInfo previousFile) {
		this.previousFile = previousFile;
	}

}
