package com.perdemia.permissionanalyzer.scanner.acl;

import java.nio.file.AccessDeniedException;
import java.nio.file.Path;
import java.nio.file.attribute.AclFileAttributeView;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.dialogs.MessageDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SpecialPermission;
import com.sun.jna.platform.win32.WinNT;

/**
 * This class is written to read the ACE data from a text file which can be imported in the application.
 * 
 * @author Reinier
 * @see @{link JnaAceReader}
 * @see {@link JavaNioAceReader}
 * @see {@link EmcIsilonAceReader}
 */
public final class EmcIsilonAceReader extends AceReader {
	private final static Logger logger = LoggerFactory.getLogger(EmcIsilonAceReader.class.getName());
	private EmcIsilonFileScanner importScanner;
	
	public EmcIsilonAceReader(EmcIsilonFileScanner importScanner) {
		this.importScanner = importScanner;
	}
	
	/**
	 * <p>Gets the access mask of each ACE in the given Path and checks if it has one of the generic permissions bits enabled. If so, it finds the corresponding {@link AclItem} and adds
	 * the special permissions represented by the generic permission(s). 
	 * </p><p>
	 * Because the complete SecurityDescriptor is retrieved of the Path, the method will loop through all ACE's and accepts a list of AclItems as parameter. The SecurityDescriptor of the file
	 * is retrieved using Reflection since Java doesn't return the access mask bits using the standard {@link AclFileAttributeView}!
	 * </p>
	 * @param aclItems
	 * @param path
	 * @param followLinks
	 * @param file optional FileInfo object that is being updated with the SE_DACL_PROTECTED flag from the path's security descriptor
	 */
	@Override
	public List<AclItem> getAclList(Path path, boolean followLinks, FileInfo file, WinNT.SECURITY_DESCRIPTOR_RELATIVE secDescriptor) throws AccessDeniedException {
		// List of ACE's from the import file (the ones that belong to the current file)
		List<String> aces = importScanner.getAces();
		
		// List of AclItems for the DB
		List<AclItem> aclItems = new ArrayList<AclItem>();
				
		for (String ace : aces) {
			AclItem aclItem = new AclItem();
			
			try {
				// get the member info
				Member member = new Member();
				if (ace.contains("group:")) {
					member.setGroup(true);
				}
				
				int memberStartPosition = ace.indexOf("group:") + 6;
				if (memberStartPosition == 5) { // not found
					memberStartPosition  = ace.indexOf("user:") + 5;
				}
				if (memberStartPosition == 4) { // not found
					memberStartPosition  = ace.indexOf(" SYSTEM ") + 1; // use SYSTEM as account name 
				}
				if (memberStartPosition == 0) { // still not found
					// we have encountered the lines " CONTROL:sacl_auto_inherited" in the ACL text files, don't process these lines
					continue;
				}
				
				int memberEndPosition = ace.indexOf(" allow ");
				if (memberEndPosition == -1) {
					memberEndPosition = ace.indexOf(" deny ");
					aclItem.setDeny(true);
				}
				
				member.setName(ace.substring(memberStartPosition, memberEndPosition));
				member.setDisplayName(member.getName(false));
				aclItem.setMember(member);
				
				// flags
				aclItem.setFlagDirectoryInherit(ace.contains("container_inherit"));
				aclItem.setFlagFileInherit(ace.contains("object_inherit"));
				aclItem.setFlagInheritOnly(ace.contains("inherit_only"));
				aclItem.setFlagNoPropagateInherit(ace.contains("no_prop_inherit"));
				aclItem.setFlagInheritedAce(ace.contains("inherited_ace"));
				
				// Determine the permission value (only unique items using Set, see next part generic permissions)
				Set<Integer> permissionOrdinals = new HashSet<Integer>();
				
				// Permissions that can appear in the listing for any filesystem object are:
				if (ace.contains("std_delete")) {
					permissionOrdinals.add(SpecialPermission.DELETE.ordinal());
				}
				if (ace.contains("std_read_dac")) {
					permissionOrdinals.add(SpecialPermission.READ_ACL.ordinal());
				}
				if (ace.contains("std_write_dac")) {
					permissionOrdinals.add(SpecialPermission.WRITE_ACL.ordinal());
				}
				if (ace.contains("std_write_owner")) {
					permissionOrdinals.add(SpecialPermission.WRITE_OWNER.ordinal());
				}
				if (ace.contains("std_required")) {
					permissionOrdinals.add(SpecialPermission.DELETE.ordinal());
					permissionOrdinals.add(SpecialPermission.READ_ACL.ordinal());
					permissionOrdinals.add(SpecialPermission.WRITE_ACL.ordinal());
					permissionOrdinals.add(SpecialPermission.WRITE_OWNER.ordinal());
				}
				
				// Permissions that can appear only for a directory are:
				if (ace.contains("list")) {
					permissionOrdinals.add(SpecialPermission.READ_DATA.ordinal());
				}
				if (ace.contains("add_file")) {
					permissionOrdinals.add(SpecialPermission.WRITE_DATA.ordinal());
				}
				if (ace.contains("add_subdir")) {
					permissionOrdinals.add(SpecialPermission.APPEND_DATA.ordinal());
				}
				if (ace.contains("delete_child")) {
					permissionOrdinals.add(SpecialPermission.DELETE_CHILD.ordinal());
				}
				if (ace.contains("traverse")) {
					permissionOrdinals.add(SpecialPermission.EXECUTE.ordinal());
				}
				if (ace.contains("dir_read_attr")) {
					permissionOrdinals.add(SpecialPermission.READ_ATTRIBUTES.ordinal());
				}
				if (ace.contains("dir_write_attr")) {
					permissionOrdinals.add(SpecialPermission.WRITE_ATTRIBUTES.ordinal());
				}
				if (ace.contains("dir_read_ext_attr")) {
					permissionOrdinals.add(SpecialPermission.READ_NAMED_ATTRS.ordinal());
				}
				if (ace.contains("dir_write_ext_attr")) {
					permissionOrdinals.add(SpecialPermission.WRITE_NAMED_ATTRS.ordinal());
				}
				
				// Permissions that can appear in the listing for a non-directory filesystem object are:
				if (ace.contains("file_read")) {
					permissionOrdinals.add(SpecialPermission.READ_DATA.ordinal());
				}
				if (ace.contains("file_write")) {
					permissionOrdinals.add(SpecialPermission.WRITE_DATA.ordinal());
				}
				if (ace.contains("append")) {
					permissionOrdinals.add(SpecialPermission.APPEND_DATA.ordinal());
				}
				if (ace.contains("execute")) {
					permissionOrdinals.add(SpecialPermission.EXECUTE.ordinal());
				}
				if (ace.contains("file_read_attr")) {
					permissionOrdinals.add(SpecialPermission.READ_ATTRIBUTES.ordinal());
				}
				if (ace.contains("file_write_attr")) {
					permissionOrdinals.add(SpecialPermission.WRITE_ATTRIBUTES.ordinal());
				}
				if (ace.contains("file_read_ext_attr")) {
					permissionOrdinals.add(SpecialPermission.READ_NAMED_ATTRS.ordinal());
				}
				if (ace.contains("file_write_ext_attr")) {
					permissionOrdinals.add(SpecialPermission.WRITE_NAMED_ATTRS.ordinal());
				}
								
				// add the generic permissions (again only unique items using Set)
				if (ace.contains("dir_gen_all") || ace.contains("file_gen_all")) {
					//aclItem.setGenericAll(true);
					permissionOrdinals.addAll(GENERIC_ALL_PERMISSIONS);
				}
				if (ace.contains("dir_gen_execute") || ace.contains("file_gen_execute")) {
					//aclItem.setGenericExecute(true);
					permissionOrdinals.addAll(GENERIC_EXECUTE_PERMISSIONS);
				}
				if (ace.contains("dir_gen_write") || ace.contains("file_gen_write")) {
					//aclItem.setGenericWrite(true);
					permissionOrdinals.addAll(GENERIC_WRITE_PERMISSIONS);
				}
				if (ace.contains("dir_gen_read") || ace.contains("file_gen_read")) {
					//aclItem.setGenericRead(true);
					permissionOrdinals.addAll(GENERIC_READ_PERMISSIONS);
				}
				
				aclItem.setPermissionId(PermissionLookup.getIdBySpecialPermissions(permissionOrdinals));
				
				
				if (file != null) {
					aclItem.setDirectory(file.isDirectory());
				}
				
				aclItems.add(aclItem);
				if (logger.isTraceEnabled()) {
					logger.trace("Retrieved ACE information, new AclItem: " + aclItem.toString());
				}
			} catch(Exception ex) {
				logger.error("Error while processing ace " + ace, ex);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while processing ace " + ace + ": " + ex.getMessage());
			}
		}
				
		return aclItems;
	}
	
	@Override
	public boolean isCurrentFileDaclProtected() {
		return importScanner.isCurrentFileIsDACLProtected();
	}
}
