package com.perdemia.permissionanalyzer.scanner.acl;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.FileVisitResult;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.CharSet;
import org.eclipse.jface.dialogs.MessageDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ProgramParameters;
import com.perdemia.permissionanalyzer.data.DBObject;
import com.perdemia.permissionanalyzer.data.FileInfo;

public class EmcIsilonFileScanner {
	private static Logger logger = LoggerFactory.getLogger(EmcIsilonFileScanner.class.getName());
	private List<String> aces = null;
	private boolean currentFileIsDACLProtected;
	private AclFileVisitor aclFileVisitor;
	
	public EmcIsilonFileScanner(AclFileVisitor aclFileVisitor) {
		this.aclFileVisitor = aclFileVisitor;
	}
	
	public void scan(String importFilePath) {
		// Keep a record of directories so we can call postVisitDirectory when a directory is done
		List<Path> directoryTree = new ArrayList<Path>();
		boolean processedFirstFile = false;
				
		// Feed the AclFileVisitor with files from the import file
		String line = null;
		// TODO determine file charset? Instead of UTF-8 hardcoded?
		try(BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(importFilePath), StandardCharsets.UTF_8))) {
			String lastPath = null;
			boolean lastPathIsDirectory = false;
			
		    while ((line = br.readLine()) != null) {
				if (line.length() > 10) { // minimal content length, don't process empty lines
			    	boolean isPath = !line.startsWith(" ");
			        
			        // new path?
			        if (isPath) {
			        	String newPath = line.substring(line.indexOf('/'));
			        	
			        	// not the first path in the file?
			        	if (lastPath != null) {
			        		
			        		Path lastPathObject = Paths.get(lastPath);
			        		List<Path> foldersInBetween = new ArrayList<Path>();
			        		
			        		// an ECM Isilon text file may skip folders!
							// for example, we go from \\server1\test1\subtest2\subtest3 to \\server1\test5\subtest6\subtest7
							// Check for this, and process the folders individually. Start from the last known parent
			        		FileInfo currentParent = aclFileVisitor.getCurrentParent();
			        		if (currentParent != null && !currentParent.getPath().equals(lastPathObject.getParent()) && (aclFileVisitor.getPreviousFile() == null || !aclFileVisitor.getPreviousFile().getPath().equals(lastPathObject.getParent()))) {
			        			Path currentParentPath = aclFileVisitor.getCurrentParent().getPath();
			        			Path parentPath = lastPathObject;
				        		while(parentPath.getParent() != null) {
				        			if (!currentParentPath.equals(parentPath.getParent())) {
				        				foldersInBetween.add(0, parentPath.getParent());
				        			} else {
				        				break;
				        			}
				        			
				        			parentPath = parentPath.getParent();
				        		}
				        		
				        		for (Path folderInBetween : foldersInBetween) {
			        				// at this point we have found a parent that has not been processed yet (a parent that doesn't match with aclFileVisitor.getCurrentParent())
			        				// check if this new parent is already in the database
			        				FileInfo parentInDB = FileInfo.findFirst("parent_id = ? and name_lowercase = ?", aclFileVisitor.getCurrentParent().getId(), folderInBetween.toFile().getName().toString().toLowerCase());
			        							        				
			        				if (parentInDB != null) {
			        					parentInDB.setPath(folderInBetween);
			        					aclFileVisitor.addParent(parentInDB);
			        				} else {
			        					FileInfo newParent = new FileInfo();
			        					newParent.setName(folderInBetween.toFile().getName().toString());
			        					newParent.setParentId(aclFileVisitor.getCurrentParent().getId());
			        					newParent.setDirectory(true);
			        					newParent.setPath(folderInBetween);
			        					
			        					// build the comma separated list of parent id's
			        					StringBuilder parentFileIds = new StringBuilder();
			        					for (int counter = 0; counter < aclFileVisitor.getParentFolders().size(); counter++) {
			        						FileInfo parent = aclFileVisitor.getParentFolders().get(counter);
			        						if (counter > 0) {
			        							parentFileIds.append(",");
			        						}
			        						parentFileIds.append(parent.getId());
			        					}
			        					newParent.setParentFileIds(parentFileIds.toString());
			        					
			        					newParent.save();
			        					aclFileVisitor.addParent(newParent);
			        				}
			        				
			        				// not already scanned?
			        				if (!directoryTree.contains(folderInBetween)) {
			        					directoryTree.add(0, folderInBetween);
			        				}
				        		}
				        		
			        		} else if (currentParent == null) {			        			
			        			// does this root folder already exists?
			        			// the prepared statement parameters will be escaped from \\ to \\\\, to sure why :( Use a normal query instead:
			        			FileInfo parentInDB = FileInfo.findFirst("parent_id is null and name_lowercase = '" + lastPathObject.toFile().getParentFile().toString().toLowerCase() + "'");
			        			
			        			if (parentInDB != null) {
		        					parentInDB.setPath(lastPathObject.getParent());
		        					aclFileVisitor.addParent(parentInDB);
		        				}
			        		}
			        	
		        			// process the file and its ACE's
	        				FileVisitResult result = aclFileVisitor.scanItem(lastPathObject, lastPathIsDirectory, false);
	        				
	        				if (result.equals(FileVisitResult.TERMINATE)) {
			        			break;
			        		}
		        			
			        		// store the directory for later use
			        		if (lastPathIsDirectory) {
			        			directoryTree.add(0, lastPathObject);
			        		}			        	
				        		
			        		// are we exiting a directory? Important to call this method, because the parent folder is cleaned up in the aclFileVisitor!
			        		// process this method after aclFileVisitor.scanItem, because a new parent is only added when a new subfile/folder is processed by aclFileVisitor.scanItem!
			        		Iterator<Path> iter = directoryTree.iterator();
			        		Path newPathObject = Paths.get(newPath);			        		
			        		
			        		while(iter.hasNext()) {
			        			Path parentDir = iter.next();
			        			// is parentDir not the parent of newPath?
			        			if (!newPathObject.startsWith(parentDir) && !newPathObject.equals(parentDir)) {
			        				result = aclFileVisitor.postVisitDirectory(parentDir, null);
			        				iter.remove();
			        				
			        				if (result.equals(FileVisitResult.TERMINATE)) {
					        			break;
					        		}
			        			}
			        		}			        		
			        	}
			        	
			        	lastPath = newPath;
			        	lastPathIsDirectory = line.startsWith("d");
			        	
			        	// reset the list with ACE's
		        		aces = new ArrayList<String>();
		        		currentFileIsDACLProtected = false;
		        		
			        } else {
			        	// create the first root directory
						if (!processedFirstFile) {
														
							// root folder already in the DB (by another import file)?
							FileInfo parentInDB = FileInfo.findFirst("parent_id is null and name_lowercase = '" + Paths.get(lastPath).toFile().getParentFile().toString().toLowerCase() + "'");
		        			
		        			if (parentInDB != null) {
	        					parentInDB.setPath(Paths.get(lastPath).getParent());
	        					aclFileVisitor.addParent(parentInDB);
	        					aclFileVisitor.setPreviousFile(parentInDB);
	        				} else {
	        					aclFileVisitor.scanItem(Paths.get(lastPath).getParent(), true, false);
	        				}
		        			
		        			// delete this directory info from the database?
							if (!PreferenceHelper.getInstance().isTruncateData() && !ProgramParameters.isClearData()) {
								// does this directory already exist?
								FileInfo existingDir = FileInfo.findFirst("parent_id = ? and name_lowercase = ?",  + aclFileVisitor.getPreviousFile().getId(), Paths.get(lastPath).toFile().getName().toLowerCase());
								
								if (existingDir != null) {
									logger.info("Deleting existing database info for directory " + lastPath);
									DBObject.executeStatement("delete from permissionanalyzer.fileinfo where parent_file_ids like '" + existingDir.getId() + ",%'");
									DBObject.executeStatement("delete from permissionanalyzer.fileinfo where id = " + existingDir.getId());
								}
							}
							
							processedFirstFile = true;
						}
						
			        	// its a ACE line that belong to the current file
						if (line.startsWith(" CONTROL:")) {
							currentFileIsDACLProtected = line.contains("dacl_protected");
						} else {
							aces.add(line);
						}
			        }
		        }
		    }
		    
		    // done, we ended with a list of ACE's, so process that last file
		    FileVisitResult result = null;
		    if (lastPath != null) {
		    	result = aclFileVisitor.scanItem(Paths.get(lastPath), lastPathIsDirectory, false);
		    }
		    
		    // exit the remaining directories
		    if (result == null || result.equals(FileVisitResult.CONTINUE)) {
			    for (Path dir : directoryTree) {
			    	result = aclFileVisitor.postVisitDirectory(dir, null);
			    	
			    	if (result.equals(FileVisitResult.TERMINATE)) {
	        			break;
	        		}
			    }
		    }
		    
		    directoryTree.clear();
		    
		} catch (Exception e) {
			logger.error("Error while importing the ACE file on line: " + line, e);
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while importing the ACE file:" + e.getMessage());
		}					
	}
	
	public static void main(String[] args) {
		//ApplicationUtil.openDBConnection();
		
		/*
		AclScanner scanner = new AclScanner(null);
		List<DirectoryInfo> directoryList = new ArrayList<DirectoryInfo>();
		DirectoryInfo directoryInfo = new DirectoryInfo();
		directoryInfo.setNumberOfSubdirectories(0);
		directoryInfo.setPath("C:\\Users\\Reinier\\Test");
		directoryInfo.setEnabled(true);
		directoryList.add(directoryInfo);
		
		scanner.scan(directoryList);
		*/
		
		String test = "computer\\reinier";
		String name = test.substring(test.indexOf("\\") + 1);
		
		// upper case the computer/domain name to be uniform with other parts of the scan
		String computerName = test.substring(0, test.indexOf("\\"));
		System.out.println(computerName.toUpperCase() + "\\" + name);
	}

	public List<String> getAces() {
		return aces;
	}

	public boolean isCurrentFileIsDACLProtected() {
		return currentFileIsDACLProtected;
	}
}
