package com.perdemia.permissionanalyzer.scanner.acl;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.FileVisitResult;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.eclipse.jface.dialogs.MessageDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.model.StatusListener;

public class PowerShellFileScanner {
	private static Logger logger = LoggerFactory.getLogger(PowerShellFileScanner.class.getName());
	private List<String> aces = null;
	private AclFileVisitor aclFileVisitor;

	public PowerShellFileScanner(AclFileVisitor aclFileVisitor) {
		this.aclFileVisitor = aclFileVisitor;
	}

	public void scan(String importFilePath) {
		// Keep a record of directories so we can call postVisitDirectory when a directory is done
		List<String> directoryTree = new ArrayList<String>();
		boolean processedFirstFile = false;
		
		// Feed the AclFileVisitor with files from the import file
		FileInputStream fis = null;
		ZipInputStream zis = null;
		BufferedReader reader = null;
		String line = null;
		
		try {
			fis = new FileInputStream(importFilePath);
			
			if (importFilePath.toLowerCase().endsWith(".zip")) {	
		        zis = new ZipInputStream(fis);
		        ZipEntry entry = zis.getNextEntry();
		        if (entry != null) {
		            reader = new BufferedReader(new InputStreamReader(zis, "UTF-8"));
		        } else {
		        	line = "no zip entry found in the file.";
		        	throw new Exception(line);
		        }
		        
			} else {
				reader = new BufferedReader(new InputStreamReader(fis, "UTF-8"));
			}
		
			String lastPath = null;
			boolean processingPath = false;
			FileVisitResult result = null;
			
		    while ((line = reader.readLine()) != null) {
				if (line.length() > 10) { // minimal content length, don't process empty lines

					// new path?
					if (line.startsWith("Path")) {
						processingPath = true;
						
						// remove the Path: prefix
						String newPath = line.substring(9);

						// not the first path in the file?
						if (lastPath != null) {
							
							boolean isDirectory = lastPath.endsWith(", d=True");
							
							// remove the suffix ,True or ,False
							lastPath = lastPath.substring(0, lastPath.lastIndexOf(","));
							
							// process the file and its ACE's
							result = aclFileVisitor.scanItem(Paths.get(lastPath), isDirectory, false);

							if (result.equals(FileVisitResult.TERMINATE)) {
								break;
							}

							// are we exiting a directory?
							Iterator<String> iter = directoryTree.iterator();
							while (iter.hasNext()) {
								String parentDir = iter.next();
								if (!newPath.startsWith(parentDir + "\\") && !newPath.startsWith(parentDir + "/")) {
									result = aclFileVisitor.postVisitDirectory(Paths.get(parentDir), null);
									iter.remove();
									
									if (result.equals(FileVisitResult.TERMINATE)) {
										break;
									}
								}
							}

							// store the directory for later use
							if (isDirectory) {
								directoryTree.add(0, lastPath);
							}
						}

						lastPath = newPath;
						
						// reset the list with ACE's
						aces = new ArrayList<String>();
					
					} else if (line.startsWith("Access")) {
						// create the first root directory
						if (!processedFirstFile) {
							// remove the suffix ,True or ,False
							aclFileVisitor.scanItem(Paths.get(lastPath.substring(0, lastPath.lastIndexOf(","))).getParent(), true, false);
							processedFirstFile = true;
						}
						
						processingPath = false;
						
						// its a ACE line that belong to the current file
						aces.add(line);
						
					} else if (processingPath) {
						// the Path has wrapped to a new line!
						lastPath += line.substring(9);					
						
					} else {
						// another ACE
						if (aces != null) {
							aces.add(line);
						} else {
							// no path processed yet (??)
							logger.error("Found unknown ACL line: " + line);
						}
					}
				}
			}

			// done, we ended with a list of ACE's, so process that last file
			if (lastPath != null && !result.equals(FileVisitResult.TERMINATE)) {
				boolean isDirectory = lastPath.endsWith(", d=True");
				
				// remove the suffix ,True or ,False
				lastPath = lastPath.substring(0, lastPath.lastIndexOf(","));
				
				result = aclFileVisitor.scanItem(Paths.get(lastPath), isDirectory, false);
			}

			// exit the remaining directories
			if (result == null || result.equals(FileVisitResult.CONTINUE)) {
				for (String dir : directoryTree) {
					result = aclFileVisitor.postVisitDirectory(Paths.get(dir), null);

					if (result.equals(FileVisitResult.TERMINATE)) {
						break;
					}
				}
			}

			directoryTree.clear();

		} catch (Exception e) {
			logger.error("Error while importing the ACE file: " + line, e);
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while importing the ACE file:" + e.getMessage());
		} finally {
			
			try {
				if (reader != null) {
					reader.close();
				}
				if (zis != null) {
					zis.close();
				}
				if (fis != null) {
					fis.close();
				}
			} catch (IOException e) {
				logger.error("Could not close file stream in ACE import.", e);
			}		
		}
	}

	public static void main(String[] args) {
		// ApplicationUtil.openDBConnection();

		/*
		 * AclScanner scanner = new AclScanner(null); List<DirectoryInfo> directoryList = new ArrayList<DirectoryInfo>(); DirectoryInfo directoryInfo = new DirectoryInfo();
		 * directoryInfo.setNumberOfSubdirectories(0); directoryInfo.setPath("C:\\Users\\Reinier\\Test"); directoryInfo.setEnabled(true); directoryList.add(directoryInfo);
		 * 
		 * scanner.scan(directoryList);
		 */

		String test = "computer\\reinier";
		String name = test.substring(test.indexOf("\\") + 1);

		// upper case the computer/domain name to be uniform with other parts of the scan
		String computerName = test.substring(0, test.indexOf("\\"));
		System.out.println(computerName.toUpperCase() + "\\" + name);
	}

	public List<String> getAces() {
		return aces;
	}
}
