package com.perdemia.permissionanalyzer.scanner.ldap;

import java.awt.Composite;
import java.security.GeneralSecurityException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.net.SocketFactory;
import javax.net.ssl.SSLContext;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.widgets.Display;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.LdapPasswordDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchLdapDialog;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo;
import com.perdemia.permissionanalyzer.application.model.LdapOUInfo;
import com.perdemia.permissionanalyzer.application.model.LdapOUInfo.SearchObjectType;
import com.perdemia.permissionanalyzer.application.model.StatusListener;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ProgramParameters;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.LdapContainer;
import com.perdemia.permissionanalyzer.data.LdapContainer.Type;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.Member.GroupType;
import com.perdemia.permissionanalyzer.data.Membership;
import com.perdemia.permissionanalyzer.data.Rdn;
import com.perdemia.permissionanalyzer.data.util.DomainCache;
import com.perdemia.permissionanalyzer.data.util.MemberCache;
import com.perdemia.permissionanalyzer.data.util.MembershipCache;
import com.perdemia.permissionanalyzer.data.util.RdnCache;
import com.perdemia.permissionanalyzer.scanner.acl.JnaAceReader;
import com.sun.jna.platform.win32.Advapi32Util;
import com.sun.jna.platform.win32.Netapi32Util;
import com.unboundid.asn1.ASN1OctetString;
import com.unboundid.ldap.sdk.Attribute;
import com.unboundid.ldap.sdk.BindRequest;
import com.unboundid.ldap.sdk.BindResult;
import com.unboundid.ldap.sdk.CRAMMD5BindRequest;
import com.unboundid.ldap.sdk.DIGESTMD5BindRequest;
import com.unboundid.ldap.sdk.DIGESTMD5BindRequestProperties;
import com.unboundid.ldap.sdk.ExtendedRequest;
import com.unboundid.ldap.sdk.Filter;
import com.unboundid.ldap.sdk.GSSAPIBindRequest;
import com.unboundid.ldap.sdk.GSSAPIBindRequestProperties;
import com.unboundid.ldap.sdk.LDAPConnection;
import com.unboundid.ldap.sdk.LDAPConnectionOptions;
import com.unboundid.ldap.sdk.LDAPException;
import com.unboundid.ldap.sdk.LDAPResult;
import com.unboundid.ldap.sdk.Modification;
import com.unboundid.ldap.sdk.ModificationType;
import com.unboundid.ldap.sdk.ModifyRequest;
import com.unboundid.ldap.sdk.RootDSE;
import com.unboundid.ldap.sdk.SASLQualityOfProtection;
import com.unboundid.ldap.sdk.SearchRequest;
import com.unboundid.ldap.sdk.SearchResult;
import com.unboundid.ldap.sdk.SearchResultEntry;
import com.unboundid.ldap.sdk.SearchScope;
import com.unboundid.ldap.sdk.SimpleBindRequest;
import com.unboundid.ldap.sdk.controls.PasswordExpiredControl;
import com.unboundid.ldap.sdk.controls.PasswordExpiringControl;
import com.unboundid.ldap.sdk.controls.SimplePagedResultsControl;
import com.unboundid.ldap.sdk.extensions.StartTLSExtendedRequest;
import com.unboundid.util.Debug;
import com.unboundid.util.LDAPTestUtils;
import com.unboundid.util.ssl.SSLUtil;
import com.unboundid.util.ssl.TrustAllTrustManager;

/**
 * Provides methods to communicate with the LDAP.  
 * 
 * @author Reinier L'abee
 */
public final class LdapConnectorImpl implements LdapConnector {
	private LDAPConnection ldapConnection = null;
	private LdapServerType serverType;

	/** LDAP Implementations */
	public enum LdapServerType {
		ACTIVE_DIRECTORY, NOVELL, NETSCAPE, SUN, ORACLE_INTERNET_DIR, ORACLE_VIRTUAL_DIR, ORACLE_UNIFIED_DIR, IBM, OPEN_LDAP
	};

	/** LDAP Group types */
	private final String LOCAL_SECURITY_GROUP = "-2147483644";
	private final String GLOBAL_SECURITY_GROUP = "-2147483646";
	private final String UNIVERSAL_SECURITY_GROUP = "-2147483640";
	private final String DOMAIN_BUILTIN_SECURITY_GROUP = "-2147483643";
	
	/** Ldap attributes used to retrieve group and/or user information */
	private final static String ACCOUNT_NAME_ATTRIBUTE = PreferenceHelper.getInstance().getLdapAccountNameAttribute(); //"sAMAccountName"
	private final static String USER_DISPLAY_NAME_ATTRIBUTE = PreferenceHelper.getInstance().getLdapUserDisplayNameAttribute(); //"displayName"
	private final static String GROUP_DISPLAY_NAME_ATTRIBUTE = PreferenceHelper.getInstance().getLdapGroupDisplayNameAttribute(); //"description"
	private final static String GROUP_TYPE_ATTRIBUTE = PreferenceHelper.getInstance().getLdapGroupTypeAttribute(); //"groupType"
	private final static String DN_ATTRIBUTE = PreferenceHelper.getInstance().getLdapDNAttribute(); //"distinguishedName"
	private final static String PRIMARY_GROUP_ID_ATTRIBUTE = PreferenceHelper.getInstance().getLdapPrimaryGroupIdAttribute(); //"primaryGroupID"
	private final static String GROUP_MEMBER_ATTRIBUTE = PreferenceHelper.getInstance().getLdapGroupMemberAttribute(); //"member"
	private final static String MEMBEROF_ATTRIBUTE = PreferenceHelper.getInstance().getLdapMemberOfAttribute(); //"memberOf"
	private final static String OBJECT_SID_ATTRIBUTE = "objectSid";
	
	/**
	 * Attribute to determine the enabled/disable status of a user
	 * @see https://support.microsoft.com/nl-nl/help/305144/how-to-use-the-useraccountcontrol-flags-to-manipulate-user-account-properties
	 */
	private final static String USER_ACCOUNT_CONTROL = "UserAccountControl";
	private final static long USER_DISABLED_BIT = 2;
	
	private final String[] LDAP_ATTRIBUTES = new String[]{USER_DISPLAY_NAME_ATTRIBUTE, GROUP_DISPLAY_NAME_ATTRIBUTE, ACCOUNT_NAME_ATTRIBUTE, GROUP_TYPE_ATTRIBUTE, DN_ATTRIBUTE, PRIMARY_GROUP_ID_ATTRIBUTE, GROUP_MEMBER_ATTRIBUTE, MEMBEROF_ATTRIBUTE, OBJECT_SID_ATTRIBUTE, USER_ACCOUNT_CONTROL};
	
	private static Logger logger = LoggerFactory.getLogger(LdapConnectorImpl.class.getName());
	
	//TODO explain on website
	//The global catalog stores the membership (the member attribute) of only universal groups. 
	//The membership of other groups can be ascertained at the domain level.
	//Because a universal group can have members from domains other than the domain where the group object is stored and can be used to provide access to resources in any domain, 
	//only a global catalog server is guaranteed to have all universal group memberships that are required for authentication. 
	
	/** statistics */
	private int numberOfUsers;
	private int numberOfGroups;
	private int totalNumberOfUsers;
	private int totalNumberOfGroups;
	
	/** Cache the LDAP password of the bind user once asked using InputDialog */
	private String password;	
	
	/** Cache the PrimaryGroupId lookup (on SID) */
	private Map<String, Integer> primaryGroupCache = new HashMap<String, Integer>();
	
	/**
	 * Opens the connection on the private ldapConnection object
	 */
	public void openConnection(final LdapConnectionInfo connection) throws LDAPException, GeneralSecurityException {
		password = connection.getPassword();
		if ((password == null || password.length() == 0 && connection.getUsername() != null && connection.getUsername().length() > 0)) {
			//TODO password given in ProgramParameters?
			
			// ask for the password
			final LdapPasswordDialog dialog = new LdapPasswordDialog(null, connection);
			
			Display.getDefault().syncExec(new Runnable(){
				@Override
				public void run() {
					if (dialog.open() == IDialogConstants.OK_ID) {
						password = dialog.getPassword();
						
						connection.setUsername(dialog.getUser());
						connection.setPassword(password);
						connection.setOnlyStorePasswordInMemory(!dialog.isStorePassword());
						PreferenceHelper.getInstance().storePreferences(false);
					}					
				}				
			});			
		}
		
		SSLUtil sslUtil = new SSLUtil(null, new TrustAllTrustManager());
		BindRequest bindRequest = null;
		
		// Setup connection (using plain, ssl or starttls) 
		switch (connection.getEncryption()) {
			case PLAIN:
				ldapConnection = new LDAPConnection(connection.getHostName(), connection.getHostPort());
				break;
				
			case STARTTLS:
				// Plain connection on the normal port, but with a request to upgrade to an encrypted line (using StartTLSExtendedRequest) 
				SSLContext context = sslUtil.createSSLContext();
	            ExtendedRequest extRequest = new StartTLSExtendedRequest(context);
	            
				ldapConnection = new LDAPConnection(connection.getHostName(), connection.getHostPort());
				ldapConnection.processExtendedOperation(extRequest);			
				break;
				
			case SSL:
				// Use no key manager, and trust all certificates.			
				SocketFactory socketFactory = sslUtil.createSSLSocketFactory();
				ldapConnection = new LDAPConnection(socketFactory, connection.getHostName(), connection.getHostPort());
				break;
		}
						
		// Specifies the list of allowed qualities of protection that may be used for communication that occurs on the connection after the authentication has
		// completed, in order from most preferred to least preferred.
		List<SASLQualityOfProtection> qopList = new ArrayList<SASLQualityOfProtection>();
		if (connection.isQualityOfProtectionAuthConfAllowed()) {
			qopList.add(SASLQualityOfProtection.AUTH_CONF);
		}
		if (connection.isQualityOfProtectionAuthIntAllowed()) {
			qopList.add(SASLQualityOfProtection.AUTH_INT);
		}
		if (connection.isQualityOfProtectionAuthAllowed()) {
			qopList.add(SASLQualityOfProtection.AUTH);
		}
		
		// Authentication
		switch (connection.getAuthentication()) {
			case SIMPLE:
				bindRequest = new SimpleBindRequest(connection.getUsername(), password);
				break;
				
			case CRAMMD5:
				// Username should be an "authzId" value as described in section 5.2.1.8 of RFC 4513. That is, it should be either "dn:" followed by the distinguished name of the target user, 
				// or "u:" followed by the username. If the "u:" form is used, then the mechanism used to resolve the provided username to an entry may vary from server to server.
				bindRequest = new CRAMMD5BindRequest("u:" + connection.getUsername(), password);
				break;
				
			case DIGESTMD5:
				// Username should be an "authzId" value as described in section 5.2.1.8 of RFC 4513. That is, it should be either "dn:" followed by the distinguished name of the target user, 
				// or "u:" followed by the username. If the "u:" form is used, then the mechanism used to resolve the provided username to an entry may vary from server to server.
				DIGESTMD5BindRequestProperties digestMD5Properties = new DIGESTMD5BindRequestProperties("u:" + connection.getUsername(), password);
				
				// list of allowed Quality of Protection mechanisms
				if (qopList.size() > 0) {
					digestMD5Properties.setAllowedQoP(qopList);
				}				
				
				// An optional string which specifies the realm into which the user should authenticate. 
				// If this is not provided, an attempt will be made to determine the appropriate value from the system configuration
				if (connection.getRealm() != null && connection.getRealm().length() > 0) {
					digestMD5Properties.setRealm(connection.getRealm());
				}
				
				bindRequest = new DIGESTMD5BindRequest(digestMD5Properties);
				break;
				
			case GSSAPI:
				GSSAPIBindRequestProperties gssapiProperties = new GSSAPIBindRequestProperties(connection.getUsername(), password);
				
				// An optional string which specifies the IP address or resolvable name for the Kerberos key distribution center. 
				// If this is not provided, an attempt will be made to determine the appropriate value from the system configuration
				if (connection.getKDCAddress() != null && connection.getKDCAddress().length() > 0) {
					gssapiProperties.setKDCAddress(connection.getKDCAddress());
				}
				
				// An optional string which specifies the realm into which the user should authenticate. 
				// If this is not provided, an attempt will be made to determine the appropriate value from the system configuration
				if (connection.getRealm() != null && connection.getRealm().length() > 0) {
					gssapiProperties.setRealm(connection.getRealm());
				}
				
				// list of allowed Quality of Protection mechanisms
				if (qopList.size() > 0) {
					gssapiProperties.setAllowedQoP(qopList);
				}
				
				gssapiProperties.setUseTicketCache(connection.isUseTicketCache());
								
				if (connection.getConfigPath() != null && connection.getConfigPath().length() > 0) {
					gssapiProperties.setConfigFilePath(connection.getConfigPath());
				}
				
				bindRequest = new GSSAPIBindRequest(gssapiProperties);
				break;
		}
		
		
		// String dn = "uid=testadmin,ou=users,dc=testdomain,dc=local";
		//String dn = "testadmin@testdomain";
		//String password = "T@st12345";
		long maxResponseTimeMillis = 5000;

		// connection options
		LDAPConnectionOptions options = new LDAPConnectionOptions();
		options.setFollowReferrals(PreferenceHelper.getInstance().isFollowReferrals());
		options.setReferralHopLimit(10);
		ldapConnection.setConnectionOptions(options);
		
		// debug?
		Debug.setEnabled(PreferenceHelper.getInstance().isDebugLdap());
				
		// bind + login		
		bindRequest.setResponseTimeoutMillis(maxResponseTimeMillis);
		BindResult bindResult = ldapConnection.bind(bindRequest);

		// Check for the password expiring or password expired
		PasswordExpiredControl pwdExpired = PasswordExpiredControl.get(bindResult);
		if (pwdExpired != null) {
			throw new GeneralSecurityException("Your password has expired and you must change your password before you will be allowed to perform any other operations.");
		}

		PasswordExpiringControl pwdExpiring = PasswordExpiringControl.get(bindResult);
		if (pwdExpiring != null) {
			int hours = pwdExpiring.getSecondsUntilExpiration() / 3600;
			logger.warn("Your LDAP password will expire in " + hours + " hours.");
		}
	}

	/**
	 * Closes the connection on the private ldapConnection object
	 */
	public void closeConnection() {
		if (ldapConnection != null) {
			if (ldapConnection.isConnected()) {
				ldapConnection.close();
			}
		}		
	}

	/**
	 * Retrieves the namingContexts attribute from the rootDSE. This is used as initial input for the
	 * TreeViewer in the {@link SearchLdapDialog}.
	 * 
	 * @see SearchLdapDialog#createDialogArea(Composite)
	 */
	public List<LdapContainer> getRootDNs() throws LDAPException {
		getServerType();
		RootDSE rootDSE = ldapConnection.getRootDSE();
		String[] values = rootDSE.getAttributeValues("namingContexts");
		List<LdapContainer> result = new ArrayList<LdapContainer>();
		LdapContainer ldapContainer;

		for (int counter = 0; counter < values.length; counter++) {
			ldapContainer = new LdapContainer();
			ldapContainer.setName(values[counter]);
			ldapContainer.setDn(values[counter]);

			if (values[counter].startsWith("DC")) {
				ldapContainer.setType(Type.DOMAIN);
			} else {
				ldapContainer.setType(Type.OTHER);
			}
			result.add(ldapContainer);
		}

		return result;
	}
	
	/**
	 * Retrieves the namingContexts attribute from the rootDSE. This is used as initial input to get the NETBIOS name
	 * of a domain based on the members DN.
	 * @see {@link #getNetBiosDomainName(String)}
	 */
	public String getConfigurationNamingContext() throws LDAPException {
		getServerType();
		RootDSE rootDSE = ldapConnection.getRootDSE();
		String[] values = rootDSE.getAttributeValues("configurationNamingContext");
		
		if (values.length > 0) {
			return values[0];
		} else {
			return null;
		}
	}
	
	/**
	 * Gets the domain NETBIOS name based on the given DN. Note that the NETBIOS name may be different compared to the
	 * DC attributes in the DN!
	 * Search query being used is (&(nCName="DC=domain,DC=local")(nETBIOSName=*))
	 * @param dn
	 * @return
	 * @throws LDAPException 
	 */
	private String getNetBiosDomainName(String dn) throws LDAPException {
		String netbiosName = null;
		
		if (dn != null) {
			// get the domain component(s) from the DN
			int startIndex = dn.toLowerCase().indexOf("dc=");
			String dc = dn.substring(startIndex);
			
			if (DomainCache.getNetbiosnames().containsKey(dc.toLowerCase())) {
				return DomainCache.getNetbiosnames().get(dc.toLowerCase());
			}
			
			String configNamingContext = getConfigurationNamingContext();
			
			if (configNamingContext != null) {
				Filter dcFilter = Filter.createEqualityFilter("nCName", dc);
								
				SearchRequest searchRequest = new SearchRequest(getConfigurationNamingContext(), SearchScope.SUB, dcFilter, "nETBIOSName");
				SearchResult searchResult = ldapConnection.search(searchRequest);
				
				// found?
				if (searchResult.getSearchEntries() != null && !searchResult.getSearchEntries().isEmpty()) {
					SearchResultEntry domainEntry = searchResult.getSearchEntries().get(0);
					netbiosName = domainEntry.getAttributeValue("nETBIOSName");
				}
			}
		
			if (netbiosName != null) {
				logger.info("Determined netbiosName " + netbiosName + " for " + dc);
			}
			
			DomainCache.getNetbiosnames().put(dc.toLowerCase(), netbiosName);
		}
		
		return netbiosName;
	}

	/**
	 * Gets all child element of a LDAP object, excluding groups and persons. This method is used when a tree item is expanded in the 
	 * SearchLdapDialog.
	 * 
	 * @see SearchLdapDialog#LdapContentProvider
	 */
	public List<LdapContainer> getChildContainers(String parentDN) throws LDAPException {
		Filter excludePersonFilter = Filter.createNOTFilter(Filter.createEqualityFilter("objectCategory", "person"));
		Filter excludeGroupFilter = Filter.createNOTFilter(Filter.createEqualityFilter("objectCategory", "group"));

		SearchRequest searchRequest = new SearchRequest(parentDN, SearchScope.ONE, Filter.createANDFilter(excludePersonFilter, excludeGroupFilter), "CN", "ou", DN_ATTRIBUTE, "objectClass", "dc", "c", "o");

		List<LdapContainer> result = new ArrayList<LdapContainer>();

		// loop through the results using pages of 1000 items
		ASN1OctetString resumeCookie = null;
		while (true) {
			searchRequest.setControls(new SimplePagedResultsControl(1000, resumeCookie));
			SearchResult searchResult = ldapConnection.search(searchRequest);
			for (SearchResultEntry entry : searchResult.getSearchEntries()) {
				result.add(convertToLdapContainer(entry));
			}

			LDAPTestUtils.assertHasControl(searchResult, SimplePagedResultsControl.PAGED_RESULTS_OID);
			SimplePagedResultsControl responseControl = SimplePagedResultsControl.get(searchResult);
			if (responseControl.moreResultsToReturn()) {
				// The resume cookie can be included in the simple paged results
				// control included in the next search to get the next page of results.
				resumeCookie = responseControl.getCookie();
			} else {
				break;
			}
		}
		return result;
	}
	
	/**
	 * Extracts the right attributes from a {@link SearchResultEntry} and builds a {@link LdapContainer} object.
	 * 
	 * @param entry
	 * @return new LdapContainer object containing the ldap attributes from the SearchResultEntry
	 */
	private LdapContainer convertToLdapContainer(SearchResultEntry entry) {
		LdapContainer ldapContainer = new LdapContainer();

		// name
		if (entry.getAttributeValue("CN") != null) {
			ldapContainer.setName(entry.getAttributeValue("CN"));
		} else if (entry.getAttributeValue("ou") != null) {
			ldapContainer.setName(entry.getAttributeValue("ou"));
		} else if (entry.getAttributeValue("dc") != null) {
			ldapContainer.setName(entry.getAttributeValue("dc"));
		} else if (entry.getAttributeValue("o") != null) { //organisation?
			ldapContainer.setName(entry.getAttributeValue("o"));
		} else if (entry.getAttributeValue("c") != null) { //country?
			ldapContainer.setName(entry.getAttributeValue("c"));
		}
		
		// type
		if (entry.getAttributeValues("objectClass") != null) {
			List<String> objectClasses = Arrays.asList(entry.getAttributeValues("objectClass"));
			if (objectClasses.contains("container") || objectClasses.contains("organizationalUnit")) {
				ldapContainer.setType(Type.CONTAINER);
			} else if (objectClasses.contains("domain")) {
				ldapContainer.setType(Type.DOMAIN);
			} else {
				ldapContainer.setType(Type.OTHER);
			}
		} else {
			ldapContainer.setType(Type.OTHER);
		}
		
		// dn
		ldapContainer.setDn(entry.getAttributeValue(DN_ATTRIBUTE));
		
		return ldapContainer;
	}

	/**
	 * Searches the LDAP on CN or OU and the given filter value. Method is used to fill the TableViewer when the user uses the search box 
	 * in the SearchLdapDialog.
	 * 
	 * @param searchText the value to match CN or OU. This filter value may contain the wildcard *
	 * @see SearchLdapDialog#searchLdap(String)
	 */
	public List<LdapContainer> searchContainers(String searchText) throws LDAPException {
		Filter excludePersonFilter = Filter.createNOTFilter(Filter.createEqualityFilter("objectCategory", "person"));
		Filter excludeGroupFilter = Filter.createNOTFilter(Filter.createEqualityFilter("objectCategory", "group"));
		Filter excludeFilter = Filter.createANDFilter(excludePersonFilter, excludeGroupFilter);

		Filter matchCNFilter = Filter.create("(CN=" + searchText + ")");
		Filter matchOUFilter = Filter.create("(OU=" + searchText + ")");
		Filter matchFilter = Filter.createORFilter(matchCNFilter, matchOUFilter);

		SearchRequest searchRequest = new SearchRequest(ldapConnection.getRootDSE().getAttributeValue("defaultNamingContext"), SearchScope.SUB, Filter.createANDFilter(excludeFilter, matchFilter), "CN", "ou", DN_ATTRIBUTE, "objectClass", "dc", "c", "o");
		searchRequest.setControls(new SimplePagedResultsControl(500, null));

		List<LdapContainer> result = new ArrayList<LdapContainer>();

		// loop through the results using pages of 1000 items
		//ASN1OctetString resumeCookie = null;
		//while (true) {
		//	searchRequest.setControls(new SimplePagedResultsControl(1000, resumeCookie));
			SearchResult searchResult = ldapConnection.search(searchRequest);
			for (SearchResultEntry entry : searchResult.getSearchEntries()) {
				result.add(convertToLdapContainer(entry));
			}

		//	LDAPTestUtils.assertHasControl(searchResult, SimplePagedResultsControl.PAGED_RESULTS_OID);
		//	SimplePagedResultsControl responseControl = SimplePagedResultsControl.get(searchResult);
		//	if (responseControl.moreResultsToReturn()) {
				// The resume cookie can be included in the simple paged results
				// control included in the next search to get the next page of results.
		//		resumeCookie = responseControl.getCookie();
		//	} else {
		//		break;
		//	}
		//}

		return result;
	}
		
	public void scanOUs(List<LdapOUInfo> ouInfoList, StatusListener statusListener) {
		totalNumberOfGroups = 0;
		totalNumberOfUsers = 0;
		
		for (LdapOUInfo ouInfo : ouInfoList) {
			
			if (statusListener.isCancelled()) {
				return;
			}
			
			String baseDn = (ouInfo.getBaseOU().equals("") ? "'root'" : ouInfo.getBaseOU());
			if (ouInfo.isEnabled() || ProgramParameters.isForceScanAllLdapContainers() || ProgramParameters.isLdapOUInList(ouInfo.getBaseOU())) {
				try {
					numberOfUsers = 0;
					numberOfGroups = 0;
					logger.info(MessageFormat.format("Start scanning users and groups using connection [{0}] and baseDN [{1}]", ouInfo.getConnection().getConnectionName(), baseDn));
					if (logger.isDebugEnabled()) {
						logger.debug("LDAP container details: " + ouInfo.toString());
					}
					
					openConnection(ouInfo.getConnection());
					processOU(ouInfo, statusListener);
					
					closeConnection();
					
					if (statusListener.isCancelled()) {
						return;
					}
						
				} catch (LDAPException e) {
					String userFriendlyMessage = LdapFactory.getUserFriendlyLdapError(e.toString());
					if (userFriendlyMessage != null) {
						logger.error(MessageFormat.format("Error while scanning users and groups using connection [{0}] and baseDN [{1}]: {2}", ouInfo.getConnection().getConnectionName(), baseDn, userFriendlyMessage), e);
					} else {
						logger.error(MessageFormat.format("Error while scanning users and groups using connection [{0}] and baseDN [{1}]: {2}", ouInfo.getConnection().getConnectionName(), baseDn, e.getMessage()), e);
					}
					statusListener.ldapOUScanned(ouInfo, false);
					
					// reset the (wrong) password, so the user can enter another one
					if (ouInfo.getConnection().isOnlyStorePasswordInMemory()) {
						ouInfo.getConnection().setPassword("");
					}
				} catch (Exception e) {
					logger.error(MessageFormat.format("Error while scanning users and groups using connection [{0}] and baseDN [{1}]: {2}", ouInfo.getConnection().getConnectionName(), baseDn, e.getMessage()), e);
					statusListener.ldapOUScanned(ouInfo, false);
				}
				
				
				// We might not have all the member information, the member and memberOf attributes only delivered the DN of members. If these members we not found in the OU's that have
				// been scanned, then the displayName and other attributes are not filled yet. Lookup those members separately.
				try {
					logger.info(MessageFormat.format("Adding missing LDAP information for group members and memberOf entries", ouInfo.getConnection().getConnectionName(), baseDn));
					
					openConnection(ouInfo.getConnection());
					
					List<String> dnNotFound = new ArrayList<String>();
					
					// New members may have been found in the member and memberOf attributes, search max 10 times to make sure we collect all member info
					for (int counter=0; counter < 10; counter++) {
						if (statusListener.isCancelled()) {
							break;
						}
						
						// Use DISPLAY_NAME_LOWERCASE, because that is indexed!
						List<Member> members = Member.where("DISPLAY_NAME_LOWERCASE IS NULL AND CN IS NOT NULL", null);
						
						if (members.size() > 0 && counter == 9) {
							// processed 10 rounds and we still found new members, report the status and quit
							StringBuilder remainingMembers = new StringBuilder();
							for (Member member : members) {
								remainingMembers.append(member.getCn());
								remainingMembers.append(",");
							}
							Exception ex = new Exception(remainingMembers.toString());
							logger.warn("There are still " + members.size() + " members in the database of which the memberOf and member attributes have not been scanned because they were out of scope.", ex);
							break;
							
						} else if (members.size() == 0) {
							break;
						}
						
						for (Member member : members) {
							if (statusListener.isCancelled()) {
								break;
							}
							
							try {
								// get member DN
								if (member.getRdnId() != null) {
									Rdn rdn = RdnCache.getRdn(member.getRdnId());
									String dn = "CN=" + member.getCn() + "," + rdn.getRdn();
									
									if (dnNotFound.contains(dn)) {
										continue;
									}
									
									statusListener.updateStatus("Looking up member " + dn);
									
									// look up in LDAP
									SearchResultEntry entry = ldapConnection.getEntry(dn, LDAP_ATTRIBUTES);
									
									// add the LDAP info to the member and save it in the database
									member = createOrUpdateMember(entry, dn, member.getCn(), member.getRdnId(), member.getDomainId(), member, ouInfo.getSearchObjectType().equals(SearchObjectType.GROUPS));
									
									if (entry == null) {
										logger.warn("New member was found in a memberOf or members attribute, but the full DN was not found: " + dn);
										dnNotFound.add(dn);
										continue;
									}
									
									// the member might be null in case we only scan groups and the member appeared to be a user
									if (member != null) {
										// save membership information
										// use the member attribute for groups
										if (member.isGroup()) {
											processMemberAttribute(member, dn, GROUP_MEMBER_ATTRIBUTE, entry, false, ouInfo.getSearchObjectType().equals(SearchObjectType.GROUPS), statusListener);
										}
										
										// use the memberOf attribute for both users and groups
										processMemberAttribute(member, dn, MEMBEROF_ATTRIBUTE, entry, true, ouInfo.getSearchObjectType().equals(SearchObjectType.GROUPS), statusListener);
									}
								}
							} catch (Exception ex) {
								logger.error("Error while adding LDAP information to member " + member.toString() + ": " + ex.getMessage(), ex);
							}
						}				
					}
					
					if (Member.isTotalNumberOfUsersReached() && Member.isTotalNumberOfGroupsReached()) {
						break;
					}
					
					logger.info(MessageFormat.format("Done scanning users and groups using connection [{0}] and baseDN [{1}]. Found {2} users and {3} groups.", ouInfo.getConnection().getConnectionName(), baseDn, numberOfUsers, numberOfGroups));
					statusListener.ldapOUScanned(ouInfo, true);
				} catch (Exception e) {
					logger.error("Error while adding LDAP information to users and groups: " + e.getMessage(), e);
					statusListener.ldapOUScanned(ouInfo, false);
				} finally {
					closeConnection();
				}
			}
			
		} // next OU
		
		// now scan all the groups
//		for (LdapOUInfo ouInfo : ouInfoList) {
//			
//			if (statusListener.isCancelled()) {
//				return;
//			}
//			
//			String baseDn = (ouInfo.getBaseOU().equals("") ? "'root'" : ouInfo.getBaseOU());
//			if (ouInfo.isEnabled()) {
//				try {
//					if (ouInfo.getSearchObjectType().equals(SearchObjectType.USERS_GROUPS) || ouInfo.getSearchObjectType().equals(SearchObjectType.GROUPS)) {
//						numberOfGroups = 0;
//						logger.info(MessageFormat.format("Start scanning groups using connection [{0}] and baseDN [{1}]", ouInfo.getConnection().getConnectionName(), baseDn));
//						if (logger.isDebugEnabled()) {
//							logger.info("LDAP container details: " + ouInfo.toString());
//						}
//						
//						openConnection(ouInfo.getConnection());
//						processOU(ouInfo.getBaseOU(), ouInfo.getSearchScope(), false, true, statusListener);
//						
//						logger.info(MessageFormat.format("Done scanning groups using connection [{0}] and baseDN [{1}]. Found {2} groups.", ouInfo.getConnection().getConnectionName(), baseDn, numberOfGroups));
//						statusListener.ldapOUScanned(ouInfo, true);
//						
//						closeConnection();
//						
//						if (statusListener.isCancelled()) {
//							return;
//						}
//						
//						if (Member.isTotalNumberOfGroupsReached()) {
//							break;
//						}
//					}			
//				} catch (LDAPException e) {
//					String userFriendlyMessage = LdapFactory.getUserFriendlyLdapError(e.toString());
//					if (userFriendlyMessage != null) {
//						logger.error(MessageFormat.format("Error while scanning groups using connection [{0}] and baseDN [{1}]: {2}", ouInfo.getConnection().getConnectionName(), baseDn, userFriendlyMessage));
//						logger.error("Error while scanning groups", e);
//					} else {
//						logger.error(MessageFormat.format("Error while scanning groups using connection [{0}] and baseDN [{1}]: {2}", ouInfo.getConnection().getConnectionName(), baseDn, e.getMessage()));
//						logger.error("Error while scanning groups", e);
//					}
//					statusListener.ldapOUScanned(ouInfo, false);
//				} catch (Exception e) {
//					logger.error(MessageFormat.format("Error while scanning groups using connection [{0}] and baseDN [{1}]: {2}", ouInfo.getConnection().getConnectionName(), baseDn, e.getMessage()));
//					logger.error("Error while scanning groups", e);
//					statusListener.ldapOUScanned(ouInfo, false);
//				}
//			}
//		}

		if (totalNumberOfUsers > 0 || totalNumberOfGroups > 0) {
			statusListener.usersScanned(totalNumberOfUsers);
			statusListener.groupsScanned(totalNumberOfGroups);
		}
		
		// clear the member cache by CN, we don't need it anymore
		MemberCache.clearCacheByCN();
	}
	
	public void testOU(LdapOUInfo ouInfo) throws LDAPException, GeneralSecurityException {
		SearchRequest searchRequest = new SearchRequest(ouInfo.getBaseOU(), SearchScope.BASE, Filter.createEqualityFilter("objectCategory", "person"), LDAP_ATTRIBUTES);
		ldapConnection.search(searchRequest);
		//no exception at this point, test successful
	}
	
	/**
	 * Searches for all users or groups in the given baseDn with the given scope. Every member is saved in the database and all the members of a group and the memberOf members
	 * are saved as well (including a record in the membership table).
	 *  
	 * @param baseDn DN to search from
	 * @param searchScope scope, like first level subs, all subs, only container etc...
	 * @param processUsers indicates that only users are being processed. Users and groups are scanned separately for performance reasons. 
	 * 			If we don't, we would get users by the member attribute of a group, which requires separate ldap queries on DN. 
	 * 			Now we first scan all members, cache most of them, and then scan the groups. Most of the members of the groups are already saved in the database and cache.
	 * @param processGroups indicates if only groups are being scanned. Either processGroups or processUsers can be true.
	 * 
	 * @throws LDAPException
	 */
	private void processOU(LdapOUInfo ouInfo, StatusListener statusListener) throws LDAPException, DBException {
		Filter userFilter;
		Filter groupFilter;
		
		// see http://social.technet.microsoft.com/wiki/contents/articles/5392.active-directory-ldap-syntax-filters.aspx#Filter_on_objectCategory_and_objectClass
		// Filter for users
		if (getServerType().equals(LdapServerType.ACTIVE_DIRECTORY)) {
			userFilter = Filter.createEqualityFilter("sAMAccountType", "805306368"); //all users
		} else {
			Filter categoryFilter = Filter.createEqualityFilter("objectCategory", "person");
			Filter classFilter = Filter.createEqualityFilter("objectClass", "user");
			userFilter = Filter.createANDFilter(categoryFilter, classFilter);
		}

		// Filter for groups
		Filter category = Filter.createEqualityFilter("objectCategory", "group");
		// Filter includeLocalGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(LOCAL_SECURITY_GROUP));
		// Filter includeGlobalGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(GLOBAL_SECURITY_GROUP));
		// Filter includeUniversalGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(UNIVERSAL_SECURITY_GROUP));
		// Filter includeBuiltInGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(DOMAIN_BUILTIN_SECURITY_GROUP));
		// Filter includeGroupFilter = Filter.createANDFilter(category, Filter.createORFilter(includeLocalGroupFilter, includeGlobalGroupFilter, includeUniversalGroupFilter, includeBuiltInGroupFilter));
		groupFilter = Filter.createANDFilter(category, Filter.create("groupType:1.2.840.113556.1.4.803:=2147483648")); // all security groups
				
		SearchScope ldapSearchScope = null;
		switch (ouInfo.getSearchScope()) {
			case ONLY_CONTAINER: ldapSearchScope = SearchScope.BASE; break;
			case ONE_LEVEL_SUBS_NOT_CONTAINER_ITSELF: ldapSearchScope = SearchScope.ONE; break;
			case ONLY_SUBS: ldapSearchScope = SearchScope.SUBORDINATE_SUBTREE; break;
			case CONTAINER_AND_SUBS: ldapSearchScope = SearchScope.SUB; break;
		}
		
		SearchRequest searchRequest;
		
		if (ouInfo.getSearchObjectType().equals(SearchObjectType.USERS_GROUPS)) {
			// get groups and users
			searchRequest = new SearchRequest(ouInfo.getBaseOU(), ldapSearchScope, Filter.createORFilter(userFilter, groupFilter), LDAP_ATTRIBUTES);
		} else if (ouInfo.getSearchObjectType().equals(SearchObjectType.USERS)) {
			// just get users
			searchRequest = new SearchRequest(ouInfo.getBaseOU(), ldapSearchScope, userFilter, LDAP_ATTRIBUTES);
		} else {
			// just get groups
			searchRequest = new SearchRequest(ouInfo.getBaseOU(), ldapSearchScope, groupFilter, LDAP_ATTRIBUTES);
		}
		
		// determine the domain name for all groups found in this baseDN
		String domainName = getDomain(ouInfo.getBaseOU());
		int domainId = DomainCache.getDomainId(domainName);
		
		// loop through the results using pages of 1000 items
		ASN1OctetString resumeCookie = null;
		while (true) {
			searchRequest.setControls(new SimplePagedResultsControl(1000, resumeCookie));
			SearchResult searchResult = ldapConnection.search(searchRequest);
			Rdn rdn = null;
			for (SearchResultEntry entry : searchResult.getSearchEntries()) {
				boolean isGroup = entry.getAttributeValue(GROUP_TYPE_ATTRIBUTE) != null;
				
				//cancelled by user?
				if (statusListener != null && statusListener.isCancelled()) {
					logger.info("Scan process canncelled by the user");
					return;
				}
				
				//license limit of maximum number of groups reached?
				if ((isGroup && Member.isTotalNumberOfGroupsReached()) || (!isGroup && Member.isTotalNumberOfUsersReached())) {
					return;
				}
				
				// group attributes
				String memberDN = entry.getAttributeValue(DN_ATTRIBUTE);
				String memberCN = getCnFromDn(memberDN);
				
				statusListener.updateStatus("Scanning user/group in LDAP container " + ouInfo.getBaseOU() + ": " + memberDN);
								
				// RDN of this group, same as the previous one?
				String rdnValue = getRdnFromDn(memberDN);
				if (rdn == null || !rdn.getRdn().equals(rdnValue)) {
					//already in DB? If not, it will be created
					rdn = RdnCache.getRdn(rdnValue);
				}
				
				// TODO member.getId() can be null in case the license limit has been reached?
				// The member id may be null in case the license limit has been reached and the member is not saved in the database
				// does this group or user already exists?
				Member member = createOrUpdateMember(entry, memberDN, memberCN, rdn.getId(), domainId, null, false);
				
				// in case of a user get the primaryGroupId
				if (!isGroup) {
					byte[] userSid = entry.getAttributeValueBytes(OBJECT_SID_ATTRIBUTE);
					String userSidString = convertSidToStringSid(userSid);
					String primaryGroupId = entry.getAttributeValue(PRIMARY_GROUP_ID_ATTRIBUTE);
					
					if (logger.isTraceEnabled()) {
						logger.trace("user SID: " + userSidString);
						logger.trace("PrimaryGroupID: " + primaryGroupId);
					}
					
					// -----------------------------------------------------------------
					// The primaryGroupId is the relative identifier (RID) of the group object's security identifier (SID). For example, Domain Users has a SID, 
					// S-1-5-21-1921824074-547830868-893353407-1105 the RID is, 1105 (the right-most portion of the SID) the primaryGroupId for my users is, 1105
					// The binary expression of a SID has the following format:		            
					//		byte[0] - Revision Level
					//		byte[1] - count of Sub-Authorities
					//		byte[2-7] - 48 bit Authority (big-endian)
					//		...and then n Sub-Authorities, 32 bits each (little-endian)
					//
					//		e.g. -
					//		[1,5,0,0,0,0,0,5,21,0,0,0,37,-20,73,58,97,-107,0,-80,109,-55,112,10,47,-24,5,0]                
					//		            
					//		            
					// The last sub-authority of a SID is known as the RID (Relative Identifier), and it is this RID that differentiates objects from within the same domain.
					// This basically means that by replacing the RID in an SID you can generate the SID for a different object. The 'primaryGroupID' attribute from the 'user' class is a RID. 
					// So, we can take the SID of the user, and replace the RID part with the primary group id, we can then lookup the group in LDAP using this SID as the key.
					// -----------------------------------------------------------------
					if (primaryGroupId != null) {
						String primaryGroupSid = userSidString.substring(0, userSidString.lastIndexOf('-') + 1) + primaryGroupId;
												
						if (!primaryGroupCache.containsKey(primaryGroupSid)) {
							try {
								// search the LDAP entity based on the primary group's sid
								Filter categoryGroupFilter = Filter.createEqualityFilter("objectCategory", "group");
								Filter sidFilter = Filter.createEqualityFilter("objectSid", primaryGroupSid);
								SearchRequest sidSearchRequest = new SearchRequest(ldapConnection.getRootDSE().getAttributeValue("defaultNamingContext"), SearchScope.SUB, Filter.createANDFilter(categoryGroupFilter, sidFilter), LDAP_ATTRIBUTES);
								SearchResult sidSearchResult = ldapConnection.search(sidSearchRequest);
								
								// create a database record for this LDAP entry
								if (sidSearchResult.getSearchEntries() != null && !sidSearchResult.getSearchEntries().isEmpty()) {
									SearchResultEntry primaryGroupEntry = sidSearchResult.getSearchEntries().get(0);
									
									String primaryGroupDN = primaryGroupEntry.getAttributeValue(DN_ATTRIBUTE);
									String primaryGroupCN = getCnFromDn(primaryGroupDN);
									
									String primaryGroupDomain = getDomain(primaryGroupDN);
									String primaryGroupRdn = getRdnFromDn(primaryGroupDN);
									int primaryGroupDomainId = DomainCache.getDomainId(primaryGroupDomain);
									int rdnprimaryGroupRdnId = RdnCache.getRdn(primaryGroupRdn).getId();
									
									Member primaryGroupMember = createOrUpdateMember(primaryGroupEntry, primaryGroupDN, primaryGroupCN, rdnprimaryGroupRdnId, primaryGroupDomainId, null, false);
									primaryGroupCache.put(primaryGroupSid, primaryGroupMember.getId());
									
								} else {
									logger.warn("Primary group of user " + memberDN + " not found: " + primaryGroupSid);
									// cache the failure
									primaryGroupCache.put(primaryGroupSid, null);
								}
							} catch (Exception ex) {
								logger.warn("Primary group of user " + memberDN + " not found: " + primaryGroupSid);
								// cache the failure
								primaryGroupCache.put(primaryGroupSid, null);
							}
						}
						
						// found in cache or LDAP?
						Integer primaryGroupIdInCache = primaryGroupCache.get(primaryGroupSid);
						
						// this id might be null in case the SID lookup failed (to prevent thousands of failing lookups)
						// member.getId() is null when the license limit has been reached
						if (primaryGroupIdInCache != null && member.getId() != null) {
							Membership membership = new Membership(primaryGroupIdInCache, member.getId());
							if (!MembershipCache.membershipExists(membership)) {
								membership.save();
								MembershipCache.cacheMembership(membership);
							}
						}
					}
				}
				
				// save membership information
				// use the member attribute for groups
				if (isGroup) {
					processMemberAttribute(member, memberDN, GROUP_MEMBER_ATTRIBUTE, entry, false, ouInfo.getSearchObjectType().equals(SearchObjectType.GROUPS), statusListener);
				}
				
				// use the memberOf attribute for both users and groups
				processMemberAttribute(member, memberDN, MEMBEROF_ATTRIBUTE, entry, true, false, statusListener);
			}
			
			LDAPTestUtils.assertHasControl(searchResult, SimplePagedResultsControl.PAGED_RESULTS_OID);
			SimplePagedResultsControl responseControl = SimplePagedResultsControl.get(searchResult);
			if (responseControl.moreResultsToReturn()) {
				// The resume cookie can be included in the simple paged results
				// control included in the next search to get the next page of results.
				resumeCookie = responseControl.getCookie();
			} else {
				break;
			}
		}
	}
	
//	/**
//	 * Searches for all groups in the given baseDn with the given scope. Every group is saved in the database and all its members are saved in the membership table.
//	 *  
//	 * @param baseDn DN to search from
//	 * @param searchScope scope, like first level subs, all subs, only container etc...
//	 * 
//	 * @throws LDAPException
//	 */
//	private void processGroups(String baseDn, SearchScope searchScope, StatusListener statusListener) throws LDAPException, DBException {
//		Filter category = Filter.createEqualityFilter("objectCategory", "group");
//		Filter includeLocalGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(LOCAL_SECURITY_GROUP));
//		Filter includeGlobalGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(GLOBAL_SECURITY_GROUP));
//		Filter includeUniversalGroupFilter = Filter.createEqualityFilter(GROUP_TYPE_ATTRIBUTE, String.valueOf(UNIVERSAL_SECURITY_GROUP));
//		Filter includeGroupFilter = Filter.createANDFilter(category, Filter.createORFilter(includeLocalGroupFilter, includeGlobalGroupFilter, includeUniversalGroupFilter));
//		
//		SearchScope ldapSearchScope = null;
//		switch (searchScope) {
//			case ONLY_CONTAINER: ldapSearchScope = SearchScope.BASE; break;
//			case ONE_LEVEL_SUBS_NOT_CONTAINER_ITSELF: ldapSearchScope = SearchScope.ONE; break;
//			case ONLY_SUBS: ldapSearchScope = SearchScope.SUBORDINATE_SUBTREE; break;
//			case CONTAINER_AND_SUBS: ldapSearchScope = SearchScope.SUB; break;
//		}
//		
//		SearchRequest searchRequest = new SearchRequest(baseDn, ldapSearchScope, includeGroupFilter, LDAP_ATTRIBUTES);
//		
//		
//		// determine the domain name for all groups found in this baseDN
//		String domainName = getDomain(baseDn);
//		int domainId = DomainCache.getDomainId(domainName);
//		
////		Domain domain = Domain.findFirst("domain = ?", domainName);
////		if (domain == null) {
////			domain = new Domain(domainName);
////			domain.save();
////		}
//		
//		// loop through the results using pages of 1000 items
//		ASN1OctetString resumeCookie = null;
//		while (true) {
//			searchRequest.setControls(new SimplePagedResultsControl(1000, resumeCookie));
//			SearchResult searchResult = ldapConnection.search(searchRequest);
//			Rdn rdn = null;
//			for (SearchResultEntry entry : searchResult.getSearchEntries()) {
//				numberOfGroups++;
//								
//				//cancelled by user?
//				if (statusListener != null && statusListener.isCancelled()) {
//					logger.info("Scan process canncelled by the user");
//					return;
//				}
//				
//				//license limit of maximum number of groups reached?
//				if (Member.isTotalNumberOfGroupsReached()) {
//					return;
//				}
//				
//				// group attributes
//				String groupDN = entry.getAttributeValue(DN_ATTRIBUTE);
//				String groupCN = getCnFromDn(groupDN);
//				
//				statusListener.updateStatus("Scanning groups in LDAP container " + baseDn + ": " + groupDN);
//				
//				// RDN of this group, same as the previous one?
//				String rdnValue = getRdnFromDn(groupDN);
//				if (rdn == null || !rdn.getRdn().equals(rdnValue)) {
//					//already in DB? If not, it will be created
//					rdn = RdnCache.getRdn(rdnValue);
//				}
//				
//				// does this group already exists?
//				Member group = createOrUpdateMember(entry, null, groupCN, rdn.getId(), domainId);
//				
//				// save membership information (both the member attribute as the memberOf attribute)
//				processMemberAttribute(group, groupDN, GROUP_MEMBER_ATTRIBUTE, entry, false, statusListener);			
//				processMemberAttribute(group, groupDN, MEMBEROF_ATTRIBUTE, entry, true, statusListener);
//			}
//			
//			LDAPTestUtils.assertHasControl(searchResult, SimplePagedResultsControl.PAGED_RESULTS_OID);
//			SimplePagedResultsControl responseControl = SimplePagedResultsControl.get(searchResult);
//			if (responseControl.moreResultsToReturn()) {
//				// The resume cookie can be included in the simple paged results
//				// control included in the next search to get the next page of results.
//				resumeCookie = responseControl.getCookie();
//			} else {
//				break;
//			}
//		}
//	}
	
	/**
	 * This method handles logic to read an LDAP attribute that contains more than 1500 values. In that case LDAP Range logic must be applied to get the attributes in pages.
	 * The method is used for the attributes member and memberOf. This first containing all members of a group, and the second all the group memberships for a user/group. It creates
	 * a Membership record for every member that is being found.
	 * 
	 * @param memberBeingProcessed
	 * @param memberBeingProcessedDN
	 * @param memberAttribute
	 * @param entry
	 * @param isMemberOf 
	 * @param onlyScanGroups
	 * @param statusListener
	 * @throws DBException
	 * @throws LDAPException
	 * 
	 * @see {@link #addMembership(Member, String, boolean, StatusListener)}
	 */
	private void processMemberAttribute(Member memberBeingProcessed, String memberBeingProcessedDN, String memberAttribute, SearchResultEntry entry, boolean isMemberOf, boolean onlyScanGroups, StatusListener statusListener) throws DBException, LDAPException {
		String[] members = entry.getAttributeValues(memberAttribute);
				
		if (members != null && members.length > 0) {
			for (String memberDN : members) {
				addMembership(memberBeingProcessed, memberDN, isMemberOf, onlyScanGroups, statusListener);
			}
		} else {
			// the member attribute may contain a large amount of values, in which case the member attribute is returned with a name like
			// member;Range=0-1499 (depending on the server implementation)
			int range = 0;
			for (Attribute attribute : entry.getAttributes()) {
				if (attribute.getName().toLowerCase().startsWith(memberAttribute + ";range")) {
					// the range (like 0-1499) is mentioned in the attribute name and depends on LDAP implementation
					range = Integer.parseInt(attribute.getName().substring(attribute.getName().indexOf('-') + 1));
					break;
				}
			}
				 
			if (range != 0) {
				// process the fist set of members for this group
				for (String memberDN : entry.getAttributeValues(memberAttribute + ";range=0-" + range)) {
					addMembership(memberBeingProcessed, memberDN, isMemberOf, onlyScanGroups, statusListener);
				}
				
				// read the next set of member values
				boolean moreMembers = true;
				int offset = range + 1;
				while (moreMembers) {
					
					//cancelled by user?
					if (statusListener != null && statusListener.isCancelled()) {
						logger.info("Scan process canncelled by the user");
						return;
					}
					
					String newRangeAttribute = memberAttribute + ";range=" + offset + "-" + (offset + range);
					SearchResultEntry nextMemberSetEntry = ldapConnection.getEntry(memberBeingProcessedDN, newRangeAttribute);
					if (nextMemberSetEntry != null) {
						if (nextMemberSetEntry.getAttributeValues(newRangeAttribute) != null) {
							// process this next set of member values
							for (String memberDN : nextMemberSetEntry.getAttributeValues(newRangeAttribute)) {
								addMembership(memberBeingProcessed, memberDN, isMemberOf, onlyScanGroups, statusListener);
							}
						} else if (nextMemberSetEntry.getAttributeValues(memberAttribute + ";range=" + offset + "-*") != null) {
							// with the * character the server is indicating that this is the last set of member values
							moreMembers = false;
							for (String memberDN : nextMemberSetEntry.getAttributeValues(memberAttribute + ";range=" + offset + "-*")) {
								addMembership(memberBeingProcessed, memberDN, isMemberOf, onlyScanGroups, statusListener);
							}
						} else {
							//this should never occur
							logger.error(MessageFormat.format("Next set of member values for group {0} did not return a value, attribute name: {1}.", memberBeingProcessedDN, newRangeAttribute));
							moreMembers = false;
						}								
					} else {
						logger.error(MessageFormat.format("Could not find group DN {0} while retrieving more values for the {1} attribute ({2}).", memberBeingProcessedDN, memberAttribute, newRangeAttribute));
						moreMembers = false;
					}
					
					offset += range + 1;
				}
			}
		}	
	}
	
//	/**
//	 * Searches for all users in the given baseDn with the given scope. Every user is saved in the database.
//	 *  
//	 * @param baseDn DN to search from
//	 * @param searchScope scope, like first level subs, all subs, only container etc...
//	 * 
//	 * @throws LDAPException
//	 */
//	private void processUsers(String baseDn, SearchScope searchScope, StatusListener statusListener) throws LDAPException, DBException {
//		Filter userFilter;
//		if (getServerType().equals(LdapServerType.ACTIVE_DIRECTORY)) {
//			userFilter = Filter.createEqualityFilter("sAMAccountType", "805306368"); //all users
//		} else {
//			Filter categoryFilter = Filter.createEqualityFilter("objectCategory", "person");
//			Filter classFilter = Filter.createEqualityFilter("objectClass", "user");
//			userFilter = Filter.createANDFilter(categoryFilter, classFilter);
//		}
//		
//		SearchScope ldapSearchScope = null;
//		switch (searchScope) {
//			case ONLY_CONTAINER: ldapSearchScope = SearchScope.BASE; break;
//			case ONE_LEVEL_SUBS_NOT_CONTAINER_ITSELF: ldapSearchScope = SearchScope.ONE; break;
//			case ONLY_SUBS: ldapSearchScope = SearchScope.SUBORDINATE_SUBTREE; break;
//			case CONTAINER_AND_SUBS: ldapSearchScope = SearchScope.SUB; break;
//		}
//		
//		SearchRequest searchRequest = new SearchRequest(baseDn, ldapSearchScope, userFilter, LDAP_ATTRIBUTES);
//		
//		
//		// determine the domain name for all users found in this baseDN
//		String domainName = getDomain(baseDn);	
//		int domainId = DomainCache.getDomainId(domainName);
////		Domain domain = Domain.findFirst("domain = ?", domainName);
////		if (domain == null) {
////			domain = new Domain(domainName);
////			domain.save();
////		}
//				
//		// loop through the results using pages of 1000 items
//		ASN1OctetString resumeCookie = null;
//		while (true) {
//			searchRequest.setControls(new SimplePagedResultsControl(1000, resumeCookie));
//			SearchResult searchResult = ldapConnection.search(searchRequest);
//			Rdn rdn = null;
//			
//			for (SearchResultEntry entry : searchResult.getSearchEntries()) {
//				numberOfUsers++;
//				
//				//cancelled by user?
//				if (statusListener.isCancelled()) {
//					logger.info("Scan process canncelled by the user");
//					return;
//				}
//				
//				//license limit of maximum number of users reached?
//				if (Member.isTotalNumberOfUsersReached()) {
//					return;
//				}
//								
//				// user attributes
//				String userDN = entry.getAttributeValue(DN_ATTRIBUTE);
//				String userCN = getCnFromDn(userDN);
//				
//				//TODO what is the performance impact of the status messages?
//				statusListener.updateStatus("Scanning users in LDAP container " + baseDn + ": " + userDN);
//				
//				// RDN of this user, same as the previous one?
//				String rdnValue = getRdnFromDn(userDN);
//				if (rdn == null || !rdn.getRdn().equals(rdnValue)) {
//					//already in DB? If not, it will be created in the db
//					rdn = RdnCache.getRdn(rdnValue);
//				}
//				
//				// create or update the user info?
//				Member user = createOrUpdateMember(entry, null, userCN, rdn.getId(), domainId);
//				
//				// process the membership using the memberOf attribute
//				processMemberAttribute(user, userDN, MEMBEROF_ATTRIBUTE, entry, true, statusListener);
//			}
//			
//			LDAPTestUtils.assertHasControl(searchResult, SimplePagedResultsControl.PAGED_RESULTS_OID);
//			SimplePagedResultsControl responseControl = SimplePagedResultsControl.get(searchResult);
//			if (responseControl.moreResultsToReturn()) {
//				// The resume cookie can be included in the simple paged results
//				// control included in the next search to get the next page of results.
//				resumeCookie = responseControl.getCookie();
//			} else {
//				break;
//			}
//		}
//	}
	
	/**
	 * Extracts the RDN from a DN. So the DN <code>CN=JohnDoe,OU=Admins,OU=Users,C=US,DC=domain,DC=com</code> would result in the RDN
	 * <code>OU=Admins,OU=Users,C=US,DC=domain,DC=com</code>
	 * 
	 * @param dn distinguishedName
	 * @return rdn value
	 */
	private String getRdnFromDn(String dn) {
		return dn.substring(dn.indexOf(',') + 1);
	}
	
	/**
	 * Extracts the CN from a DN. So the DN <code>CN=JohnDoe,OU=Admins,OU=Users,C=US,DC=domain,DC=com</code> would result in the CN
	 * <code>JohnDoe</code>
	 * 
	 * @param dn distinguishedName
	 * @return cn value
	 */
	private String getCnFromDn(String dn) {
		return dn.substring(dn.indexOf('=') + 1, dn.indexOf(','));
	}
	
	/**
	 * Extracts the domain for the given baseDN. So the baseDN <code>OU=Admins,OU=Users,C=US,DC=domain,DC=com</code> would result in the domain String
	 * <code>domain</code>
	 * @param baseDn
	 * @return
	 */
	private String getDomain(String baseDn) {
		baseDn = baseDn.toUpperCase();
		int index = baseDn.indexOf("DC=");
		
		if (index >= 0) {
			int endIndex = (baseDn.indexOf(",", index + 1));
			
			if (endIndex > 0) {
				return baseDn.substring(index + 3, endIndex); 
			} else {
				return baseDn.substring(index + 3);
			}
		} else {
			return "";
		}
	}

	/**
	 * Updates the given (@link Member} object based on the given {@link SearchResultEntry} by copying ldap attributes. It first checks if the member already exists in the database, by
	 * checking the combination CN / RDN. This because the members of a group only have the DN's. If not found, it uses the SearchResultEntry entry to retrieve the account name
	 * (sAMAccountName) and it will check if the combination account name / domain already exists. If not, it creates a new member in the database.
	 * 
	 * @param entry Search result containing all the LDAP attributes
	 * @param dn only required when the entry is null. It will then be used to lookup the member details.
	 * @param cn common name used to check if it already exists
	 * @param rdnId rdn combined with the cn for a unique combination
	 * @param domainId domain to check if the combination account name + domain already exists in the database
	 * @param member the member object may already be known, for example when we loop through all members with an empty display name (see scanOUs).
	 * 					in that case we just add the LDAP info to the member and we save it in the database.
	 * @param onlyScanGroups true in case the OU in the GUI has been configured to scan only groups.
	 * 
	 * @return updated or created Member object
	 * 
	 * @throws DBException
	 * @throws LDAPException
	 */
	private Member createOrUpdateMember(SearchResultEntry entry, String dn, String cn, int rdnId, int domainId, Member member, boolean onlyScanGroups) throws DBException, LDAPException {
		// does this member already exists? Check on CN because we might not have the LDAP entry (when processing the member or memberOf attribute for example)
		if (member == null) {
			// if we have an LDAP entry we also check on account name (in case the account has already been found on a file system)
			if (entry != null) {
				String accountName = entry.getAttributeValue(ACCOUNT_NAME_ATTRIBUTE);
				member = MemberCache.getMemberByCN(cn, rdnId, accountName, domainId);
			} else {
				member = MemberCache.getMemberByCN(cn, rdnId, null, 0);
			}
		}

		//if (entry == null) {
		//	member = MemberCache.getMemberByCN(cn, rdnId);
		//}
		
		/* DON'T RETRIEVE INFO FROM LDAP, DO THAT AT THE END OF THE SCAN.
		if (member == null || member.getName(false) == null) {
			
			if (entry == null) {
				entry = ldapConnection.getEntry(dn, LDAP_ATTRIBUTES);
			}
			
			if (entry == null) {
				throw new LDAPException(null, MessageFormat.format("Could not find details for DN {0}. This DN was found as member of a group.", dn));
			}
			
			// not by cn, perhaps by account name?
			String accountName = entry.getAttributeValue(ACCOUNT_NAME_ATTRIBUTE);
			if (member == null) {
				member = MemberCache.getMemberByName(accountName, domainId);
			}
			
			// still not found?
			if (member == null) {
				member = new Member();
			}	
			
			member.setCn(cn);
			member.setRdnId(rdnId);
			member.setDomainId(domainId);	
		
			// copy the attributes from the ldap entry
			member.setName(entry.getAttributeValue(ACCOUNT_NAME_ATTRIBUTE));
						
			String groupType = entry.getAttributeValue(GROUP_TYPE_ATTRIBUTE);
			if (LOCAL_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.DOMAIN_LOCAL);
			} else if (GLOBAL_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.GLOBAL);
			} else if (UNIVERSAL_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.UNIVERSAL);
			}
			member.setGroup(groupType != null && groupType.length() > 0);
			
			if (member.isGroup()) {
				member.setDisplayName(entry.getAttributeValue(GROUP_DISPLAY_NAME_ATTRIBUTE));
			} else {
				member.setDisplayName(entry.getAttributeValue(USER_DISPLAY_NAME_ATTRIBUTE));
			}
			
			member.save();
			
			// cache for the next time
			MemberCache.cacheMemberByCN(member);
						
			if (logger.isTraceEnabled()) {
				logger.trace(MessageFormat.format("Updated member object based on ldap entry: {0}", member));
			}
		} */
				
		// not in DB yet?
		if (member == null) { 
			member = new Member();
			member.setCn(cn);
			
			String netbiosName = getNetBiosDomainName(dn);
			if (netbiosName != null) {
				// use the netbios name
				domainId = DomainCache.getDomainId(netbiosName);
			} else {
				// use the domain id given based on the OU that is being scanned
			}
			member.setRdnId(rdnId);
			member.setDomainId(domainId);	
		}
		
		// missing LDAP info?
		if (member.getDisplayName() == null && entry != null) {
			
			// copy the attributes from the ldap entry
			if (entry.getAttributeValue(ACCOUNT_NAME_ATTRIBUTE) != null) {
				member.setName(entry.getAttributeValue(ACCOUNT_NAME_ATTRIBUTE));
			} else {
				// it sometimes happens that the CN (which is extracted from the DN) contains a SID. Try to resolve it to a name:
				if (JnaAceReader.isLocalBuiltinAccount(cn)) {
					member.setName(JnaAceReader.getBuiltinAccountName(cn));
					member.setCn(JnaAceReader.getBuiltinAccountName(cn));
					member.setBuiltIn(true);
				} else {
					member.setName(cn);
				}
			}			
			
			// reset the group type for a user, because this flag might have been set to COMPUTER_LOCAL in the LocalGroupHelper
			// we now discover that this user is a domain user and not a local computer user
			if (!member.isGroup()) {
				member.setGroupType(GroupType.NONE);
			}
			
			String groupType = entry.getAttributeValue(GROUP_TYPE_ATTRIBUTE);
			if (LOCAL_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.DOMAIN_LOCAL);
			} else if (GLOBAL_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.GLOBAL);
			} else if (UNIVERSAL_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.UNIVERSAL);
			} else if (DOMAIN_BUILTIN_SECURITY_GROUP.equals(groupType)) {
				member.setGroupType(GroupType.DOMAIN_BUILTIN);
				member.setBuiltIn(true);
			}
			member.setGroup(groupType != null && groupType.length() > 0);
									
			String displayName;
			if (member.isGroup()) {
				displayName = entry.getAttributeValue(GROUP_DISPLAY_NAME_ATTRIBUTE);
			} else {
				displayName = entry.getAttributeValue(USER_DISPLAY_NAME_ATTRIBUTE);
			}
			member.setDisplayName(displayName != null ? displayName : "");
			
			// enabled/disabled status
			Integer userAccountControl = entry.getAttributeValueAsInteger(USER_ACCOUNT_CONTROL);
			member.setEnabled(userAccountControl == null || (userAccountControl & USER_DISABLED_BIT) == 0); //disabled bit is not set
			
			if (!member.isGroup() && onlyScanGroups) {
				if (logger.isTraceEnabled()) {
					logger.trace(MessageFormat.format("Deleting member because it's a user: {0}", member));
				}
				Membership.delete("MEMBER_ID = ?", member.getId());
				MemberCache.clearMemberByCN(member.getCn(), member.getRdnId());
				member.delete();
				member = null; //don't save
				numberOfUsers--;
				totalNumberOfUsers--;
			}
		}
		
		if (member != null) {
			
			// new member?
			if (member.getId() == null) {
				if (member.isGroup()) {
					numberOfGroups++;
					totalNumberOfGroups++;
				} else {
					numberOfUsers++;
					totalNumberOfUsers++;
				}
			}
						
			member.save();
			
			// cache for the next time
			MemberCache.cacheMemberByCN(member);
						
			if (logger.isTraceEnabled()) {
				logger.trace(MessageFormat.format("Updated member object based on ldap entry: {0}", member));
			}	
		}
		
		return member;
	}
	
	/**
	 * Create a new record in the Membership table. It checks if the given member (CN + RDN ID) already exists in the database. If not, it will retrieve additional member information
	 * based on the <code>memberDN</code>. With this additional ldap information it creates a new Member object and saves it in the database. At last it creates a new Membership record consisting
	 * of a group id and member id (both pointing to the Member table).
	 * 
	 * Note that this method is used for both the member attribute (group members) as the memberOf attribute! In the first case the given memberDN parameter is used as the group
	 * that has the first parameter as member, in the second case the memberDN is treated as a member of the group given in the first parameter.
	 * 
	 * @param memberBeingProcessed the member that is currently being processed. 
	 * @param memberDN member that will either be added as group to the memberBeingProcessed or as member to the memberBeingProcessed, depending on the processMemberOf parameter.
	 * @param processMemberOf true in case the membership is based on the memberOf attribute. This means the parameter memberDN is actually the group containing the memberBeingProcessed.
	 * @param onlyScanGroups true in case the OU in the GUI has been configured to scan only groups.
	 */
	private void addMembership(Member memberBeingProcessed, String memberDN, boolean processMemberOf, boolean onlyScanGroups, StatusListener statusListener) throws DBException {
		if (logger.isTraceEnabled()) {
			logger.trace(MessageFormat.format("Adding group membership, group {0}, member {1}, rdn {2} and domain {3}", memberBeingProcessed, memberDN));
		}
		
		String domain = getDomain(memberDN);
		String rdn = getRdnFromDn(memberDN);
		int domainId = DomainCache.getDomainId(domain);
		int rdnId = RdnCache.getRdn(rdn).getId();
		
		try {
			String memberCN = getCnFromDn(memberDN);
			Member member = createOrUpdateMember(null, memberDN, memberCN, rdnId, domainId, null, onlyScanGroups);
			
			// The member id may be null in case the license limit has been reached and the member is not saved in the database
			if (member.getId() != null) {
				Membership membership;
				if (processMemberOf) {
					membership = new Membership(member.getId(), memberBeingProcessed.getId());
				} else {
					membership = new Membership(memberBeingProcessed.getId(), member.getId());
				}
				
				// this membership might already exists because we both scan the member attribute as the memberOf attribute!
				if (!MembershipCache.membershipExists(membership)) {
					membership.save();
					MembershipCache.cacheMembership(membership);
				}				
			}
		} catch (LDAPException e) {
			logger.error(MessageFormat.format("Error while processing membership of {0} in group {1}", memberDN, memberBeingProcessed.getName(false)), e);
		}
	}

	/**
	 * Determines the LDAP implementation using several attributes from the RootDSE.
	 * 
	 * @return a specific implementation using enum LdapServerType.
	 * @throws LDAPException
	 */
	private LdapServerType getServerType() throws LDAPException {
		if (serverType == null) {
			if (ldapConnection.getRootDSE().getAttributeValue("forestFunctionality") != null) {
				serverType = LdapServerType.ACTIVE_DIRECTORY;
			} else if ("Novell, Inc.".equals(ldapConnection.getRootDSE().getAttributeValue("vendorName"))) {
				serverType = LdapServerType.NOVELL;
			} else if ("Netscape Communications Corp.".equals(ldapConnection.getRootDSE().getAttributeValue("vendorName"))) {
				serverType = LdapServerType.NETSCAPE;
			} else if ("Sun Microsystems\\, Inc.".equals(ldapConnection.getRootDSE().getAttributeValue("vendorName"))) {
				serverType = LdapServerType.SUN;
			} else if ("OID 10.1.4.3.0".equals(ldapConnection.getRootDSE().getAttributeValue("orcldirectoryversion"))) {
				serverType = LdapServerType.ORACLE_INTERNET_DIR;
			} else if ("OVD 11.1.1.6.0".equals(ldapConnection.getRootDSE().getAttributeValue("orcldirectoryversion"))) {
				serverType = LdapServerType.ORACLE_VIRTUAL_DIR;
			} else if ("Oracle Corporation".equals(ldapConnection.getRootDSE().getAttributeValue("vendorName"))) {
				serverType = LdapServerType.ORACLE_UNIFIED_DIR;
			} else if ("[International Business Machines (IBM)]".equals(ldapConnection.getRootDSE().getAttributeValue("vendorName"))) {
				serverType = LdapServerType.IBM;
			} else if (Arrays.asList(ldapConnection.getRootDSE().getObjectClassValues()).contains("OpenLDAProotDSE")) {
				serverType = LdapServerType.OPEN_LDAP;
			}

			logger.info("Determined LDAP server type: " + serverType);
		}

		return serverType;
	}
	
	/**
	 * Returns a list of domain controllers for the given domain name
	 * 
	 * @param domain
	 * @return
	 * @throws NamingException
	 */
	public static List<String> findLDAPServersInWindowsDomain(String domain) {
        List<String> servers = new ArrayList<String>();
        Hashtable<String, String> env = new Hashtable<String, String>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.dns.DnsContextFactory");
        env.put("java.naming.provider.url", "dns:");
        
        DirContext ctx = null;
		try {
			ctx = new InitialDirContext(env);
	        Attributes attributes = ctx.getAttributes("_ldap._tcp." + domain, new String[] { "SRV" }); // that's how Windows domain controllers are registered in DNS
	        javax.naming.directory.Attribute a = attributes.get("SRV");
	        for (int i = 0; i < a.size(); i++) {
	            String srvRecord = a.get(i).toString();
	            // each SRV record is in the format "0 100 389 dc1.company.com."
	            // priority weight port server (space separated)
	            servers.add(srvRecord.split(" ")[3]);
	        }
	        
		} catch (NamingException e) {
			//ignore, bad luck
		} finally {
			try {
				ctx.close();
			} catch (NamingException e) {}
		}
		
        return servers;
    }
	
	public static List<String> findLDAPServersInWindowsDomain() throws NamingException {
		return findLDAPServersInWindowsDomain(getDnsForestName());
	}
	
	public static String convertSidToStringSid(byte[] sid) {
	    int offset, size;
	    
	    // sid[0] is the Revision, we allow only version 1, because it's the only that exists right now.
	    if (sid[0] != 1) {
	        throw new IllegalArgumentException("SID revision must be 1");
	    }
	 
	    StringBuilder stringSidBuilder = new StringBuilder("S-1-");
	 
	    // The next byte specifies the numbers of sub authorities (number of dashes minus two)
	    int subAuthorityCount = sid[1] & 0xFF;
	 
	    // IdentifierAuthority (6 bytes starting from the second) (big endian)
	    long identifierAuthority = 0;
	    offset = 2;
	    size = 6;
	    for (int i = 0; i < size; i++) {
	        identifierAuthority |= (long) (sid[offset + i] & 0xFF) << (8 * (size - 1 - i));
	        // The & 0xFF is necessary because byte is signed in Java
	    }
	    
	    if (identifierAuthority < Math.pow(2, 32)) {
	        stringSidBuilder.append(Long.toString(identifierAuthority));
	    } else {
	        stringSidBuilder.append("0x").append(Long.toHexString(identifierAuthority).toUpperCase());
	    }
	 
	    // Iterate all the SubAuthority (little-endian)
	    offset = 8;
	    size = 4; // 32-bits (4 bytes) for each SubAuthority
	    for (int i = 0; i < subAuthorityCount; i++, offset += size) {
	        long subAuthority = 0;
	        for (int j = 0; j < size; j++) {
	            subAuthority |= (long) (sid[offset + j] & 0xFF) << (8 * j);
	            // The & 0xFF is necessary because byte is signed in Java
	        }
	        stringSidBuilder.append("-").append(subAuthority);
	    }
	 
	    return stringSidBuilder.toString();
	}
	
	public static void main(String[] args) {
		/*
		LdapConnectorImpl conn = new LdapConnectorImpl();
					
		LdapConnectionInfo connection = new LdapConnectionInfo();
		connection.setHostName("192.168.178.50");
		connection.setHostPort(389);
		connection.setUsername("testadmin@testdomain");
		connection.setPassword("T@st12345");

		//CN=sa_ibm_intranet,OU=Service Accounts,OU=Administrators,OU=Users,OU=MME,DC=mme,DC=intra
		//P59aO2Y81420J6w
		//10.176.47.15

		LdapOUInfo ouInfo = new LdapOUInfo();
		ouInfo.setConnection(connection);
		ouInfo.setBaseOU("DC=testdomain,DC=local");
		ouInfo.setSearchObjectType(SearchObjectType.USERS_GROUPS);
		ouInfo.setSearchScope(SearchScope.CONTAINER_AND_SUBS);
		
		List<LdapOUInfo> ouList = new ArrayList<LdapOUInfo>();
		ouList.add(ouInfo);
		
		conn.scanOUs(ouList, null);*/
		
	
//		IADs rootDSE = COM4J.getObject(IADs.class, "LDAP://RootDSE", null);
//		defaultNamingContext = (String) rootDSE.get("defaultNamingContext");
//
//		con = ClassFactory.createConnection();
//		con.provider("ADsDSOObject");
//		con.open("Active Directory Provider", ""/* default */, ""/* default */, -1/* default */);
//
//		_Command cmd = ClassFactory.createCommand();
//		cmd.activeConnection(con);
//		cmd.commandText(";(sAMAccountName=" + username
//				+ ");distinguishedName;subTree");
//		_Recordset rs = cmd.execute(null, Variant.MISSING, -1/* default */);
//		if (rs.eof())
//			throw new UsernameNotFoundException("No such user: " + username);
//		String dn = rs.fields().item("distinguishedName").value().toString();
//		// now we got the DN of the user
//		IADsOpenDSObject dso = COM4J.getObject(IADsOpenDSObject.class, "LDAP:",
//				null);
//		// to do bind with DN as the user name, the flag must be 0
//		IADsUser usr;
//		try {
//			usr = dso.openDSObject("LDAP://" + dn, dn, password, 0)
//					.queryInterface(IADsUser.class);
//		} catch (ComException e) {
//			throw new BadCredentialsException("Incorrect password for "
//					+ username);
//		}
//		for (Com4jObject g : usr.groups()) {
//			IADsGroup grp = g.queryInterface(IADsGroup.class);
//			System.out.println("Belong to group " + grp.name());
//		}
		
		//WindowsAuthProviderImpl authProvider = new WindowsAuthProviderImpl();
		//System.out.println(authProvider.getCurrentComputer().getComputerName());
		//IWindowsDomain[] domains = authProvider.getDomains();
		
		//for (IWindowsDomain domain : domains) {
		//	System.out.println(domain.getFqn());
		//}
	}
	
	public static String getDomainControllerName() {
		String domainController = Netapi32Util.getDCName();
		if (domainController.startsWith("\\\\")) {
			domainController = domainController.substring(2);
		}
		return domainController;
	}
	
	public static String getDomainName() {
		return Netapi32Util.getDomainName(null);
	}
	
	public static String getCurrentUsername() {
		return Advapi32Util.getUserName();
	}
	
	public static String getDnsForestName() {
		return Netapi32Util.getDC().dnsForestName;
	}
	
	/**
	 * Adds a member to a LDAP group
	 * 
	 * @param groupDN DN of the group, like cn=Admin,ou=Groups,dc=example,dc=com
	 * @param memberDN member to add, like test,o=testOrg,dc=example,dc=com
	 * @return the LDAPResult of the query
	 * @throws LDAPException
	 */
	public LDAPResult addUserToGroup(String groupDN, String memberDN) throws LDAPException {
		Modification mod = new Modification(ModificationType.ADD, "member", memberDN); 
		ModifyRequest modifyRequest = new ModifyRequest( groupDN, mod);
		LDAPResult modifyResult = ldapConnection.modify(modifyRequest);
		return modifyResult;
	}
}
