/*
 * Created 2013, Alex
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (GPLv2 only)
 * or the terms of the GNU Lesser General Public License (LGPLv2.1 only)
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses>.
 */
package com.perdemia.permissionanalyzer.scanner.ldap;

import java.io.IOException;

import jcifs.ntlmssp.NtlmFlags;
import jcifs.ntlmssp.Type1Message;
import jcifs.ntlmssp.Type2Message;
import jcifs.ntlmssp.Type3Message;

import com.unboundid.asn1.ASN1OctetString;
import com.unboundid.ldap.sdk.BindResult;
import com.unboundid.ldap.sdk.Control;
import com.unboundid.ldap.sdk.GenericSASLBindRequest;
import com.unboundid.ldap.sdk.LDAPConnection;
import com.unboundid.ldap.sdk.LDAPException;
import com.unboundid.ldap.sdk.ResultCode;
import com.unboundid.ldap.sdk.SASLBindInProgressException;
import com.unboundid.ldap.sdk.SASLBindRequest;
import com.unboundid.util.Validator;

/** This class implements NTLM authentication for LDAP for the UnboundID SDK<br>
 * Use it for Microsoft AD and AD LDS Directories<br>
 * Currently only NTLM authentication is supported, but no NTLM protocol signing/encryption of data traffic<br>
 * - A server might deny NTLM authentication, if unencrypted LDAP connections have been disabled on the server<br>
 * - Also security restricted operations like password changes could be prohibited by the server when running an unencrypted LDAP connection
 * <br><br>
 * It is based on information from http://davenport.sourceforge.net/ntlm.html and on using the http://jcifs.samba.org Library
 *<br><br>
 * Note, that a failed NTLM bind will return more that just LDAP error codes, there are additional Microsoft specific error codes:<br>
 * - errorMessage ...<br> 
 * - LdapErr ....<br>
 * - data ...<br>
 * Search the internet for these error codes to get more fine grained details, of what went wrong during authentication
 * */
public class NtlmBindRequest extends SASLBindRequest
{
	/** The serial version UID for this serializable class. */
	private static final long serialVersionUID = -3183282879133397511L;

	/** NTLM authentication requires the LDAP SASL mechanism GSS-SPNEGO */
	private static final String mechanism = "GSS-SPNEGO";
	
	/** The SASL credentials that should be used for the bind request */
	private final String password;

	/* The bind DN to use for the bind request */
	private final String username;

	/**
	 * Creates a new NTLMv2 bind request with the provided information.
	 * @param username
	 *            The NTLM username that should be used for the request. It may not be
	 *            {@code null}<br>
	 *            Syntax options:<br>
	 *              use 'domain&#92;accountname' for domains and server local accounts (a.k.a. as domainname&#92;samaccountname),<br>
	 *              for server local account: istate the hostname instead of the domain name, findout with 'ipconfig /all'<br>
	 *              servername and domainname are both treated case ignore<br>
	 *              or use 'accountname@domain.company.org' for domain accounts (a.k.a the 'userprincipalname')<br>
	 *              Remember to escape the `\` in Java, when using the 'server&#92;username' syntax
	 * @param password
	 *            The password that should be used for the NTLM bind. It may not be {@code null}
	 * <br><br>
	 * This class should be able to connect to all Active Directory 2000/2003/2008/2012, ADAM 2003 and LDS 2008/2012 (and maybe future versions, if NTLM2 supported)
	 * The class does not rely on Windows clients, it but can be run on any OS
	 * <br>
	 * Currently this class can only authenticate with NTLM, but does not activate NTLM encryption/NTLM signing for the LDAP connection
	 * <br>
	 * Password changes over this connections may fail, if SSL/TLS is not used, depending on the security configuration of the
	 * AD/AD LDS server
	 * 
	 * The class relys on JCIF library of samba.org to handle the NTML handshake
	 */
	public NtlmBindRequest(String username, String password)
	{
		this(username, password, (Control[])null);
	}
	
	
	/**
	 * Creates a new NTLMv2 bind request with the provided information.
	 * @param username
	 *            The NTLM username that should be used for the request. It may not be
	 *            {@code null}<br>
	 *            Syntax options:<br>
	 *              use 'domain&#92;accountname' for domains and server local accounts (a.k.a. as domainname&#92;samaccountname),<br>
	 *              for server local account: istate the hostname instead of the domain name, findout with 'ipconfig /all'<br>
	 *              servername and domainname are both treated case ignore<br>
	 *              or use 'accountname@domain.company.org' for domain accounts (a.k.a the 'userprincipalname')<br>
	 *              Remember to escape the `\` in Java, when using the 'server&#92;username' syntax
	 * @param password
	 *            The password that should be used for the NTLM bind. It may not be {@code null}
	 * @param controls
	 *            The set of controls to include in the bind request. It
	 *            may be {@code null} or empty if no request controls are needed.
	 * <br><br>
	 * This class should be able to connect to all Active Directory 2000/2003/2008/2012, ADAM 2003 and LDS 2008/2012 (and maybe future versions, if NTLM2 supported)
	 * The class does not rely on Windows clients, it but can be run on any OS
	 * <br>
	 * Currently this class can only authenticate with NTLM, but does not activate NTLM encryption/NTLM signing for the LDAP connection
	 * <br>
	 * Password changes over this connections may fail, if SSL/TLS is not used, depending on the security configuration of the
	 * AD/AD LDS server
	 * 
	 * The class relys on JCIF library of samba.org to handle the NTML handshake
	 */
	public NtlmBindRequest(String username, String password, Control... controls)
	{
		super(controls);

		Validator.ensureNotNull(username);
		Validator.ensureNotNull(password);

		this.username = username;
		this.password = password;
	}

	/** Retrieves the username for this bind request */
	public String getUsername()
	{
		return username;
	}

	/** Retrieve the LDAP SASL mechanism name */
	@Override()
	public String getSASLMechanismName()
	{
		return mechanism;
	}

	/** {@inheritDoc} */
	@Override()
	protected BindResult process(final LDAPConnection connection, final int depth) throws LDAPException
	{
		// NTLM Auth uses SASL mechanism 'GSS-SPNEGO'. The original SPNEGO design is actually thought to negotiate the best possible protocol
		// between server and client, that are: Kerberos, NTLMv2, NTMLv1
		
		// since Kerberos needs OS-interaction and NTLMv1 seems not used in Microsoft LDAP protocol, we basically only use the remaining NTMLv2 process
		//
		// use separate 'GSS-API' (GSSAPIBindRequest) mechanism, if requiring a a single-sign-on Windows Kerberos authentication
		// this implementation does not rely on OS on client side for NTLM auth
		
		// NTLM Auth consists of 2 steps:
		// 1. client sends a Type1Message to the server, which is basically a request for a challenge
		//    basically the Type1Message request consists of request flags
		//    server will respond with a Type2Message (the challenge code) and a response code.
		//    THe API will treat that the response code as a SASLBindInProgressException exception,
		//    which is basically a success message for step 1, asking to continue with step 2
		// 2. client then processes username, password and the challenge to a type3Message (challenge response) and sends that to server
		//    server will answer with LDAP code 0 if that is successful
		
		try
		{
			// create and send the type1message
			int flags = NtlmFlags.NTLMSSP_NEGOTIATE_UNICODE;
			Type1Message ntlmType1Message = new Type1Message( flags, null, null);
			ASN1OctetString asn1ofType1Message=new ASN1OctetString(ntlmType1Message.toByteArray());	
			GenericSASLBindRequest getChallenge = new GenericSASLBindRequest(null, mechanism, asn1ofType1Message, getControls());
			connection.bind(getChallenge);
		}
		catch (SASLBindInProgressException e)
		{
			// get the resulting type2message and decode it
			BindResult bindResult = e.getBindResult();
			Type2Message ntlmType2Message;
			try
			{
				ntlmType2Message = new Type2Message(bindResult.getServerSASLCredentials().getValue());
			} catch (IOException e1)
			{
				e1.printStackTrace();
				throw new LDAPException(ResultCode.INVALID_CREDENTIALS, e1);	
			}

			// depending on username format 'server\name', 'domain\name' or 'username@domain', user and domain must be set accordingly
			String domain;
			String user;
			if (username.indexOf("\\")>=-1) // if '\' is contained in username, we need to split it in 2 variables
			{
				domain=username.split("\\\\")[0]; // we actually split at '\', we need to escape it twice: once for java, once for Regex
				user=username.split("\\\\")[1];   // we actually split at '\', we need to escape it twice: once for java, once for Regex
			}
			else // we expect an "@", then domain is then not stated in a separate variable, but stays in username string
			{
			    domain="";
				user="username";
			}

			// now construct the challenge response as a type3message and send it
			// type3 requests need no flags
			int flags=0;
			Type3Message ntlmType3Message = new Type3Message(ntlmType2Message, password, domain, user, null, flags);
			ASN1OctetString asn3ofType3Message = new ASN1OctetString(ntlmType3Message.toByteArray());
			GenericSASLBindRequest getChallenge = new GenericSASLBindRequest(null, mechanism, asn3ofType3Message, getControls());
			return connection.bind(getChallenge);
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override()
	public NtlmBindRequest duplicate()
	{
		return duplicate(getControls());
	}

	/** {@inheritDoc} */
	@Override()
	public NtlmBindRequest duplicate(final Control[] controls)
	{
		return new NtlmBindRequest(username, password, controls);
	}

	/** Returns a string representation of the object for debugging purpose */
	@Override()
	public void toString(final StringBuilder buffer)
	{
		buffer.append("NTLMLBindRequest(");

		buffer.append("username='");
		buffer.append(username);
		buffer.append("'");
		buffer.append(", password='");
		buffer.append(password);
		buffer.append("'");

		final Control[] controls = getControls();
		if (controls.length > 0)
		{
			buffer.append(", controls={");
			for (int i = 0; i < controls.length; i++)
			{
				if (i > 0)
				{
					buffer.append(", ");
				}

				buffer.append(controls[i]);
			}
			buffer.append('}');
		}

		buffer.append(')');
	}
}
